@@ -1,6 +1,6 @@
 /*
  * qsnprint.h
- * $Id: qsnprint.h 4767 2012-06-16 20:48:51Z sezero $
+ * $Id: qsnprint.h 5447 2015-09-28 00:00:58Z sezero $
  *
  * (v)snprintf wrappers
  * Copyright (C) 2007 O. Sezer <sezero@users.sourceforge.net>
@@ -24,10 +24,15 @@
 #ifndef __Q_SNPRINF_H
 #define __Q_SNPRINF_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 extern int q_snprintf (char *str, size_t size, const char *format, ...) __attribute__((__format__(__printf__,3,4)));
 extern int q_vsnprintf(char *str, size_t size, const char *format, va_list args)
 									__attribute__((__format__(__printf__,3,0)));
-
+#ifdef __cplusplus
+}
+#endif
 
 #endif	/* __Q_SNPRINF_H */
 
