@@ -1,6 +1,6 @@
 /*
  * screen.c -- master for refresh, status bar, console, chat, notify, etc
- * $Id: gl_screen.c 5156 2013-03-02 10:33:57Z sezero $
+ * $Id: gl_screen.c 5485 2015-10-25 15:55:15Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
@@ -827,7 +827,7 @@ static void SCR_ScreenShot_f (void)
 	char	pcxname[80];
 	char	checkname[MAX_OSPATH];
 	int	i, size, temp;
-	int	mark;
+	qboolean	freebuf = false;
 	byte	*buffer;
 
 	FS_MakePath_BUF (FS_USERDIR, NULL, checkname, sizeof(checkname), "shots");
@@ -849,8 +849,15 @@ static void SCR_ScreenShot_f (void)
 	}
 
 	size = glwidth * glheight * 3 + 18;
-	mark = Hunk_LowMark();
-	buffer = (byte *) Hunk_AllocName(size, "buffer_sshot");
+	buffer = (byte *) Hunk_TempAlloc(size);
+	if (!buffer) {
+		buffer = (byte *) malloc(size);
+		if (!buffer) {
+			Con_Printf("%s: not enough memory\n", __thisfunc__);
+			return;
+		}
+		freebuf = true;
+	}
 	memset (buffer, 0, 18);
 	buffer[2] = 2;		// uncompressed type
 	buffer[12] = glwidth & 255;
@@ -859,6 +866,7 @@ static void SCR_ScreenShot_f (void)
 	buffer[15] = glheight >> 8;
 	buffer[16] = 24;	// pixel size
 
+	glPixelStorei_fp (GL_PACK_ALIGNMENT, 1);/* for widths that aren't a multiple of 4 */
 	glReadPixels_fp (glx, gly, glwidth, glheight, GL_RGB, GL_UNSIGNED_BYTE, buffer+18);
 
 	// swap rgb to bgr
@@ -871,7 +879,7 @@ static void SCR_ScreenShot_f (void)
 
 	i = FS_WriteFile (pcxname, buffer, size);
 
-	Hunk_FreeToLowMark(mark);
+	if (freebuf) free(buffer);
 
 	if (i == 0)
 		Con_Printf ("Wrote %s\n", pcxname);
@@ -1224,7 +1232,7 @@ void SCR_UpdateScreen (void)
 	if (block_drawing)
 		return;
 
-//	vid.numpages = (gl_triplebuffer.integer) ? 3 : 2;
+//	vid.numpages = (gl_triplebuffer.integer)? 3 : 2;
 	scr_copytop = 0;
 	scr_copyeverything = 0;
 
@@ -1234,19 +1242,20 @@ void SCR_UpdateScreen (void)
 #else
 	if (scr_disabled_for_loading)
 	{
-		if (realtime - scr_disabled_time > 20)
-		{
+		if (realtime - scr_disabled_time > 25) {
 		/* this can happen with clients connected to servers
 		 * older than uHexen2-1.5.6 who don't issue an error
-		 * upon changelevel failures.
+		 * upon changelevel failures. Or, it could happen if
+		 * loading is taking a really long time.
 		 */
 			scr_disabled_for_loading = false;
 			total_loading_size = 0;
 			loading_stage = 0;
-			Con_Printf ("load failed.\n");
+			Con_Printf ("load timeout.\n");
 		}
-		else
+		else {
 			return;
+		}
 	}
 
 	if (cls.state == ca_dedicated)
