@@ -1,12 +1,12 @@
 /*
- * gl_vidamiga.c -- GL vid component for AROS and MorphOS
- * $Id: gl_vidamiga.c 5173 2013-03-10 14:00:27Z sezero $
+ * gl_vidamiga.c -- GL vid component for AmigaOS & variants.
+ * $Id: gl_vidamiga.c 5702 2016-08-01 06:32:36Z bszili $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
  * Copyright (C) 2004-2005  Steven Atkinson <stevenaaus@yahoo.com>
- * Copyright (C) 2005-2012  O.Sezer <sezero@users.sourceforge.net>
- * Copyright (C) 2012 Szilrd Bir <col.lawrence@gmail.com>
+ * Copyright (C) 2005-2016  O.Sezer <sezero@users.sourceforge.net>
+ * Copyright (C) 2012-2016  Szilrd Bir <col.lawrence@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,9 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef GL_DLSYM
+#error GL_DLSYM not supported.
+#endif
 #define	__GL_FUNC_EXTERN
 
 #include <cybergraphx/cybergraphics.h>
@@ -32,21 +35,42 @@
 #include <proto/graphics.h>
 #include <proto/cybergraphics.h>
 
-//#include <GL/gl.h>
-#ifdef __AROS__
+#if defined(__AROS__) && defined(AROS_USE_GLA)
+#include <GL/gla.h>
+#elif defined(__AROS__)
 #include <GL/arosmesa.h>
 #elif defined __MORPHOS__
 #include <intuition/intuitionbase.h>
-#include <proto/tinygl.h>
+#elif defined __amigaos4__
+#error AmigaOS4 support not yet available
+#elif defined(__AMIGA__) && defined(REFGL_MINIGL)
+#include <mgl/gl.h>
+#elif defined(__AMIGA__) && defined(REFGL_AMESA)
+#include <GL/Amigamesa.h>
+#else
+#error Unknown / Unsupported AmigaOS variant.
+#endif
+
+#if defined(__AROS__) && defined(AROS_USE_GLA)
+typedef GLAProc AMIGAGL_Proc;
+#elif defined(__AROS__)
+typedef AROSMesaProc AMIGAGL_Proc;
+#else
+typedef void (*AMIGAGL_Proc)();
 #endif
 
 #include "quakedef.h"
 #include "cfgfile.h"
 #include "bgmusic.h"
 #include "cdaudio.h"
-#include <unistd.h>
 #include "filenames.h"
 
+#if !(defined(__AROS__) || defined(__MORPHOS__))
+#if !defined(IPTR) /* SDI headers may define it */
+typedef ULONG IPTR;
+#define IPTR IPTR
+#endif
+#endif
 
 #define WARP_WIDTH		320
 #define WARP_HEIGHT		200
@@ -117,18 +141,26 @@ typedef struct {
 		depth,
 		stencil;
 } attributes_t;
-
 static attributes_t	vid_attribs;
-struct Window *window = NULL; // used by in_amiga.c
+
+static void VID_KillContext (void);
+struct Window *window = NULL; /* used by in_amiga.c */
 static struct Screen *screen = NULL;
-#ifdef __AROS__
+#if defined(__AROS__) && defined(AROS_USE_GLA)
+static GLAContext context = NULL;
+#elif defined(__AROS__)
 static AROSMesaContext context = NULL;
 #elif defined __MORPHOS__
 GLContext *__tglContext = NULL;
 static qboolean contextinit = false;
+#elif defined(__AMIGA__) && defined(REFGL_MINIGL)
+static int lockmode = MGL_LOCK_MANUAL;
+#elif defined(__AMIGA__) && defined(REFGL_AMESA)
+struct Library *CyberGfxBase = NULL;
+static AmigaMesaContext context = NULL;
 #endif
 static qboolean	vid_menu_fs;
-static qboolean	fs_toggle_works = true;
+static qboolean	fs_toggle_works = false;
 
 // vars for vid state
 viddef_t	vid;			// global video state
@@ -148,10 +180,15 @@ static cvar_t	vid_mode = {"vid_mode", "0", CVAR_NONE};
 static cvar_t	vid_config_consize = {"vid_config_consize", "640", CVAR_ARCHIVE};
 static cvar_t	vid_config_glx = {"vid_config_glx", "640", CVAR_ARCHIVE};
 static cvar_t	vid_config_gly = {"vid_config_gly", "480", CVAR_ARCHIVE};
-static cvar_t	vid_config_swx = {"vid_config_swx", "320", CVAR_ARCHIVE};
+#ifdef PLATFORM_AMIGAOS3
+static cvar_t	vid_config_fscr= {"vid_config_fscr", "1", CVAR_ARCHIVE};
+#else
 static cvar_t	vid_config_fscr= {"vid_config_fscr", "0", CVAR_ARCHIVE};
+#endif
 // cvars for compatibility with the software version
+static cvar_t	vid_config_swx = {"vid_config_swx", "320", CVAR_ARCHIVE};
 static cvar_t	vid_config_swy = {"vid_config_swy", "240", CVAR_ARCHIVE};
+static cvar_t	vid_config_mon = {"vid_config_mon", "0", CVAR_ARCHIVE};
 
 byte		globalcolormap[VID_GRADES*256];
 float		RTint[256], GTint[256], BTint[256];
@@ -170,9 +207,9 @@ static const char	*gl_extensions;
 qboolean	is_3dfx = false;
 
 GLint		gl_max_size = 256;
-static qboolean	gl_has_NPOT = false;
+static qboolean	have_NPOT = false;
 qboolean	gl_tex_NPOT = false;
-cvar_t		gl_texture_NPOT = {"gl_texture_NPOT", "0", CVAR_ARCHIVE};
+static cvar_t	gl_texture_NPOT = {"gl_texture_NPOT", "0", CVAR_ARCHIVE};
 GLfloat		gl_max_anisotropy;
 float		gldepthmin, gldepthmax;
 
@@ -182,11 +219,12 @@ qboolean	is8bit = false;
 static cvar_t	vid_config_gl8bit = {"vid_config_gl8bit", "0", CVAR_ARCHIVE};
 
 static qboolean	gammaworks = false;	// whether hw-gamma works
-qboolean	gl_dogamma = false;	// none of the above two, use gl tricks
 
 // multitexturing
 qboolean	gl_mtexable = false;
 static GLint	num_tmus = 1;
+static qboolean	have_mtex = false;
+static cvar_t	gl_multitexture = {"gl_multitexture", "0", CVAR_ARCHIVE};
 
 // stencil buffer
 qboolean	have_stencil = false;
@@ -274,7 +312,7 @@ static void VID_ConWidth (int modenum)
 	}
 
 	w = vid_config_consize.integer;
-	w &= 0xfff8; /* make it a multiple of 8 */
+	w &= ~7; /* make it a multiple of eight */
 	if (w < MIN_WIDTH)
 		w = MIN_WIDTH;
 	else if (w > modelist[modenum].width)
@@ -303,14 +341,14 @@ void VID_ChangeConsize (int dir)
 	{
 	case -1: /* smaller text */
 		w = ((float)vid.conwidth/(float)vid.width + 0.05f) * vid.width; /* use 0.10f increment ?? */
-		w &= 0xfff8; /* make it a multiple of 8 */
+		w &= ~7; /* make it a multiple of eight */
 		if (w > modelist[vid_modenum].width)
 			w = modelist[vid_modenum].width;
 		break;
 
 	case 1: /* bigger text */
 		w = ((float)vid.conwidth/(float)vid.width - 0.05f) * vid.width;
-		w &= 0xfff8; /* make it a multiple of 8 */
+		w &= ~7; /* make it a multiple of eight */
 		if (w < MIN_WIDTH)
 			w = MIN_WIDTH;
 		break;
@@ -337,18 +375,25 @@ float VID_ReportConsize(void)
 }
 
 
-static int VID_SetMode (int modenum)
+static qboolean VID_SetMode (int modenum)
 {
 	ULONG flags;
 
 	in_mode_set = true;
 
-	VID_Shutdown();
+	VID_KillContext();
 
 	flags = WFLG_ACTIVATE | WFLG_RMBTRAP;
 
 	Con_SafePrintf ("Requested mode %d: %dx%dx%d\n", modenum, modelist[modenum].width, modelist[modenum].height, bpp);
 
+#if defined(REFGL_MINIGL)
+	/* Hyperion's MiniGL 1.2 handles window creation within itself. */
+	if (vid_config_fscr.integer)
+		mglChooseWindowMode(GL_FALSE);
+	else	mglChooseWindowMode(GL_TRUE );
+
+#else /* ! REFGL_MINIGL */
 	if (vid_config_fscr.integer)
 	{
 	    ULONG ModeID;
@@ -359,17 +404,25 @@ static int VID_SetMode (int modenum)
 			CYBRBIDTG_NominalHeight, modelist[modenum].height,
 			TAG_DONE);
 
+		#ifndef __MORPHOS__
+		screen = OpenScreenTags(0,
+			ModeID != INVALID_ID ? SA_DisplayID : TAG_IGNORE, ModeID,
+			SA_Width, modelist[modenum].width,
+			SA_Height, modelist[modenum].height,
+			SA_Depth, bpp,
+			SA_Quiet, TRUE,
+			TAG_DONE);
+		#else
 		screen = OpenScreenTags(0,
 			ModeID != INVALID_ID ? SA_DisplayID : TAG_IGNORE, ModeID,
 			SA_Width, modelist[modenum].width,
 			SA_Height, modelist[modenum].height,
 			SA_Depth, bpp,
 			SA_Quiet, TRUE,
-#ifdef __MORPHOS__
 			SA_GammaControl, TRUE,
 			SA_3DSupport, TRUE,
-#endif
 			TAG_DONE);
+		#endif
 	}
 
 	if (screen)
@@ -385,19 +438,36 @@ static int VID_SetMode (int modenum)
 	window = OpenWindowTags(0,
 		WA_InnerWidth, modelist[modenum].width,
 		WA_InnerHeight, modelist[modenum].height,
-		WA_Title, WM_TITLEBAR_TEXT,
+		WA_Title, (IPTR)WM_TITLEBAR_TEXT,
 		WA_Flags, flags,
-		screen ? WA_CustomScreen : TAG_IGNORE, screen,
+		screen ? WA_CustomScreen : TAG_IGNORE, (IPTR)screen,
 		WA_IDCMP, IDCMP_CLOSEWINDOW | IDCMP_ACTIVEWINDOW | IDCMP_INACTIVEWINDOW,
 		TAG_DONE);
 
-	if (window)
-	{
-		WRWidth = vid.width = vid.conwidth = modelist[modenum].width;
-		WRHeight = vid.height = vid.conheight = modelist[modenum].height;
+	if (!window) goto fail;
+#endif /* ! REFGL_MINIGL */
 
-#ifdef __AROS__
-		context = AROSMesaCreateContextTags(
+	WRWidth = vid.width = vid.conwidth = modelist[modenum].width;
+	WRHeight = vid.height = vid.conheight = modelist[modenum].height;
+
+#if defined(__AROS__) && defined(AROS_USE_GLA)
+	context = glACreateContextTags(
+			GLA_Window, window,
+			GLA_Left, screen ? 0 : window->BorderLeft,
+			GLA_Top, screen ? 0 : window->BorderTop,
+			GLA_Width, vid.width,
+			GLA_Height, vid.height,
+			screen ? GLA_Screen : TAG_IGNORE, screen,
+			GLA_DoubleBuf, GL_TRUE,
+			GLA_RGBMode, GL_TRUE,
+			GLA_NoAccum, GL_TRUE,
+			TAG_DONE);
+
+	if (!context) goto fail;
+	glAMakeCurrent(context);
+
+#elif defined(__AROS__)
+	context = AROSMesaCreateContextTags(
 			AMA_Window, window,
 			AMA_Left, screen ? 0 : window->BorderLeft,
 			AMA_Top, screen ? 0 : window->BorderTop,
@@ -409,57 +479,66 @@ static int VID_SetMode (int modenum)
 			AMA_NoAccum, GL_TRUE,
 			TAG_DONE);
 
-		if (context)
-		{
-			AROSMesaMakeCurrent(context);
+	if (!context) goto fail;
+	AROSMesaMakeCurrent(context);
+
 #elif defined __MORPHOS__
-		__tglContext = GLInit();
-		if (__tglContext)
-		{
-			if (screen && !(TinyGLBase->lib_Version == 0 && TinyGLBase->lib_Revision < 4))
-				contextinit = glAInitializeContextScreen(screen);
-			else
-				contextinit = glAInitializeContextWindowed(window);
-
-			if (contextinit)
-#endif
-			{
-				vid.height = vid.conheight = modelist[modenum].height;
-				vid.rowbytes = vid.conrowbytes = vid.width = vid.conwidth = modelist[modenum].width;
-
-				// set vid_modenum properly and adjust other vars
-				vid_modenum = modenum;
-				modestate = (screen) ? MS_FULLDIB : MS_WINDOWED;
-				Cvar_SetValueQuick (&vid_config_glx, modelist[vid_modenum].width);
-				Cvar_SetValueQuick (&vid_config_gly, modelist[vid_modenum].height);
-
-				// setup the effective console width
-				VID_ConWidth(modenum);
-
-				Con_SafePrintf ("Video Mode Set : %dx%dx%d\n", modelist[modenum].width, modelist[modenum].height, bpp);
-
-				// collect the actual attributes
-				memset (&vid_attribs, 0, sizeof(attributes_t));
-				glGetIntegerv_fp(GL_RED_BITS, &vid_attribs.red);
-				glGetIntegerv_fp(GL_GREEN_BITS, &vid_attribs.green);
-				glGetIntegerv_fp(GL_BLUE_BITS, &vid_attribs.blue);
-				glGetIntegerv_fp(GL_ALPHA_BITS, &vid_attribs.alpha);
-				glGetIntegerv_fp(GL_DEPTH_BITS, &vid_attribs.depth);
-				glGetIntegerv_fp(GL_STENCIL_BITS, &vid_attribs.stencil);
-
-				Con_SafePrintf ("vid_info: red: %d, green: %d, blue: %d, alpha: %d, depth: %d\n",
-						vid_attribs.red, vid_attribs.green, vid_attribs.blue, vid_attribs.alpha, vid_attribs.depth);
-
-				//IN_HideMouse ();
-
-				in_mode_set = false;
-
-				return true;
-			}
-		}
+	__tglContext = GLInit();
+	if (!__tglContext) goto fail;
+	if (screen && !(TinyGLBase->lib_Version == 0 && TinyGLBase->lib_Revision < 4)) {
+		if (!(contextinit = glAInitializeContextScreen(screen)))
+			goto fail;
+	} else {
+		if (!(contextinit = glAInitializeContextWindowed(window)))
+			goto fail;
 	}
 
-	VID_Shutdown();
+#elif defined(__AMIGA__) && defined(REFGL_MINIGL)
+	if (!mglCreateContext(0,0,vid.width,vid.height))
+		goto fail;
+	mglLockMode(lockmode);
+	window = MGLGetWindowHandle(mini_CurrentContext);
+	ModifyIDCMP(window, IDCMP_CLOSEWINDOW | IDCMP_ACTIVEWINDOW | IDCMP_INACTIVEWINDOW);
+
+#elif defined(__AMIGA__) && defined(REFGL_AMESA)
+	context = AmigaMesaCreateContextTags(
+			AMA_Window, window,
+			AMA_Left, screen ? 0 : window->BorderLeft,
+			AMA_Bottom, screen ? 0 : window->BorderBottom,
+			AMA_Width, vid.width,
+			AMA_Height, vid.height,
+			screen ? AMA_Screen : TAG_IGNORE, screen,
+			AMA_DoubleBuf, GL_TRUE,
+			AMA_RGBMode, GL_TRUE,
+			AMA_NoAccum, GL_TRUE,
+			TAG_DONE);
+
+	if (!context) goto fail;
+	AmigaMesaMakeCurrent(context, context->buffer);
+#endif
+
+	vid.height = vid.conheight = modelist[modenum].height;
+	vid.rowbytes = vid.conrowbytes = vid.width = vid.conwidth = modelist[modenum].width;
+
+	// set vid_modenum properly and adjust other vars
+	vid_modenum = modenum;
+	modestate = (screen) ? MS_FULLDIB : MS_WINDOWED;
+	Cvar_SetValueQuick (&vid_config_glx, modelist[vid_modenum].width);
+	Cvar_SetValueQuick (&vid_config_gly, modelist[vid_modenum].height);
+
+	// setup the effective console width
+	VID_ConWidth(modenum);
+
+	Con_SafePrintf ("Video Mode Set : %dx%dx%d\n", modelist[modenum].width, modelist[modenum].height, bpp);
+
+	//IN_HideMouse ();
+
+	in_mode_set = false;
+
+	return true;
+
+fail:
+	VID_KillContext();
 
 	in_mode_set = false;
 
@@ -469,52 +548,174 @@ static int VID_SetMode (int modenum)
 
 //====================================
 
-#ifdef __MORPHOS__
-static void myglMultiTexCoord2fARB (GLenum unit, GLfloat s, GLfloat t)
+#if defined(__AROS__) && defined(AROS_USE_GLA)
+static AMIGAGL_Proc AMIGAGL_GetProcAddress (const char *s)
+{
+	return glAGetProcAddress((const GLubyte *) s);
+}
+
+#elif defined(__AROS__)
+static AMIGAGL_Proc AMIGAGL_GetProcAddress (const char *s)
+{
+	return AROSMesaGetProcAddress((const GLubyte *) s);
+}
+
+#elif defined(__MORPHOS__)
+static void MY_glMultiTexCoord2fARB (GLenum unit, GLfloat s, GLfloat t)
 {
 	GLMultiTexCoord2fARB(__tglContext, unit, s, t);
 }
 
-static void myglActiveTextureARB (GLenum unit)
+static void MY_glActiveTextureARB (GLenum unit)
 {
 	GLActiveTextureARB(__tglContext, unit);
 }
 
-static void *AROSMesaGetProcAddress (const char *s)
+static AMIGAGL_Proc AMIGAGL_GetProcAddress (const char *s)
 {
 	if (strcmp(s, "glMultiTexCoord2fARB") == 0)
-		return (void *)myglMultiTexCoord2fARB;
-	else if (strcmp(s, "glActiveTextureARB") == 0)
-		return (void *)myglActiveTextureARB;
+		return (AMIGAGL_Proc)MY_glMultiTexCoord2fARB;
+	if (strcmp(s, "glActiveTextureARB") == 0)
+		return (AMIGAGL_Proc)MY_glActiveTextureARB;
+
+	return NULL;
+}
+
+#elif defined(__AMIGA__) && defined(REFGL_MINIGL)
+static void MY_glMultiTexCoord2fARB (GLenum unit, GLfloat s, GLfloat t)
+{
+	GLMultiTexCoord2fARB(mini_CurrentContext, unit, s, t);
+}
+
+static void MY_glActiveTextureARB (GLenum unit)
+{
+	GLActiveTextureARB(mini_CurrentContext, unit);
+}
+
+static AMIGAGL_Proc AMIGAGL_GetProcAddress (const char *s)
+{
+	if (strcmp(s, "glMultiTexCoord2fARB") == 0)
+		return (AMIGAGL_Proc)MY_glMultiTexCoord2fARB;
+	if (strcmp(s, "glActiveTextureARB") == 0)
+		return (AMIGAGL_Proc)MY_glActiveTextureARB;
+
+	return NULL;
+}
+
+#elif defined(__AMIGA__) && defined(REFGL_AMESA)
+/* NOTE: StormMesa 3.0 has EXT_multitexture, not ARB_multitexture.. */
+/*                     250 == ( GL_TEXTURE0_ARB - GL_TEXTURE0_EXT ) */
+static void MY_glMultiTexCoord2fARB (GLenum unit, GLfloat s, GLfloat t)
+{
+	glMultiTexCoord2fEXT (unit - 250, s, t);
+}
+
+static void MY_glActiveTextureARB (GLenum unit)
+{
+	glSelectTextureEXT (unit - 250);
+}
+
+static void MY_glColorTableEXT (GLenum target, GLenum internalfmt, GLsizei width, GLenum format, GLenum type, const GLvoid *table)
+{
+	glColorTableEXT (target, internalfmt, width, format, type, table);
+}
+
+static void MY_glGetTexParameterfv (GLenum target, GLenum pname, GLfloat *params)
+{
+	glGetTexParameterfv (target, pname, params);
+}
+
+static AMIGAGL_Proc AMIGAGL_GetProcAddress (const char *s)
+{
+	if (strcmp(s, "glMultiTexCoord2fARB") == 0)
+		return (AMIGAGL_Proc)MY_glMultiTexCoord2fARB;
+	if (strcmp(s, "glActiveTextureARB") == 0)
+		return (AMIGAGL_Proc)MY_glActiveTextureARB;
+	if (strcmp(s, "glColorTableEXT") == 0)
+		return (AMIGAGL_Proc)MY_glColorTableEXT;
+	if (strcmp(s, "glGetTexParameterfv") == 0)
+		return (AMIGAGL_Proc)MY_glGetTexParameterfv;
 
 	return NULL;
 }
 #endif
 
-static void VID_Init8bitPalette (void)
+#if 0 /* No.. */
+static void CheckSetGlobalPalette (void)
 {
-	// Check for 8bit Extensions and initialize them.
-	int			i;
-	char	thePalette[256*3];
-	char	*oldPalette, *newPalette;
+	gl3DfxSetPaletteEXT_f gl3DfxSetPaletteEXT_fp;
 
-	have8bit = false;
-	is8bit = false;
-
-	if (GL_ParseExtensionList(gl_extensions, "GL_EXT_shared_texture_palette"))
+	if (GL_ParseExtensionList(gl_extensions, "3DFX_set_global_palette"))
 	{
-		glColorTableEXT_fp = (glColorTableEXT_f)AROSMesaGetProcAddress("glColorTableEXT");
-		if (glColorTableEXT_fp == NULL)
+		gl3DfxSetPaletteEXT_fp = (gl3DfxSetPaletteEXT_f) AMIGAGL_GetProcAddress("gl3DfxSetPaletteEXT");
+		if (!gl3DfxSetPaletteEXT_fp)
+			gl3DfxSetPaletteEXT_fp = (gl3DfxSetPaletteEXT_f) AMIGAGL_GetProcAddress("3DFX_set_global_palette");
+		if (!gl3DfxSetPaletteEXT_fp)
 			return;
-
-		have8bit = true;
-		if (!vid_config_gl8bit.integer)
+		Con_SafePrintf("Found 3DFX_set_global_palette\n");
+	}
+	else if (GL_ParseExtensionList(gl_extensions, "POWERVR_set_global_palette"))
+	{
+		gl3DfxSetPaletteEXT_fp = (gl3DfxSetPaletteEXT_f) AMIGAGL_GetProcAddress("glSetGlobalPalettePOWERVR");
+		if (!gl3DfxSetPaletteEXT_fp)
+			gl3DfxSetPaletteEXT_fp = (gl3DfxSetPaletteEXT_f) AMIGAGL_GetProcAddress("POWERVR_set_global_palette");
+		if (!gl3DfxSetPaletteEXT_fp)
 			return;
+		Con_SafePrintf("Found POWERVR_set_global_palette\n");
+	}
+	else {
+		return;
+	}
 
+	have8bit = true;
+	if (!vid_config_gl8bit.integer)
+		return;
+	else
+	{
+		int i;
+		GLubyte table[256][4];
+		char *oldpal;
+
+		is8bit = true;
+		oldpal = (char *) d_8to24table;
+		for (i = 0; i < 256; i++) {
+			table[i][2] = *oldpal++;
+			table[i][1] = *oldpal++;
+			table[i][0] = *oldpal++;
+			table[i][3] = 255;
+			oldpal++;
+		}
+		glEnable_fp (GL_SHARED_TEXTURE_PALETTE_EXT);
+		gl3DfxSetPaletteEXT_fp ((GLuint *)table);
+	}
+}
+#endif /* #if 0 */
+
+static void CheckSharedTexturePalette (void)
+{
+	glColorTableEXT_f glColorTableEXT_fp;
+
+	if (!GL_ParseExtensionList(gl_extensions, "GL_EXT_shared_texture_palette"))
+		return;
+
+	glColorTableEXT_fp = (glColorTableEXT_f) AMIGAGL_GetProcAddress("glColorTableEXT");
+	if (glColorTableEXT_fp == NULL)
+		return;
+
+	have8bit = true;
+	Con_SafePrintf("Found GL_EXT_shared_texture_palette\n");
+	if (!vid_config_gl8bit.integer)
+		return;
+	else
+	{
+		int i;
+		char thePalette[256*3];
+		char *oldPalette, *newPalette;
+
+		is8bit = true;
 		oldPalette = (char *) d_8to24table;
 		newPalette = thePalette;
-		for (i = 0; i < 256; i++)
-		{
+		for (i = 0; i < 256; i++) {
 			*newPalette++ = *oldPalette++;
 			*newPalette++ = *oldPalette++;
 			*newPalette++ = *oldPalette++;
@@ -524,30 +725,41 @@ static void VID_Init8bitPalette (void)
 		glEnable_fp (GL_SHARED_TEXTURE_PALETTE_EXT);
 		glColorTableEXT_fp (GL_SHARED_TEXTURE_PALETTE_EXT, GL_RGB, 256,
 					GL_RGB, GL_UNSIGNED_BYTE, (void *) thePalette);
-		is8bit = true;
-		Con_SafePrintf("8-bit palettized textures enabled\n");
 	}
 }
 
+static void VID_Init8bitPalette (void)
+{
+	have8bit = false;
+	is8bit = false;
+
+	/* Check for 8bit Extensions and initialize them */
+	CheckSharedTexturePalette();
+#if 0	/* No.. */
+	if (!have8bit)
+		CheckSetGlobalPalette();
+#endif
+
+	if (is8bit)
+		Con_SafePrintf("8-bit palettized textures enabled\n");
+}
+
 
 static void VID_InitGamma (void)
 {
-	if (!gl_dogamma)
-	{
-#ifdef __MORPHOS__
-		if (screen && (IntuitionBase->LibNode.lib_Version > 50 || 
-			(IntuitionBase->LibNode.lib_Version == 50 && IntuitionBase->LibNode.lib_Revision >= 74)))
-			gammaworks = true;
-		else
-#endif
-			gl_dogamma = true;
-	}
+	gammaworks = false;
 
-	if (gl_dogamma)
-		Con_SafePrintf("gamma not available, using gl tricks\n");
+#ifdef __MORPHOS__
+	if (screen && (IntuitionBase->LibNode.lib_Version > 50 ||
+			(IntuitionBase->LibNode.lib_Version == 50 && IntuitionBase->LibNode.lib_Revision >= 74)))
+		gammaworks = true;
+#endif
+
+	if (!gammaworks)
+		Con_SafePrintf("gamma adjustment not available\n");
 }
 
-void VID_ShiftPalette (unsigned char *palette)
+void VID_ShiftPalette (const unsigned char *palette)
 {
 #ifdef __MORPHOS__
 	if (screen)
@@ -564,32 +776,46 @@ void VID_ShiftPalette (unsigned char *palette)
 
 static void CheckMultiTextureExtensions (void)
 {
-	gl_mtexable = false;
+	gl_mtexable = have_mtex = false;
 
 	if (COM_CheckParm("-nomtex"))
 	{
 		Con_SafePrintf("Multitexture extensions disabled\n");
 	}
+	#if defined (REFGL_AMESA) /* StormMesa 3.0 has EXT_multitexture .. */
+	else if (GL_ParseExtensionList(gl_extensions, "GL_EXT_multitexture"))
+	#else
 	else if (GL_ParseExtensionList(gl_extensions, "GL_ARB_multitexture"))
+	#endif
 	{
 		Con_SafePrintf("ARB Multitexture extensions found\n");
 
+		#if defined (REFGL_AMESA)
+		glGetIntegerv_fp(GL_MAX_TEXTURES_EXT, &num_tmus);
+		#else
 		glGetIntegerv_fp(GL_MAX_TEXTURE_UNITS_ARB, &num_tmus);
+		#endif
 		if (num_tmus < 2)
 		{
-			Con_SafePrintf("not enough TMUs, ignoring multitexture\n");
+			Con_SafePrintf("ignoring multitexture (%i TMUs)\n", (int) num_tmus);
 			return;
 		}
 
-		glMultiTexCoord2fARB_fp = (glMultiTexCoord2fARB_f) AROSMesaGetProcAddress("glMultiTexCoord2fARB");
-		glActiveTextureARB_fp = (glActiveTextureARB_f) AROSMesaGetProcAddress("glActiveTextureARB");
-		if ((glMultiTexCoord2fARB_fp == NULL) ||
-		    (glActiveTextureARB_fp == NULL))
+		glMultiTexCoord2fARB_fp = (glMultiTexCoord2fARB_f) AMIGAGL_GetProcAddress("glMultiTexCoord2fARB");
+		glActiveTextureARB_fp = (glActiveTextureARB_f) AMIGAGL_GetProcAddress("glActiveTextureARB");
+		if (glMultiTexCoord2fARB_fp == NULL || glActiveTextureARB_fp == NULL)
 		{
 			Con_SafePrintf ("Couldn't link to multitexture functions\n");
 			return;
 		}
 
+		have_mtex = true;
+		if (!gl_multitexture.integer)
+		{
+			Con_SafePrintf("ignoring multitexture (cvar disabled)\n");
+			return;
+		}
+
 		Con_SafePrintf("Found %i TMUs support\n", (int) num_tmus);
 		gl_mtexable = true;
 		glDisable_fp(GL_TEXTURE_2D);
@@ -608,10 +834,11 @@ static void CheckAnisotropyExtensions (void)
 	Con_SafePrintf("Anisotropic filtering ");
 	if (GL_ParseExtensionList(gl_extensions, "GL_EXT_texture_filter_anisotropic"))
 	{
-		GLfloat test1, test2;
+		GLfloat test1 = 0, test2 = 0;
 		GLuint tex;
+		glGetTexParameterfv_f glGetTexParameterfv_fp;
 
-		glGetTexParameterfv_fp = (glGetTexParameterfv_f) AROSMesaGetProcAddress("glGetTexParameterfv");
+		glGetTexParameterfv_fp = (glGetTexParameterfv_f) AMIGAGL_GetProcAddress("glGetTexParameterfv");
 		if (glGetTexParameterfv_fp == NULL)
 		{
 			Con_SafePrintf("... can't check driver-lock status\n... ");
@@ -655,45 +882,43 @@ static void CheckNonPowerOfTwoTextures (void)
  * MH says NVIDIA once did the same with their GeForce FX on Windows:
  * http://forums.inside3d.com/viewtopic.php?f=10&t=4832
  * Therefore, advertisement of this extension is an unreliable way of
- * detecting the actual capability: we are binding NPOT support to a
- * cvar defaulting to disabled.
+ * detecting the actual capability.
  */
+	gl_tex_NPOT = have_NPOT = false;
 	if (GL_ParseExtensionList(gl_extensions, "GL_ARB_texture_non_power_of_two"))
 	{
-		gl_has_NPOT = true;
+		have_NPOT = true;
 		Con_SafePrintf("Found ARB_texture_non_power_of_two\n");
+		if (!gl_texture_NPOT.integer) {
+			Con_SafePrintf("ignoring texture_NPOT (cvar disabled)\n");
+		}
+		else {
+			gl_tex_NPOT = true;
+		}
 	}
 	else
 	{
-		gl_has_NPOT = false;
-		Cvar_SetROM("gl_texture_NPOT", "0");
+		Con_SafePrintf("GL_ARB_texture_non_power_of_two not found\n");
 	}
-	gl_tex_NPOT = !!gl_texture_NPOT.integer;
 }
 
 static void CheckStencilBuffer (void)
 {
-	have_stencil = false;
-
-	if (vid_attribs.stencil)
-	{
-		Con_SafePrintf("Stencil buffer created with %d bits\n", vid_attribs.stencil);
-		have_stencil = true;
-	}
+	have_stencil = !!vid_attribs.stencil;
 }
 
 static void GL_ResetFunctions (void)
 {
+#define GL_FUNCTION_OPT(ret, func, params) \
+	func##_fp = NULL;
+#include "gl_func.h"
+
 	have_stencil = false;
-
 	gl_mtexable = false;
-	glActiveTextureARB_fp = NULL;
-	glMultiTexCoord2fARB_fp = NULL;
-
+	have_mtex = false;
 	have8bit = false;
 	is8bit = false;
-
-	gl_has_NPOT = false;
+	have_NPOT = false;
 	gl_tex_NPOT = false;
 }
 
@@ -704,6 +929,18 @@ GL_Init
 */
 static void GL_Init (void)
 {
+	// collect the visual attributes
+	memset (&vid_attribs, 0, sizeof(attributes_t));
+	glGetIntegerv_fp(GL_RED_BITS, &vid_attribs.red);
+	glGetIntegerv_fp(GL_GREEN_BITS, &vid_attribs.green);
+	glGetIntegerv_fp(GL_BLUE_BITS, &vid_attribs.blue);
+	glGetIntegerv_fp(GL_ALPHA_BITS, &vid_attribs.alpha);
+	glGetIntegerv_fp(GL_DEPTH_BITS, &vid_attribs.depth);
+	glGetIntegerv_fp(GL_STENCIL_BITS, &vid_attribs.stencil);
+	Con_SafePrintf ("R:%d G:%d B:%d A:%d, Z:%d, S:%d\n",
+			vid_attribs.red, vid_attribs.green, vid_attribs.blue, vid_attribs.alpha,
+			vid_attribs.depth, vid_attribs.stencil);
+
 	gl_vendor = (const char *)glGetString_fp (GL_VENDOR);
 	Con_SafePrintf ("GL_VENDOR: %s\n", gl_vendor);
 	gl_renderer = (const char *)glGetString_fp (GL_RENDERER);
@@ -735,7 +972,11 @@ static void GL_Init (void)
 	 * drawing of entity models compared to normal polygons.
 	 * (See: R_DrawBrushModel.)
 	 * (Only works if gl_ztrick is turned off) */
+	#if defined(__AMIGA__) && defined(REFGL_MINIGL)
+	#error port to use mglSetZOffset() instead of glPolygonOffset()
+	#else
 	glPolygonOffset_fp(0.05f, 25.0f);
+	#endif
 #endif /* #if 0 */
 	glPolygonMode_fp (GL_FRONT_AND_BACK, GL_FILL);
 	glShadeModel_fp (GL_FLAT);
@@ -751,6 +992,9 @@ static void GL_Init (void)
 //	glTexEnvf_fp(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 	glTexEnvf_fp(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 
+#if defined(__AMIGA__) && defined(REFGL_MINIGL)
+	glHint_fp (GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
+#endif
 }
 
 /*
@@ -765,17 +1009,26 @@ void GL_BeginRendering (int *x, int *y, int *width, int *height)
 	*height = WRHeight;
 
 //	glViewport_fp (*x, *y, *width, *height);
+#if defined(__AMIGA__) && defined(REFGL_MINIGL)
+	mglLockDisplay();
+#endif
 }
 
 
 void GL_EndRendering (void)
 {
 	if (!scr_skipupdate)
-#ifdef __AROS__
+#if defined(__AROS__) && defined(AROS_USE_GLA)
+		glASwapBuffers(context);
+#elif defined(__AROS__)
 		AROSMesaSwapBuffers(context);
 #elif defined __MORPHOS__
 		glASwapBuffers();
 		//GLASwapBuffers(__tglContext);
+#elif defined(__AMIGA__) && defined(REFGL_MINIGL)
+		mglSwitchDisplay(); //performs unlock
+#elif defined(__AMIGA__) && defined(REFGL_AMESA)
+		AmigaMesaSwapBuffers(context);
 #endif
 
 // handle the mouse state when windowed if that's changed
@@ -793,18 +1046,16 @@ void GL_EndRendering (void)
 }
 
 
-int ColorIndex[16] =
-{
+const int ColorIndex[16] = {
 	0, 31, 47, 63, 79, 95, 111, 127, 143, 159, 175, 191, 199, 207, 223, 231
 };
 
-unsigned int ColorPercent[16] =
-{
+const unsigned int ColorPercent[16] = {
 	25, 51, 76, 102, 114, 127, 140, 153, 165, 178, 191, 204, 216, 229, 237, 247
 };
 
 #define	INVERSE_PALNAME	"gfx/invpal.lmp"
-static int ConvertTrueColorToPal (unsigned char *true_color, unsigned char *palette)
+static int ConvertTrueColorToPal (const unsigned char *true_color, const unsigned char *palette)
 {
 	int	i;
 	long	min_dist;
@@ -838,7 +1089,7 @@ static int ConvertTrueColorToPal (unsigned char *true_color, unsigned char *pale
 	return min_index;
 }
 
-static void VID_CreateInversePalette (unsigned char *palette)
+static void VID_CreateInversePalette (const unsigned char *palette)
 {
 	long	r, g, b;
 	long	idx = 0;
@@ -865,9 +1116,9 @@ static void VID_CreateInversePalette (unsigned char *palette)
 	FS_WriteFile (INVERSE_PALNAME, inverse_pal, INVERSE_PAL_SIZE);
 }
 
-static void VID_InitPalette (unsigned char *palette)
+static void VID_InitPalette (const unsigned char *palette)
 {
-	byte	*pal;
+	const unsigned char	*pal;
 	unsigned short	r, g, b;
 	unsigned short	i, p, c;
 	unsigned int	v, *table;
@@ -957,7 +1208,7 @@ static void VID_InitPalette (unsigned char *palette)
 	}
 }
 
-void VID_SetPalette (unsigned char *palette)
+void VID_SetPalette (const unsigned char *palette)
 {
 // nothing to do
 }
@@ -1015,7 +1266,8 @@ static void VID_ChangeVideoMode (int newmode)
 	IN_DeactivateMouse();
 	//IN_ShowMouse ();
 
-	VID_SetMode (newmode);
+	if (!VID_SetMode (newmode))
+		Sys_Error ("Couldn't set video mode");
 
 	// Reload graphics wad file (Draw_PicFromWad writes glpic_t data (sizes,
 	// texnums) right on top of the original pic data, so the pic data will
@@ -1069,6 +1321,20 @@ static void VID_Restart_f (void)
 	VID_ChangeVideoMode (vid_mode.integer);
 }
 
+static int sort_modes (const void *arg1, const void *arg2)
+{
+	const vmode_t *a1, *a2;
+	a1 = (const vmode_t *) arg1;
+	a2 = (const vmode_t *) arg2;
+
+	if (a1->width == a2->width)
+		return a1->height - a2->height;	// lowres-to-highres
+	//	return a2->height - a1->height;	// highres-to-lowres
+	else
+		return a1->width - a2->width;	// lowres-to-highres
+	//	return a2->width - a1->width;	// highres-to-lowres
+}
+
 static void VID_PrepareModes (void)
 {
 	int	i;
@@ -1106,6 +1372,13 @@ static void VID_PrepareModes (void)
 			if (!GetDisplayInfoData(handle, &diminfo, sizeof(diminfo), DTAG_DIMS, 0))
 				continue;
 
+#ifdef __AROS__
+			if (diminfo.MaxDepth != 24)
+#else
+			if (diminfo.MaxDepth != 16)
+#endif
+				continue;
+
 			fmodelist[num_fmodes].width = diminfo.Nominal.MaxX + 1;
 			fmodelist[num_fmodes].height = diminfo.Nominal.MaxY + 1;
 			fmodelist[num_fmodes].fullscreen = 1;
@@ -1113,10 +1386,14 @@ static void VID_PrepareModes (void)
 			q_snprintf (fmodelist[num_fmodes].modedesc, MAX_DESC, "%d x %d", (fmodelist[num_fmodes].width), (fmodelist[num_fmodes].height));
 			//Con_SafePrintf ("fmodelist[%d].modedesc = %s maxdepth %d\n", num_fmodes, fmodelist[num_fmodes].modedesc, diminfo.MaxDepth);
 
-			num_fmodes++;
+			if (++num_fmodes == MAX_MODE_LIST)
+				break;
 		}
 	}
 
+	if (num_fmodes > 1)
+		qsort(fmodelist, num_fmodes, sizeof(vmode_t), sort_modes);
+
 	// disaster scenario #1: no fullscreen modes. bind to the
 	// windowed modes list. limit it to 640x480 max. because
 	// we don't know the desktop dimensions
@@ -1207,17 +1484,12 @@ static void VID_NumModes_f (void)
 	Con_Printf ("%d video modes in current list\n", *nummodes);
 }
 
-static void VID_ShowInfo_f (void)
-{
-	// implement me
-}
-
 /*
 ===================
 VID_Init
 ===================
 */
-void	VID_Init (unsigned char *palette)
+void	VID_Init (const unsigned char *palette)
 {
 	int	i, temp, width, height;
 	const char	*read_vars[] = {
@@ -1227,6 +1499,7 @@ void	VID_Init (unsigned char *palette)
 				"vid_config_gly",
 				"vid_config_consize",
 				"gl_texture_NPOT",
+				"gl_multitexture",
 				"gl_lightmapfmt" };
 #define num_readvars	( sizeof(read_vars)/sizeof(read_vars[0]) )
 
@@ -1234,6 +1507,7 @@ void	VID_Init (unsigned char *palette)
 	Cvar_RegisterVariable (&vid_config_fscr);
 	Cvar_RegisterVariable (&vid_config_swy);
 	Cvar_RegisterVariable (&vid_config_swx);
+	Cvar_RegisterVariable (&vid_config_mon);
 	Cvar_RegisterVariable (&vid_config_gly);
 	Cvar_RegisterVariable (&vid_config_glx);
 	Cvar_RegisterVariable (&vid_config_consize);
@@ -1241,8 +1515,8 @@ void	VID_Init (unsigned char *palette)
 	Cvar_RegisterVariable (&_enable_mouse);
 	Cvar_RegisterVariable (&gl_texture_NPOT);
 	Cvar_RegisterVariable (&gl_lightmapfmt);
+	Cvar_RegisterVariable (&gl_multitexture);
 
-	Cmd_AddCommand ("vid_showinfo", VID_ShowInfo_f);
 	Cmd_AddCommand ("vid_listmodes", VID_ListModes_f);
 	Cmd_AddCommand ("vid_nummodes", VID_NumModes_f);
 	Cmd_AddCommand ("vid_restart", VID_Restart_f);
@@ -1251,6 +1525,19 @@ void	VID_Init (unsigned char *palette)
 
 	vid.numpages = 2;
 
+#if defined(REFGL_MINIGL)
+	mini_CurrentContext = NULL;/* should I do this? */
+	if (MGLInit() == GL_FALSE) {
+		Sys_Error ("Couldn't initialize MiniGL");
+	}
+	mglChoosePixelDepth (16);/* set pixel depth to 16 */
+#endif
+#if defined(REFGL_AMESA)
+	CyberGfxBase = OpenLibrary("cybergraphics.library", 0);
+	if (!CyberGfxBase)
+		Sys_Error ("Cannot open cybergraphics.library!");
+#endif
+
 	// prepare the modelists, find the actual modenum for vid_default
 	VID_PrepareModes();
 
@@ -1359,7 +1646,8 @@ void	VID_Init (unsigned char *palette)
 	temp = scr_disabled_for_loading;
 	scr_disabled_for_loading = true;
 	//set the mode
-	VID_SetMode (vid_mode.integer);
+	if (!VID_SetMode (vid_mode.integer))
+		Sys_Error ("Couldn't set video mode");
 	ClearAllStates ();
 
 	GL_SetupLightmapFmt();
@@ -1387,7 +1675,28 @@ void	VID_Init (unsigned char *palette)
 
 void	VID_Shutdown (void)
 {
-#ifdef __AROS__
+	VID_KillContext();
+#ifdef REFGL_MINIGL
+	MGLTerm();
+#endif
+#ifdef REFGL_AMESA
+	if (CyberGfxBase) {
+		CloseLibrary(CyberGfxBase);
+		CyberGfxBase = NULL;
+	}
+#endif
+}
+
+
+static void VID_KillContext (void)
+{
+#if defined(__AROS__) && defined(AROS_USE_GLA)
+	if (context)
+	{
+		glADestroyContext(context);
+		context = NULL;
+	}
+#elif defined(__AROS__)
 	if (context)
 	{
 		AROSMesaDestroyContext(context);
@@ -1402,14 +1711,28 @@ void	VID_Shutdown (void)
 			glADestroyContextWindowed();
 		contextinit = false;
 	}
-
 	if (__tglContext)
 	{
 		GLClose(__tglContext);
 		__tglContext = NULL;
 	}
+#elif defined(__AMIGA__) && defined(REFGL_MINIGL)
+	if (mini_CurrentContext)
+	{
+		mglDeleteContext();
+		window = NULL;
+		screen = NULL;
+		mini_CurrentContext = NULL;
+	}
+#elif defined(__AMIGA__) && defined(REFGL_AMESA)
+	if (context)
+	{
+		AmigaMesaDestroyContext(context);
+		context = NULL;
+	}
 #endif
 
+#ifndef REFGL_MINIGL
 	if (window)
 	{
 		CloseWindow(window);
@@ -1421,6 +1744,7 @@ void	VID_Shutdown (void)
 		CloseScreen(screen);
 		screen = NULL;
 	}
+#endif
 }
 
 
@@ -1480,6 +1804,8 @@ enum {
 	VID_FULLSCREEN,	// make sure the fullscreen entry (0)
 	VID_RESOLUTION,	// is lower than resolution entry (1)
 	VID_MULTISAMPLE,
+	VID_MULTITEXTURE,
+	VID_NPOT,
 	VID_PALTEX,
 	VID_BLANKLINE,	// spacer line
 	VID_RESET,
@@ -1525,6 +1851,8 @@ static void VID_MenuDraw (void)
 	want_fstoggle = ( ((modestate == MS_WINDOWED) && vid_menu_fs) || ((modestate != MS_WINDOWED) && !vid_menu_fs) );
 
 	need_apply = (vid_menunum != vid_modenum) || want_fstoggle ||
+			(have_mtex && (gl_mtexable != !!gl_multitexture.integer)) ||
+			(have_NPOT && (gl_tex_NPOT != !!gl_texture_NPOT.integer)) ||
 			(have8bit && (is8bit != !!vid_config_gl8bit.integer));
 
 	M_Print (76, 92 + 8*VID_FULLSCREEN, "Fullscreen: ");
@@ -1539,6 +1867,18 @@ static void VID_MenuDraw (void)
 	M_Print (76, 92 + 8*VID_MULTISAMPLE, "Antialiasing  :");
 	M_PrintWhite (76+16*8, 92 + 8*VID_MULTISAMPLE, "Not found");
 
+	M_Print (76, 92 + 8*VID_MULTITEXTURE, "Multitexturing:");
+	if (have_mtex)
+		M_DrawYesNo (76+16*8, 92 + 8*VID_MULTITEXTURE, gl_multitexture.integer, (gl_mtexable == !!gl_multitexture.integer));
+	else
+		M_PrintWhite (76+16*8, 92 + 8*VID_MULTITEXTURE, "Not found");
+
+	M_Print (76, 92 + 8*VID_NPOT, "NPOT textures :");
+	if (have_NPOT)
+		M_DrawYesNo (76+16*8, 92 + 8*VID_NPOT, gl_texture_NPOT.integer, (gl_tex_NPOT == !!gl_texture_NPOT.integer));
+	else
+		M_PrintWhite (76+16*8, 92 + 8*VID_NPOT, "Not found");
+
 	M_Print (76, 92 + 8*VID_PALTEX, "8 bit textures:");
 	if (have8bit)
 		M_DrawYesNo (76+16*8, 92 + 8*VID_PALTEX, vid_config_gl8bit.integer, (is8bit == !!vid_config_gl8bit.integer));
@@ -1640,9 +1980,17 @@ static void VID_MenuKey (int key)
 			break;
 		case VID_MULTISAMPLE:
 			break;
+		case VID_MULTITEXTURE:
+			if (have_mtex)
+				Cvar_SetQuick (&gl_multitexture, gl_multitexture.integer ? "0" : "1");
+			break;
+		case VID_NPOT:
+			if (have_NPOT)
+				Cvar_SetQuick (&gl_texture_NPOT, gl_texture_NPOT.integer ? "0" : "1");
+			break;
 		case VID_PALTEX:
 			if (have8bit)
-				Cvar_SetValueQuick (&vid_config_gl8bit, !vid_config_gl8bit.integer);
+				Cvar_SetQuick (&vid_config_gl8bit, vid_config_gl8bit.integer ? "0" : "1");
 			break;
 		}
 		return;
@@ -1663,9 +2011,17 @@ static void VID_MenuKey (int key)
 			break;
 		case VID_MULTISAMPLE:
 			break;
+		case VID_MULTITEXTURE:
+			if (have_mtex)
+				Cvar_SetQuick (&gl_multitexture, gl_multitexture.integer ? "0" : "1");
+			break;
+		case VID_NPOT:
+			if (have_NPOT)
+				Cvar_SetQuick (&gl_texture_NPOT, gl_texture_NPOT.integer ? "0" : "1");
+			break;
 		case VID_PALTEX:
 			if (have8bit)
-				Cvar_SetValueQuick (&vid_config_gl8bit, !vid_config_gl8bit.integer);
+				Cvar_SetQuick (&vid_config_gl8bit, vid_config_gl8bit.integer ? "0" : "1");
 			break;
 		}
 		return;
