@@ -1,10 +1,10 @@
 /*
  * gl_vidnt.c -- NT GL vid component
- * $Id: gl_vidnt.c 5158 2013-03-02 14:23:05Z sezero $
+ * $Id: gl_vidnt.c 5681 2016-07-21 05:40:23Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
- * Copyright (C) 2005-2012  O.Sezer <sezero@users.sourceforge.net>
+ * Copyright (C) 2005-2016  O.Sezer <sezero@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,6 @@
 #include "resource.h"
 #include "wgl_func.h"
 
-
 #define WARP_WIDTH		320
 #define WARP_HEIGHT		200
 #define MAXWIDTH		10000
@@ -125,14 +124,24 @@ static PIXELFORMATDESCRIPTOR pfd =
 	0,				// shift bit ignored
 	0,				// no accumulation buffer
 	0, 0, 0, 0,			// accum bits ignored
-	24,				// 24-bit z-buffer
-	8,				// 8-bit stencil buffer
+	32,				// 32-bit z-buffer
+	0,				// no stencil buffer
 	0,				// no auxiliary buffer
 	PFD_MAIN_PLANE,			// main layer
 	0,				// reserved
 	0, 0, 0				// layer masks ignored
 };
 
+typedef struct {
+	int	red,
+		green,
+		blue,
+		alpha,
+		depth,
+		stencil;
+} attributes_t;
+static attributes_t	vid_attribs;
+
 // main vid functions
 static LRESULT WINAPI MainWndProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
 static void AppActivate(BOOL fActive, BOOL minimize);
@@ -193,9 +202,9 @@ static const char	*gl_extensions;
 qboolean	is_3dfx = false;
 
 GLint		gl_max_size = 256;
-static qboolean	gl_has_NPOT = false;
+static qboolean	have_NPOT = false;
 qboolean	gl_tex_NPOT = false;
-cvar_t		gl_texture_NPOT = {"gl_texture_NPOT", "0", CVAR_ARCHIVE};
+static cvar_t	gl_texture_NPOT = {"gl_texture_NPOT", "0", CVAR_ARCHIVE};
 GLfloat		gl_max_anisotropy;
 float		gldepthmin, gldepthmax;
 
@@ -205,17 +214,18 @@ qboolean	is8bit = false;
 static cvar_t	vid_config_gl8bit = {"vid_config_gl8bit", "0", CVAR_ARCHIVE};
 
 // Gamma stuff
-typedef BOOL	(APIENTRY *GAMMA_RAMP_FN)(HDC, LPVOID);
+typedef BOOL	(WINAPI *GAMMA_RAMP_FN)(HDC, LPVOID);
 static GAMMA_RAMP_FN	GetDeviceGammaRamp_f;
 static GAMMA_RAMP_FN	SetDeviceGammaRamp_f;
 extern unsigned short	ramps[3][256];	// for hw- or 3dfx-gamma
 static unsigned short	orig_ramps[3][256];	// for hw- or 3dfx-gamma
 static qboolean	gammaworks = false;	// whether hw-gamma works
-qboolean	gl_dogamma = false;
 
 // multitexturing
 qboolean	gl_mtexable = false;
 static GLint	num_tmus = 1;
+static qboolean	have_mtex = false;
+static cvar_t	gl_multitexture = {"gl_multitexture", "0", CVAR_ARCHIVE};
 
 // stencil buffer
 qboolean	have_stencil = false;
@@ -318,7 +328,7 @@ static void VID_ConWidth (int modenum)
 	}
 
 	w = vid_config_consize.integer;
-	w &= 0xfff8; /* make it a multiple of 8 */
+	w &= ~7; /* make it a multiple of eight */
 	if (w < MIN_WIDTH)
 		w = MIN_WIDTH;
 	else if (w > modelist[modenum].width)
@@ -347,14 +357,14 @@ void VID_ChangeConsize (int dir)
 	{
 	case -1: /* smaller text */
 		w = ((float)vid.conwidth/(float)vid.width + 0.05f) * vid.width; /* use 0.10f increment ?? */
-		w &= 0xfff8; /* make it a multiple of 8 */
+		w &= ~7; /* make it a multiple of eight */
 		if (w > modelist[vid_modenum].width)
 			w = modelist[vid_modenum].width;
 		break;
 
 	case 1: /* bigger text */
 		w = ((float)vid.conwidth/(float)vid.width - 0.05f) * vid.width;
-		w &= 0xfff8; /* make it a multiple of 8 */
+		w &= ~7; /* make it a multiple of eight */
 		if (w < MIN_WIDTH)
 			w = MIN_WIDTH;
 		break;
@@ -386,6 +396,22 @@ static qboolean VID_SetWindowedMode (int modenum)
 	int	width, height;
 	RECT	rect;
 
+	pfd.cColorBits = 24;
+	pfd.cRedBits = 0;
+	pfd.cGreenBits = 0;
+	pfd.cBlueBits = 0;
+	pfd.cAlphaBits = 0;
+	pfd.cDepthBits = 32;
+	pfd.cStencilBits = 0;
+	if (vid_deskbpp >= 32) {
+		pfd.cRedBits = 8;
+		pfd.cGreenBits = 8;
+		pfd.cBlueBits = 8;
+		pfd.cAlphaBits = 8;
+		pfd.cDepthBits = 24;
+		pfd.cStencilBits = 8;
+	}
+
 	// Pa3PyX: set the original fullscreen mode if
 	// we are switching to window from fullscreen.
 	if (modestate == MS_FULLDIB)
@@ -430,6 +456,22 @@ static qboolean VID_SetFullDIBMode (int modenum)
 	RECT	rect;
 
 	pfd.cColorBits = modelist[modenum].bpp;
+	if (modelist[modenum].bpp >= 32) {
+		pfd.cRedBits = 8;
+		pfd.cGreenBits = 8;
+		pfd.cBlueBits = 8;
+		pfd.cAlphaBits = 8;
+		pfd.cDepthBits = 24;
+		pfd.cStencilBits = 8;
+	}
+	else {
+		pfd.cRedBits = 5;
+		pfd.cGreenBits = 5;
+		pfd.cBlueBits = 5;
+		pfd.cAlphaBits = 0;
+		pfd.cDepthBits = 16;
+		pfd.cStencilBits = 0;
+	}
 
 	gdevmode.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT;
 	if (!Win95old)
@@ -477,7 +519,7 @@ static qboolean VID_SetFullDIBMode (int modenum)
 }
 
 
-static int VID_SetMode (int modenum, unsigned char *palette)
+static qboolean VID_SetMode (int modenum, const unsigned char *palette)
 {
 	qboolean	status = false;
 	MSG		msg;
@@ -582,30 +624,82 @@ static int VID_SetMode (int modenum, unsigned char *palette)
 
 //====================================
 
-static void VID_Init8bitPalette (void)
+#if 0 /* No.. */
+static void CheckSetGlobalPalette (void)
 {
-	// Check for 8bit Extensions and initialize them.
-	int			i;
-	char	thePalette[256*3];
-	char	*oldPalette, *newPalette;
+	gl3DfxSetPaletteEXT_f gl3DfxSetPaletteEXT_fp;
 
-	have8bit = false;
-	is8bit = false;
-
-	if (GL_ParseExtensionList(gl_extensions, "GL_EXT_shared_texture_palette"))
+	if (GL_ParseExtensionList(gl_extensions, "3DFX_set_global_palette"))
 	{
-		glColorTableEXT_fp = (glColorTableEXT_f)wglGetProcAddress_fp("glColorTableEXT");
-		if (glColorTableEXT_fp == NULL)
+		gl3DfxSetPaletteEXT_fp = (gl3DfxSetPaletteEXT_f) wglGetProcAddress_fp("gl3DfxSetPaletteEXT");
+		if (!gl3DfxSetPaletteEXT_fp)
+			gl3DfxSetPaletteEXT_fp = (gl3DfxSetPaletteEXT_f) wglGetProcAddress_fp("3DFX_set_global_palette");
+		if (!gl3DfxSetPaletteEXT_fp)
 			return;
-
-		have8bit = true;
-		if (!vid_config_gl8bit.integer)
+		Con_SafePrintf("Found 3DFX_set_global_palette\n");
+	}
+	else if (GL_ParseExtensionList(gl_extensions, "POWERVR_set_global_palette"))
+	{
+		gl3DfxSetPaletteEXT_fp = (gl3DfxSetPaletteEXT_f) wglGetProcAddress_fp("glSetGlobalPalettePOWERVR");
+		if (!gl3DfxSetPaletteEXT_fp)
+			gl3DfxSetPaletteEXT_fp = (gl3DfxSetPaletteEXT_f) wglGetProcAddress_fp("POWERVR_set_global_palette");
+		if (!gl3DfxSetPaletteEXT_fp)
 			return;
+		Con_SafePrintf("Found POWERVR_set_global_palette\n");
+	}
+	else {
+		return;
+	}
 
+	have8bit = true;
+	if (!vid_config_gl8bit.integer)
+		return;
+	else
+	{
+		int i;
+		GLubyte table[256][4];
+		char *oldpal;
+
+		is8bit = true;
+		oldpal = (char *) d_8to24table;
+		for (i = 0; i < 256; i++) {
+			table[i][2] = *oldpal++;
+			table[i][1] = *oldpal++;
+			table[i][0] = *oldpal++;
+			table[i][3] = 255;
+			oldpal++;
+		}
+		glEnable_fp (GL_SHARED_TEXTURE_PALETTE_EXT);
+		gl3DfxSetPaletteEXT_fp ((GLuint *)table);
+	}
+}
+#endif /* #if 0 */
+
+static void CheckSharedTexturePalette (void)
+{
+	glColorTableEXT_f glColorTableEXT_fp;
+
+	if (!GL_ParseExtensionList(gl_extensions, "GL_EXT_shared_texture_palette"))
+		return;
+
+	glColorTableEXT_fp = (glColorTableEXT_f) wglGetProcAddress_fp("glColorTableEXT");
+	if (glColorTableEXT_fp == NULL)
+		return;
+
+	have8bit = true;
+	Con_SafePrintf("Found GL_EXT_shared_texture_palette\n");
+	if (!vid_config_gl8bit.integer)
+		return;
+	else
+	{
+		int i;
+		char thePalette[256*3];
+		char *oldPalette, *newPalette;
+
+		is8bit = true;
 		oldPalette = (char *) d_8to24table;
 		newPalette = thePalette;
-		for (i = 0; i < 256; i++)
-		{
+		for (i = 0; i < 256; i++) {
 			*newPalette++ = *oldPalette++;
 			*newPalette++ = *oldPalette++;
 			*newPalette++ = *oldPalette++;
@@ -615,11 +709,26 @@ static void VID_Init8bitPalette (void)
 		glEnable_fp (GL_SHARED_TEXTURE_PALETTE_EXT);
 		glColorTableEXT_fp (GL_SHARED_TEXTURE_PALETTE_EXT, GL_RGB, 256,
 					GL_RGB, GL_UNSIGNED_BYTE, (void *) thePalette);
-		is8bit = true;
-		Con_SafePrintf("8-bit palettized textures enabled\n");
 	}
 }
 
+static void VID_Init8bitPalette (void)
+{
+	have8bit = false;
+	is8bit = false;
+
+	/* Check for 8bit Extensions and initialize them */
+	CheckSharedTexturePalette();
+#if 0	/* No.. */
+	if (!have8bit)
+		CheckSetGlobalPalette();
+#endif
+
+	if (is8bit)
+		Con_SafePrintf("8-bit palettized textures enabled\n");
+}
+
+
 static void VID_Check3dfxGamma (void)
 {
 	if (!GL_ParseExtensionList(gl_extensions, "WGL_3DFX_gamma_control"))
@@ -645,23 +754,18 @@ static void VID_InitGamma (void)
 {
 	GetDeviceGammaRamp_f = NULL;
 	SetDeviceGammaRamp_f = NULL;
+	gammaworks = false;
 
 	VID_Check3dfxGamma ();
 
 	if (GetDeviceGammaRamp_f)
 		gammaworks = GetDeviceGammaRamp_f(maindc, orig_ramps);
-	else
-		gammaworks = false;
 
 	if (!gammaworks)
-	{
-		// we can still adjust the brightness...
-		Con_SafePrintf("gamma not available, using gl tricks\n");
-		gl_dogamma = true;
-	}
+		Con_SafePrintf("gamma adjustment not available\n");
 }
 
-void VID_ShiftPalette (unsigned char *palette)
+void VID_ShiftPalette (const unsigned char *palette)
 {
 	if (gammaworks && SetDeviceGammaRamp_f)
 		SetDeviceGammaRamp_f (maindc, ramps);
@@ -670,7 +774,7 @@ void VID_ShiftPalette (unsigned char *palette)
 
 static void CheckMultiTextureExtensions (void)
 {
-	gl_mtexable = false;
+	gl_mtexable = have_mtex = false;
 
 	if (COM_CheckParm("-nomtex"))
 	{
@@ -683,19 +787,25 @@ static void CheckMultiTextureExtensions (void)
 		glGetIntegerv_fp(GL_MAX_TEXTURE_UNITS_ARB, &num_tmus);
 		if (num_tmus < 2)
 		{
-			Con_SafePrintf("not enough TMUs, ignoring multitexture\n");
+			Con_SafePrintf("ignoring multitexture (%i TMUs)\n", (int) num_tmus);
 			return;
 		}
 
 		glMultiTexCoord2fARB_fp = (glMultiTexCoord2fARB_f) wglGetProcAddress_fp("glMultiTexCoord2fARB");
 		glActiveTextureARB_fp = (glActiveTextureARB_f) wglGetProcAddress_fp("glActiveTextureARB");
-		if ((glMultiTexCoord2fARB_fp == NULL) ||
-		    (glActiveTextureARB_fp == NULL))
+		if (glMultiTexCoord2fARB_fp == NULL || glActiveTextureARB_fp == NULL)
 		{
 			Con_SafePrintf ("Couldn't link to multitexture functions\n");
 			return;
 		}
 
+		have_mtex = true;
+		if (!gl_multitexture.integer)
+		{
+			Con_SafePrintf("ignoring multitexture (cvar disabled)\n");
+			return;
+		}
+
 		Con_SafePrintf("Found %i TMUs support\n", num_tmus);
 		gl_mtexable = true;
 		glDisable_fp(GL_TEXTURE_2D);
@@ -714,8 +824,9 @@ static void CheckAnisotropyExtensions (void)
 	Con_SafePrintf("Anisotropic filtering ");
 	if (GL_ParseExtensionList(gl_extensions, "GL_EXT_texture_filter_anisotropic"))
 	{
-		GLfloat test1, test2;
+		GLfloat test1 = 0, test2 = 0;
 		GLuint tex;
+		glGetTexParameterfv_f glGetTexParameterfv_fp;
 
 		glGetTexParameterfv_fp = (glGetTexParameterfv_f) GetProcAddress(hInstGL, "glGetTexParameterfv");
 		if (glGetTexParameterfv_fp == NULL)
@@ -761,37 +872,37 @@ static void CheckNonPowerOfTwoTextures (void)
  * MH says NVIDIA once did the same with their GeForce FX on Windows:
  * http://forums.inside3d.com/viewtopic.php?f=10&t=4832
  * Therefore, advertisement of this extension is an unreliable way of
- * detecting the actual capability: we are binding NPOT support to a
- * cvar defaulting to disabled.
+ * detecting the actual capability.
  */
+	gl_tex_NPOT = have_NPOT = false;
 	if (GL_ParseExtensionList(gl_extensions, "GL_ARB_texture_non_power_of_two"))
 	{
-		gl_has_NPOT = true;
+		have_NPOT = true;
 		Con_SafePrintf("Found ARB_texture_non_power_of_two\n");
+		if (!gl_texture_NPOT.integer) {
+			Con_SafePrintf("ignoring texture_NPOT (cvar disabled)\n");
+		}
+		else {
+			gl_tex_NPOT = true;
+		}
 	}
 	else
 	{
-		gl_has_NPOT = false;
-		Cvar_SetROM("gl_texture_NPOT", "0");
+		Con_SafePrintf("GL_ARB_texture_non_power_of_two not found\n");
 	}
-	gl_tex_NPOT = !!gl_texture_NPOT.integer;
 }
 
 static void CheckStencilBuffer (void)
 {
-	have_stencil = false;
-
-	if (pfd.cStencilBits)
-	{
-		Con_SafePrintf("Stencil buffer created with %d bits\n", pfd.cStencilBits);
-		have_stencil = true;
-	}
+	have_stencil = !!vid_attribs.stencil;
 }
 
 
 #ifdef GL_DLSYM
 static qboolean GL_OpenLibrary (const char *name)
 {
+	int drv_standalone = q_strcasecmp(name, "opengl32.dll");
+
 	Con_SafePrintf("Loading OpenGL library %s\n", name);
 
 	// open the library
@@ -808,10 +919,14 @@ static qboolean GL_OpenLibrary (const char *name)
 	if (func##_fp == NULL)					\
 		Sys_Error("Couldn't link to %s", #func);	\
     } while (0);
-#define GL_FUNCTION_OPT(ret, func, params)
+#define GL_FUNCTION_OPT(ret,func,def,params)			\
+    do {							\
+	func##_fp = (drv_standalone)?				\
+	   (func##_f) GetProcAddress(hInstGL, #func) : def;	\
+	if (func##_fp == NULL)					\
+		Sys_Error("Couldn't link to %s", #func);	\
+    } while (0);
 #include "wgl_func.h"
-#undef	GL_FUNCTION_OPT
-#undef	GL_FUNCTION
 
 	return true;
 }
@@ -821,10 +936,9 @@ static void GL_CloseLibrary (void)
 	// clear the wgl function pointers
 #define GL_FUNCTION(ret, func, params)	\
 	func##_fp = NULL;
-#define GL_FUNCTION_OPT(ret, func, params)
+#define GL_FUNCTION_OPT(ret,func,def,params) \
+	func##_fp = NULL;
 #include "wgl_func.h"
-#undef	GL_FUNCTION_OPT
-#undef	GL_FUNCTION
 
 	// free the library
 	if (hInstGL != NULL)
@@ -842,8 +956,6 @@ static void GL_Init_Functions (void)
     } while (0);
 #define GL_FUNCTION_OPT(ret, func, params)
 #include "gl_func.h"
-#undef	GL_FUNCTION_OPT
-#undef	GL_FUNCTION
 }
 #endif	/* GL_DLSYM */
 
@@ -852,26 +964,21 @@ static void GL_ResetFunctions (void)
 #ifdef	GL_DLSYM
 #define GL_FUNCTION(ret, func, params)	\
 	func##_fp = NULL;
-#define GL_FUNCTION_OPT(ret, func, params)
+#endif
+#define GL_FUNCTION_OPT(ret, func, params) \
+	func##_fp = NULL;
 #include "gl_func.h"
-#undef	GL_FUNCTION_OPT
-#undef	GL_FUNCTION
-#endif	/* GL_DLSYM */
-
-	have_stencil = false;
-
-	gl_mtexable = false;
-	glActiveTextureARB_fp = NULL;
-	glMultiTexCoord2fARB_fp = NULL;
-
-	have8bit = false;
-	is8bit = false;
-
-	gl_has_NPOT = false;
-	gl_tex_NPOT = false;
 
 	GetDeviceGammaRamp_f = NULL;
 	SetDeviceGammaRamp_f = NULL;
+
+	have_stencil = false;
+	gl_mtexable = false;
+	have_mtex = false;
+	have8bit = false;
+	is8bit = false;
+	have_NPOT = false;
+	gl_tex_NPOT = false;
 }
 
 
@@ -885,21 +992,36 @@ static void GL_Init (void)
 	PIXELFORMATDESCRIPTOR	new_pfd;
 
 	Con_SafePrintf ("Video mode %s initialized\n", VID_GetModeDescription (vid_modenum));
-	// DescribePixelFormat fails with old 3dfx minigl drivers: don't Sys_Error
-	if (DescribePixelFormat(maindc, GetPixelFormat(maindc), sizeof(PIXELFORMATDESCRIPTOR), &new_pfd))
+
+#ifdef GL_DLSYM
+	// initialize gl function pointers
+	GL_Init_Functions();
+#endif
+
+	// collect the visual attributes
+	memset (&new_pfd, 0, sizeof(PIXELFORMATDESCRIPTOR));
+	memset (&vid_attribs, 0, sizeof(attributes_t));
+	glGetIntegerv_fp(GL_RED_BITS, &vid_attribs.red);
+	glGetIntegerv_fp(GL_GREEN_BITS, &vid_attribs.green);
+	glGetIntegerv_fp(GL_BLUE_BITS, &vid_attribs.blue);
+	glGetIntegerv_fp(GL_ALPHA_BITS, &vid_attribs.alpha);
+	glGetIntegerv_fp(GL_DEPTH_BITS, &vid_attribs.depth);
+	glGetIntegerv_fp(GL_STENCIL_BITS, &vid_attribs.stencil);
+	Con_SafePrintf ("R:%d G:%d B:%d A:%d, Z:%d, S:%d\n",
+			vid_attribs.red, vid_attribs.green, vid_attribs.blue, vid_attribs.alpha,
+			vid_attribs.depth, vid_attribs.stencil);
+	/* FIXME: DescribePixelFormat() used to fail with old 3dfx minigl drivers ??? */
+	if (wglDescribePixelFormat_fp(maindc, wglGetPixelFormat_fp(maindc), sizeof(PIXELFORMATDESCRIPTOR), &new_pfd))
 	{
-		Con_SafePrintf("Pixel format: c: %d, z: %d, s: %d\n",
-			new_pfd.cColorBits, new_pfd.cDepthBits, new_pfd.cStencilBits);
+		Con_SafeDPrintf ("PFD: C:%d, R:%d G:%d B:%d A:%d, Z:%d, S:%d\n",
+			new_pfd.cColorBits, new_pfd.cRedBits, new_pfd.cGreenBits, new_pfd.cBlueBits,
+			new_pfd.cAlphaBits, new_pfd.cDepthBits, new_pfd.cStencilBits);
 		if ((new_pfd.dwFlags & PFD_GENERIC_FORMAT) && !(new_pfd.dwFlags & PFD_GENERIC_ACCELERATED))
 			Con_SafePrintf ("WARNING: Hardware acceleration not present\n");
 		else if (new_pfd.dwFlags & PFD_GENERIC_ACCELERATED)
 			Con_SafePrintf ("OpenGL: MCD acceleration found\n");
 	}
 
-#ifdef GL_DLSYM
-	// initialize gl function pointers
-	GL_Init_Functions();
-#endif
 	gl_vendor = (const char *)glGetString_fp (GL_VENDOR);
 	Con_SafePrintf ("GL_VENDOR: %s\n", gl_vendor);
 	gl_renderer = (const char *)glGetString_fp (GL_RENDERER);
@@ -918,6 +1040,7 @@ static void GL_Init (void)
 	is_3dfx = false;
 	if (!q_strncasecmp(gl_renderer, "3dfx", 4)	  ||
 	    !q_strncasecmp(gl_renderer, "SAGE Glide", 10) ||
+	    !q_strncasecmp(gl_renderer, "Glide ", 6)	  || /* possible with Mesa 3.x/4.x/5.0.x */
 	    !q_strncasecmp(gl_renderer, "Mesa Glide", 10))
 	{
 		Con_SafePrintf("3dfx Voodoo found\n");
@@ -985,7 +1108,7 @@ void GL_BeginRendering (int *x, int *y, int *width, int *height)
 void GL_EndRendering (void)
 {
 	if (!scr_skipupdate)
-		SwapBuffers(maindc);
+		wglSwapBuffers_fp(maindc);
 
 // handle the mouse state when windowed if that's changed
 	if (modestate == MS_WINDOWED)
@@ -1012,18 +1135,16 @@ void GL_EndRendering (void)
 }
 
 
-int ColorIndex[16] =
-{
+const int ColorIndex[16] = {
 	0, 31, 47, 63, 79, 95, 111, 127, 143, 159, 175, 191, 199, 207, 223, 231
 };
 
-unsigned int ColorPercent[16] =
-{
+const unsigned int ColorPercent[16] = {
 	25, 51, 76, 102, 114, 127, 140, 153, 165, 178, 191, 204, 216, 229, 237, 247
 };
 
 #define	INVERSE_PALNAME	"gfx/invpal.lmp"
-static int ConvertTrueColorToPal (unsigned char *true_color, unsigned char *palette)
+static int ConvertTrueColorToPal (const unsigned char *true_color, const unsigned char *palette)
 {
 	int	i;
 	long	min_dist;
@@ -1057,7 +1178,7 @@ static int ConvertTrueColorToPal (unsigned char *true_color, unsigned char *pale
 	return min_index;
 }
 
-static void VID_CreateInversePalette (unsigned char *palette)
+static void VID_CreateInversePalette (const unsigned char *palette)
 {
 	long	r, g, b;
 	long	idx = 0;
@@ -1084,9 +1205,9 @@ static void VID_CreateInversePalette (unsigned char *palette)
 	FS_WriteFile (INVERSE_PALNAME, inverse_pal, INVERSE_PAL_SIZE);
 }
 
-static void VID_InitPalette (unsigned char *palette)
+static void VID_InitPalette (const unsigned char *palette)
 {
-	byte	*pal;
+	const unsigned char	*pal;
 	unsigned short	r, g, b;
 	unsigned short	i, p, c;
 	unsigned int	v, *table;
@@ -1176,7 +1297,7 @@ static void VID_InitPalette (unsigned char *palette)
 	}
 }
 
-void VID_SetPalette (unsigned char *palette)
+void VID_SetPalette (const unsigned char *palette)
 {
 // nothing to do
 }
@@ -1188,13 +1309,13 @@ static BOOL bSetupPixelFormat (HDC hDC)
 {
 	int	pixelformat;
 
-	if ( (pixelformat = ChoosePixelFormat(hDC, &pfd)) == 0 )
+	if ( (pixelformat = wglChoosePixelFormat_fp(hDC, &pfd)) == 0 )
 	{
 		MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK);
 		return FALSE;
 	}
 
-	if (SetPixelFormat(hDC, pixelformat, &pfd) == FALSE)
+	if (wglSetPixelFormat_fp(hDC, pixelformat, &pfd) == FALSE)
 	{
 		MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK);
 		return FALSE;
@@ -1939,7 +2060,7 @@ static void VID_InitFullDIB (HINSTANCE hInstance)
 				break;
 
 			case 32:
-#if 0	// O.S: don't mess with silly 24 bit lowres modes, they don't work correctly
+#if 0	/* O.S: don't mess with silly 24 bit lowres modes, they may not work correctly */
 				bpp = 24;
 				break;
 
@@ -2152,10 +2273,12 @@ static void VID_SortModes (void)
 VID_Init
 ===================
 */
-void	VID_Init (unsigned char *palette)
+void	VID_Init (const unsigned char *palette)
 {
+	static char fxmesa_env_multitex[32] = "FX_DONT_FAKE_MULTITEX=1";
+	static char fxglide_env_nosplash[32] = "FX_GLIDE_NO_SPLASH=1";
 	int	i, j, existingmode;
-	int	width, height, bpp, zbits, findbpp, done;
+	int	width, height, bpp, findbpp, done;
 	HDC	hdc;
 	const char	*read_vars[] = {
 				"vid_config_fscr",
@@ -2165,6 +2288,7 @@ void	VID_Init (unsigned char *palette)
 				"vid_config_gly",
 				"vid_config_consize",
 				"gl_texture_NPOT",
+				"gl_multitexture",
 				"gl_lightmapfmt" };
 #define num_readvars	( sizeof(read_vars)/sizeof(read_vars[0]) )
 
@@ -2178,6 +2302,7 @@ void	VID_Init (unsigned char *palette)
 	Cvar_RegisterVariable (&_enable_mouse);
 	Cvar_RegisterVariable (&gl_texture_NPOT);
 	Cvar_RegisterVariable (&gl_lightmapfmt);
+	Cvar_RegisterVariable (&gl_multitexture);
 	// these are for compatibility with the software version
 	Cvar_RegisterVariable (&vid_wait);
 	Cvar_RegisterVariable (&_vid_default_mode);
@@ -2196,10 +2321,13 @@ void	VID_Init (unsigned char *palette)
 
 	VID_InitPalette (palette);
 
+	// don't let fxMesa cheat multitexturing
+	_putenv (fxmesa_env_multitex);
+	// disable the 3dfx splash screen.
+	_putenv (fxglide_env_nosplash);
+
 #ifdef GL_DLSYM
-	i = COM_CheckParm("--gllibrary");
-	if (i == 0)
-		i = COM_CheckParm ("-gllibrary");
+	i = COM_CheckParm ("-gllibrary");
 	if (i == 0)
 		i = COM_CheckParm ("-g");
 	if (i && i < com_argc - 1)
@@ -2461,16 +2589,6 @@ void	VID_Init (unsigned char *palette)
 					   "video options, try changing them, or delete your config.cfg\n"
 					   "altogether and try again.");
 			}
-
-			//pfd.cColorBits = modelist[vid_default].bpp;
-
-			i = COM_CheckParm("-zbits");
-			if (i && i < com_argc-1)
-			{
-				zbits = atoi(com_argv[i+1]);
-				if (zbits)
-					pfd.cDepthBits = zbits;
-			}
 		}
 	}	// end of fullscreen parsing
 
@@ -2636,6 +2754,8 @@ enum {
 	VID_FULLSCREEN,
 	VID_RESOLUTION,
 	VID_BPP,
+	VID_MULTITEXTURE,
+	VID_NPOT,
 	VID_PALTEX,
 	VID_BLANKLINE,	// spacer line
 	VID_RESET,
@@ -2683,7 +2803,10 @@ static void VID_MenuDraw (void)
 
 	want_fstoggle = ( ((modestate == MS_WINDOWED) && vid_menu_fs) || ((modestate != MS_WINDOWED) && !vid_menu_fs) );
 
-	need_apply = (vid_menunum != vid_modenum) || want_fstoggle || (have8bit && (is8bit != !!vid_config_gl8bit.integer));
+	need_apply = (vid_menunum != vid_modenum) || want_fstoggle ||
+			(have_mtex && (gl_mtexable != !!gl_multitexture.integer)) ||
+			(have_NPOT && (gl_tex_NPOT != !!gl_texture_NPOT.integer)) ||
+			(have8bit && (is8bit != !!vid_config_gl8bit.integer));
 
 	M_Print (76, 92 + 8*VID_FULLSCREEN, "Fullscreen: ");
 	M_DrawYesNo (76+12*8, 92 + 8*VID_FULLSCREEN, vid_menu_fs, !want_fstoggle);
@@ -2703,6 +2826,18 @@ static void VID_MenuDraw (void)
 			M_Print (76+12*8, 92 + 8*VID_BPP, va("%d",vid_menubpp));
 	}
 
+	M_Print (76, 92 + 8*VID_MULTITEXTURE, "Multitexturing:");
+	if (have_mtex)
+		M_DrawYesNo (76+16*8, 92 + 8*VID_MULTITEXTURE, gl_multitexture.integer, (gl_mtexable == !!gl_multitexture.integer));
+	else
+		M_PrintWhite (76+16*8, 92 + 8*VID_MULTITEXTURE, "Not found");
+
+	M_Print (76, 92 + 8*VID_NPOT, "NPOT textures :");
+	if (have_NPOT)
+		M_DrawYesNo (76+16*8, 92 + 8*VID_NPOT, gl_texture_NPOT.integer, (gl_tex_NPOT == !!gl_texture_NPOT.integer));
+	else
+		M_PrintWhite (76+16*8, 92 + 8*VID_NPOT, "Not found");
+
 	M_Print (76, 92 + 8*VID_PALTEX, "8 bit textures:");
 	if (have8bit)
 		M_DrawYesNo (76+16*8, 92 + 8*VID_PALTEX, vid_config_gl8bit.integer, (is8bit == !!vid_config_gl8bit.integer));
@@ -2901,9 +3036,17 @@ static void VID_MenuKey (int key)
 			//find a matching video mode for this bpp
 			vid_menunum = match_mode_to_bpp(i);
 			break;
+		case VID_MULTITEXTURE:
+			if (have_mtex)
+				Cvar_SetQuick (&gl_multitexture, gl_multitexture.integer ? "0" : "1");
+			break;
+		case VID_NPOT:
+			if (have_NPOT)
+				Cvar_SetQuick (&gl_texture_NPOT, gl_texture_NPOT.integer ? "0" : "1");
+			break;
 		case VID_PALTEX:
 			if (have8bit)
-				Cvar_SetValueQuick (&vid_config_gl8bit, !vid_config_gl8bit.integer);
+				Cvar_SetQuick (&vid_config_gl8bit, vid_config_gl8bit.integer ? "0" : "1");
 			break;
 		}
 		return;
@@ -2934,9 +3077,17 @@ static void VID_MenuKey (int key)
 			//find a matching video mode for this bpp
 			vid_menunum = match_mode_to_bpp(i);
 			break;
+		case VID_MULTITEXTURE:
+			if (have_mtex)
+				Cvar_SetQuick (&gl_multitexture, gl_multitexture.integer ? "0" : "1");
+			break;
+		case VID_NPOT:
+			if (have_NPOT)
+				Cvar_SetQuick (&gl_texture_NPOT, gl_texture_NPOT.integer ? "0" : "1");
+			break;
 		case VID_PALTEX:
 			if (have8bit)
-				Cvar_SetValueQuick (&vid_config_gl8bit, !vid_config_gl8bit.integer);
+				Cvar_SetQuick (&vid_config_gl8bit, vid_config_gl8bit.integer ? "0" : "1");
 			break;
 		}
 		return;
