@@ -1,6 +1,6 @@
 /*
  * gl_warp.c -- sky and water polygons
- * $Id: gl_warp.c 4767 2012-06-16 20:48:51Z sezero $
+ * $Id: gl_warp.c 5476 2015-10-15 20:00:35Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  *
@@ -331,7 +331,7 @@ will have them chained together.
 void EmitBothSkyLayers (msurface_t *fa)
 {
 	// 3dfx doesn't like GL_DECAL:
-	if (!is_3dfx && gl_multitexture.integer && gl_mtexable)
+	if (!is_3dfx && gl_mtexable)
 	{
 		EmitSkyPolysMulti (fa);
 		return;
@@ -367,7 +367,7 @@ void R_DrawSkyChain (msurface_t *s)
 	msurface_t	*fa;
 
 	// 3dfx doesn't like GL_DECAL:
-	if (!is_3dfx && gl_multitexture.integer && gl_mtexable)
+	if (!is_3dfx && gl_mtexable)
 	{
 		for (fa = s ; fa ; fa = fa->texturechain)
 			EmitSkyPolysMulti (fa);
