@@ -1,6 +1,6 @@
 /*
  * quakefs.c -- Hexen II filesystem
- * $Id: quakefs.c 4951 2012-09-15 08:25:19Z sezero $
+ * $Id: quakefs.c 5637 2016-06-28 14:25:47Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 2005-2012  O.Sezer <sezero@users.sourceforge.net>
@@ -430,14 +430,33 @@ map change from within CL_ParseServerData(), and the Server calls
 this upon a gamedir command from within SV_Gamedir_f().
 ================
 */
+#ifdef H2W
+static inline void set_hw_dir (void) { /* helper for FS_Gamedir () */
+	qerr_strlcpy(__thisfunc__, __LINE__, fs_gamedir_nopath, "hw",
+					sizeof(fs_gamedir_nopath));
+	FSERR_MakePath_BUF (__thisfunc__, __LINE__, FS_BASEDIR,
+				fs_gamedir, sizeof(fs_gamedir), "hw");
+	FSERR_MakePath_BUF (__thisfunc__, __LINE__, FS_USERBASE,
+				fs_userdir, sizeof(fs_userdir), "hw");
+	#ifdef SERVERONLY
+	/* change the *gamedir serverinfo properly */
+	Info_SetValueForStarKey (svs.info, "*gamedir", "hw", MAX_SERVERINFO_STRING);
+	#endif
+}
+#endif
+
 void FS_Gamedir (const char *dir)
 {
 	searchpath_t	*next;
 
-	if (strstr(dir, "..") || strstr(dir, "/") || strstr(dir, "\\") || strstr(dir, ":"))
+	if (!*dir || !strcmp(dir, ".") || strstr(dir, "..") || strstr(dir, "/") || strstr(dir, "\\") || strstr(dir, ":"))
 	{
-		Con_Printf ("gamedir should be a single directory name, not a path\n");
-		return;
+		if (!host_initialized)
+			Sys_Error ("gamedir should be a single directory name, not a path\n");
+		else {
+			Con_Printf("gamedir should be a single directory name, not a path\n");
+			return;
+		}
 	}
 
 	if (!q_strcasecmp(fs_gamedir_nopath, dir))
@@ -473,17 +492,7 @@ void FS_Gamedir (const char *dir)
 	/* that we reached here means the hw server decided to abandon
 	 * whatever the previous mod it was running and went back to
 	 * pure hw. weird.. do as he wishes anyway and adjust our variables. */
-	_do_hw:
-		qerr_strlcpy(__thisfunc__, __LINE__, fs_gamedir_nopath, "hw",
-						sizeof(fs_gamedir_nopath));
-		FSERR_MakePath_BUF (__thisfunc__, __LINE__, FS_BASEDIR,
-					fs_gamedir, sizeof(fs_gamedir), "hw");
-		FSERR_MakePath_BUF (__thisfunc__, __LINE__, FS_USERBASE,
-					fs_userdir, sizeof(fs_userdir), "hw");
-# ifdef SERVERONLY
-	/* change the *gamedir serverinfo properly */
-		Info_SetValueForStarKey (svs.info, "*gamedir", "hw", MAX_SERVERINFO_STRING);
-# endif /* HWSV */
+		set_hw_dir ();
 #else	/* hexen2 case: */
 	/* hw is reserved for hexenworld only. hexen2 shouldn't use it */
 		Con_Printf ("WARNING: Gamedir not set to hw :\n"
@@ -499,7 +508,7 @@ void FS_Gamedir (const char *dir)
 	 * actually a hypothetical case.
 	 * as for hexen2, it cannot reach here.  */
 #ifdef H2W
-		goto _do_hw;
+		set_hw_dir ();
 #endif
 		return;
 	}
@@ -511,7 +520,7 @@ void FS_Gamedir (const char *dir)
 	 * as for hexen2, it can only reach here by a silly
 	 * command line argument like -game data1, ignore it. */
 #ifdef H2W
-		goto _do_hw;
+		set_hw_dir ();
 #endif
 		return;
 	}
@@ -537,23 +546,6 @@ size_t		fs_filesize;	/* size of the last file opened through QFS */
 int		file_from_pak;	/* ZOID: global indicating that file came from a pak */
 
 
-/*
-================
-FS_filelength
-================
-*/
-long FS_filelength (FILE *f)
-{
-	long	pos, end;
-
-	pos = ftell (f);
-	fseek (f, 0, SEEK_END);
-	end = ftell (f);
-	fseek (f, pos, SEEK_SET);
-
-	return end;
-}
-
 /*
 ===========
 FS_CopyFile
@@ -619,23 +611,16 @@ int FS_WriteFileFromHandle (FILE *fromfile, const char *topath, size_t size)
 		return 1;
 	}
 
-	memset (buf, 0, sizeof(buf));
 	remaining = size;
 	while (remaining)
 	{
 		if (remaining < sizeof(buf))
 			count = remaining;
-		else
-			count = sizeof(buf);
+		else	count = sizeof(buf);
 
-		fread (buf, 1, count, fromfile);
-		err = ferror (fromfile);
-		if (err)
+		if (fread(buf, 1, count, fromfile) != count)
 			break;
-
-		fwrite (buf, 1, count, out);
-		err = ferror (out);
-		if (err)
+		if (fwrite(buf, 1, count, out) != count)
 			break;
 
 		remaining -= count;
@@ -643,7 +628,7 @@ int FS_WriteFileFromHandle (FILE *fromfile, const char *topath, size_t size)
 
 	fclose (out);
 
-	return err;
+	return (remaining == 0)? 0 : 1;
 }
 
 /*
@@ -1091,9 +1076,8 @@ static void FS_Maplist_f (void)
 	{
 		if (search->pack)
 		{
-			int	i;
-
-			for (i = 0; i < search->pack->numfiles; i++)
+			int i = 0;
+			for (; i < search->pack->numfiles; ++i)
 			{
 				if (strncmp("maps/", search->pack->files[i].name, 5) != 0)
 					continue;
@@ -1103,9 +1087,7 @@ static void FS_Maplist_f (void)
 		}
 		else
 		{
-			const char	*findname;
-
-			findname = Sys_FindFirstFile (va("%s/maps",search->filename), "*.bsp");
+			const char *findname = Sys_FindFirstFile(va("%s/maps",search->filename), "*.bsp");
 			while (findname)
 			{
 				if (processMapname(findname, prefix, preLen) < 0)
@@ -1204,6 +1186,7 @@ void FS_Init (void)
 	if (i && i < com_argc-1)
 	{
 		fs_basedir = com_argv[i+1];
+		if (!*fs_basedir) Sys_Error("Bad argument to -basedir");
 #if !DO_USERDIRS
 		host_parms->userdir = com_argv[i+1];
 #endif
@@ -1546,24 +1529,19 @@ size_t FS_fread(void *ptr, size_t size, size_t nmemb, fshandle_t *fh)
 	long bytes_read;
 	size_t nmemb_read;
 
-	if (!ptr)
-	{
+	if (!fh) {
+		errno = EBADF;
+		return 0;
+	}
+	if (!ptr) {
 		errno = EFAULT;
 		return 0;
 	}
-
-	if (!(size && nmemb))	/* not an error, just zero bytes wanted */
-	{
+	if (!size || !nmemb) {	/* no error, just zero bytes wanted */
 		errno = 0;
 		return 0;
 	}
 
-	if (!fh)
-	{
-		errno = EBADF;
-		return 0;
-	}
-
 	byte_size = nmemb * size;
 	if (byte_size > fh->length - fh->pos)	/* just read to end */
 		byte_size = fh->length - fh->pos;
@@ -1587,8 +1565,7 @@ int FS_fseek(fshandle_t *fh, long offset, int whence)
  * the quake/hexen2 file system is 32 bits, anyway. */
 	int ret;
 
-	if (!fh)
-	{
+	if (!fh) {
 		errno = EBADF;
 		return -1;
 	}
@@ -1610,8 +1587,7 @@ int FS_fseek(fshandle_t *fh, long offset, int whence)
 		return -1;
 	}
 
-	if (offset < 0)
-	{
+	if (offset < 0) {
 		errno = EINVAL;
 		return -1;
 	}
@@ -1629,31 +1605,25 @@ int FS_fseek(fshandle_t *fh, long offset, int whence)
 
 int FS_fclose(fshandle_t *fh)
 {
-	if (!fh)
-	{
+	if (!fh) {
 		errno = EBADF;
 		return -1;
 	}
-
 	return fclose(fh->file);
 }
 
 long FS_ftell(fshandle_t *fh)
 {
-	if (!fh)
-	{
+	if (!fh) {
 		errno = EBADF;
 		return -1;
 	}
-
 	return fh->pos;
 }
 
 void FS_rewind(fshandle_t *fh)
 {
-	if (!fh)
-		return;
-
+	if (!fh) return;
 	clearerr(fh->file);
 	fseek(fh->file, fh->start, SEEK_SET);
 	fh->pos = 0;
@@ -1661,6 +1631,10 @@ void FS_rewind(fshandle_t *fh)
 
 int FS_feof(fshandle_t *fh)
 {
+	if (!fh) {
+		errno = EBADF;
+		return -1;
+	}
 	if (fh->pos >= fh->length)
 		return -1;
 	return 0;
@@ -1668,9 +1642,25 @@ int FS_feof(fshandle_t *fh)
 
 int FS_ferror(fshandle_t *fh)
 {
+	if (!fh) {
+		errno = EBADF;
+		return -1;
+	}
 	return ferror(fh->file);
 }
 
+int FS_fgetc(fshandle_t *fh)
+{
+	if (!fh) {
+		errno = EBADF;
+		return EOF;
+	}
+	if (fh->pos >= fh->length)
+		return EOF;
+	fh->pos += 1;
+	return fgetc(fh->file);
+}
+
 char *FS_fgets(char *s, int size, fshandle_t *fh)
 {
 	char *ret;
@@ -1687,3 +1677,12 @@ char *FS_fgets(char *s, int size, fshandle_t *fh)
 	return ret;
 }
 
+long FS_filelength (fshandle_t *fh)
+{
+	if (!fh) {
+		errno = EBADF;
+		return -1;
+	}
+	return fh->length;
+}
+
