@@ -1,6 +1,6 @@
 /*
  * screen.c -- master for refresh, status bar, console, chat, notify, etc
- * $Id: screen.c 5160 2013-03-03 22:15:11Z sezero $
+ * $Id: screen.c 5712 2016-08-11 08:41:34Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
@@ -334,6 +334,10 @@ static float AdaptFovx (float fov_x, float width, float height)
 #if defined(PLATFORM_DOS) || defined(SVGAQUAKE)
 	if (vid.aspect > 1.10f)
 		return fov_x;		/* no fov_adapt for weird VGA modes */
+#endif
+#ifdef PLATFORM_AMIGAOS3
+	if (vid.noadapt)
+		return fov_x;		/* not for Amiga native chipset modes */
 #endif
 	if (!scr_fov_adapt.integer)
 		return fov_x;
@@ -1286,19 +1290,20 @@ void SCR_UpdateScreen (void)
 #else
 	if (scr_disabled_for_loading)
 	{
-		if (realtime - scr_disabled_time > 20)
-		{
+		if (realtime - scr_disabled_time > 25) {
 		/* this can happen with clients connected to servers
 		 * older than uHexen2-1.5.6 who don't issue an error
-		 * upon changelevel failures.
+		 * upon changelevel failures. Or, it could happen if
+		 * loading is taking a really long time.
 		 */
 			scr_disabled_for_loading = false;
 			total_loading_size = 0;
 			loading_stage = 0;
-			Con_Printf ("load failed.\n");
+			Con_Printf ("load timeout.\n");
 		}
-		else
+		else {
 			return;
+		}
 	}
 
 	if (cls.state == ca_dedicated)
