@@ -1,12 +1,12 @@
 /*
- * vid_cgx.c -- CyberGraphX video driver for Amiga
+ * vid_cgx.c -- CyberGraphX video driver for AmigaOS & variants.
  * Select window size and mode and init CGX in SOFTWARE mode.
- * $Id: vid_cgx.c 5122 2013-02-15 08:01:40Z sezero $
+ * $Id: vid_cgx.c 5702 2016-08-01 06:32:36Z bszili $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 2004-2005  Steven Atkinson <stevenaaus@yahoo.com>
- * Copyright (C) 2005-2012  O.Sezer <sezero@users.sourceforge.net>
- * Copyright (C) 2012 Szilrd Bir <col.lawrence@gmail.com>
+ * Copyright (C) 2005-2016  O.Sezer <sezero@users.sourceforge.net>
+ * Copyright (C) 2012-2016  Szilrd Bir <col.lawrence@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,12 @@
 #include <proto/graphics.h>
 #include <proto/cybergraphics.h>
 
+/* WriteLUTPixelArray not included in vbcc_target_m68k-amigaos.lha */
+#if defined(__VBCC__) && defined(__M68K__) && !defined(WriteLUTPixelArray)
+ULONG __WriteLUTPixelArray(__reg("a6") void *, __reg("a0") APTR srcRect, __reg("d0") UWORD SrcX, __reg("d1") UWORD SrcY, __reg("d2") UWORD SrcMod, __reg("a1") struct RastPort * a1arg, __reg("a2") APTR a2arg, __reg("d3") UWORD DestX, __reg("d4") UWORD DestY, __reg("d5") UWORD SizeX, __reg("d6") UWORD SizeY, __reg("d7") UBYTE CTFormat)="\tjsr\t-198(a6)";
+#define WriteLUTPixelArray(srcRect, SrcX, SrcY, SrcMod, a1arg, a2arg, DestX, DestY, SizeX, SizeY, CTFormat) __WriteLUTPixelArray(CyberGfxBase, (srcRect), (SrcX), (SrcY), (SrcMod), (a1arg), (a2arg), (DestX), (DestY), (SizeX), (SizeY), (CTFormat))
+#endif
+
 #include "quakedef.h"
 #include "d_local.h"
 #include "cfgfile.h"
@@ -41,19 +47,23 @@
 
 #define MIN_WIDTH		320
 #define MIN_HEIGHT		240
-#define MAX_DESC		13
+#define MAX_DESC		33
 
 /* - CGX ----------------------------------- */
 
-struct Window *window = NULL; // used by in_amiga.c
+struct Window *window = NULL; /* used by in_amiga.c */
 static struct Screen *screen = NULL;
-static char pal[256 * 4];
-static char *buffer = NULL;
+static unsigned char ppal[256 * 4];
+static pixel_t *buffer = NULL;
 static byte *directbitmap = NULL;
+#ifdef PLATFORM_AMIGAOS3
+struct Library *CyberGfxBase = NULL;
+#endif
 
 /* ----------------------------------------- */
 
-unsigned char	vid_curpal[256*3];
+static unsigned char	vid_curpal[256*3];	/* save for mode changes */
+
 unsigned short	d_8to16table[256];
 unsigned int	d_8to24table[256];
 
@@ -69,7 +79,7 @@ static int	num_wmodes;
 static int	*nummodes;
 
 static qboolean	vid_menu_fs;
-static qboolean	fs_toggle_works = false;
+//static qboolean	fs_toggle_works = false;
 
 viddef_t	vid;		// global video state
 // cvar vid_mode must be set before calling VID_SetMode, VID_ChangeVideoMode or VID_Restart_f
@@ -78,7 +88,12 @@ static cvar_t	vid_config_glx = {"vid_config_glx", "640", CVAR_ARCHIVE};
 static cvar_t	vid_config_gly = {"vid_config_gly", "480", CVAR_ARCHIVE};
 static cvar_t	vid_config_swx = {"vid_config_swx", "320", CVAR_ARCHIVE};
 static cvar_t	vid_config_swy = {"vid_config_swy", "240", CVAR_ARCHIVE};
+#ifdef PLATFORM_AMIGAOS3
+static cvar_t	vid_config_fscr= {"vid_config_fscr", "1", CVAR_ARCHIVE};
+#else
 static cvar_t	vid_config_fscr= {"vid_config_fscr", "0", CVAR_ARCHIVE};
+#endif
+static cvar_t	vid_config_mon = {"vid_config_mon", "0", CVAR_ARCHIVE};
 
 static cvar_t	vid_showload = {"vid_showload", "1", CVAR_NONE};
 
@@ -101,6 +116,10 @@ typedef struct {
 	int			fullscreen;
 	int			bpp;
 /*	int			halfscreen;*/
+	ULONG		modeid;
+#ifdef PLATFORM_AMIGAOS3
+	qboolean	noadapt;
+#endif
 	char		modedesc[MAX_DESC];
 } vmode_t;
 
@@ -130,7 +149,7 @@ static vmode_t	fmodelist[MAX_MODE_LIST+1];	// list of enumerated fullscreen mode
 static vmode_t	wmodelist[MAX_STDMODES +1];	// list of standart 4:3 windowed modes
 static vmode_t	*modelist;	// modelist in use, points to one of the above lists
 
-static int VID_SetMode (int modenum, unsigned char *palette);
+static qboolean VID_SetMode (int modenum, const unsigned char *palette);
 
 static void VID_MenuDraw (void);
 static void VID_MenuKey (int key);
@@ -142,6 +161,13 @@ static void VID_MenuKey (int key);
 #	define WM_TITLEBAR_TEXT	"Hexen II"
 #endif
 
+#if !(defined(__AROS__) || defined(__MORPHOS__))
+#if !defined(IPTR) /* SDI headers may define it */
+typedef ULONG IPTR;
+#define IPTR IPTR
+#endif
+#endif
+
 //====================================
 
 /*
@@ -199,7 +225,7 @@ static qboolean VID_AllocBuffers (int width, int height)
 
 	vid_surfcache = (byte *)d_pzbuffer +
 			width * height * sizeof (*d_pzbuffer);
-	
+
 	return true;
 }
 
@@ -233,6 +259,20 @@ static qboolean VID_CheckAdequateMem (int width, int height)
 }
 
 //------------------------------------
+static int sort_modes (const void *arg1, const void *arg2)
+{
+	const vmode_t *a1, *a2;
+	a1 = (const vmode_t *) arg1;
+	a2 = (const vmode_t *) arg2;
+
+	if (a1->width == a2->width)
+		return a1->height - a2->height;	// lowres-to-highres
+	//	return a2->height - a1->height;	// highres-to-lowres
+	else
+		return a1->width - a2->width;	// lowres-to-highres
+	//	return a2->width - a1->width;	// highres-to-lowres
+}
+
 static void VID_PrepareModes (void)
 {
 	qboolean	have_mem;
@@ -240,6 +280,11 @@ static void VID_PrepareModes (void)
 	unsigned int i;
 	APTR handle;
 	struct DimensionInfo diminfo;
+#ifdef PLATFORM_AMIGAOS3
+	ULONG monitorid;
+	struct DisplayInfo dispinfo;
+	struct NameInfo nameinfo;
+#endif
 
 	num_fmodes = 0;
 	num_wmodes = 0;
@@ -254,6 +299,10 @@ static void VID_PrepareModes (void)
 		wmodelist[num_wmodes].height = std_modes[i].height;
 		wmodelist[num_wmodes].fullscreen = 0;
 		wmodelist[num_wmodes].bpp = 8;
+		wmodelist[num_wmodes].modeid = INVALID_ID;
+#ifdef PLATFORM_AMIGAOS3
+		wmodelist[num_wmodes].noadapt = false;
+#endif
 		q_snprintf (wmodelist[num_wmodes].modedesc, MAX_DESC,
 				"%d x %d", std_modes[i].width, std_modes[i].height);
 		num_wmodes++;
@@ -263,26 +312,69 @@ static void VID_PrepareModes (void)
 	id = INVALID_ID;
 	while((id = NextDisplayInfo(id)) != INVALID_ID)
 	{
-#ifdef __AMIGA__
-		if (!IsCyberModeID(id))
+#ifdef PLATFORM_AMIGAOS3
+		//if (!IsCyberModeID(id))	continue;
+		monitorid = id & MONITOR_ID_MASK;
+		if (monitorid == DEFAULT_MONITOR_ID || monitorid == A2024_MONITOR_ID)
 			continue;
 #endif
+
 		handle = FindDisplayInfo(id);
+		if (!handle)
+			continue;
 
-		if (handle)
-		{
-			if (!GetDisplayInfoData(handle, (UBYTE *)&diminfo, sizeof(diminfo), DTAG_DIMS, 0))
-				continue;
+#ifdef PLATFORM_AMIGAOS3
+		if (!GetDisplayInfoData(handle, (UBYTE *)&dispinfo, sizeof(dispinfo), DTAG_DISP, 0))
+			continue;
+		// this is a good way to filter out HAM, EHB, DPF modes
+		if (!GetDisplayInfoData(handle, (UBYTE *)&nameinfo, sizeof(nameinfo), DTAG_NAME, 0))
+			continue;
+		//Con_SafePrintf ("modeid %08x name %s\n", id, nameinfo.Name);
+#endif
 
-			fmodelist[num_fmodes].width = diminfo.Nominal.MaxX + 1;
-			fmodelist[num_fmodes].height = diminfo.Nominal.MaxY + 1;
-			fmodelist[num_fmodes].fullscreen = 1;
-			fmodelist[num_fmodes].bpp = 8; // diminfo.MaxDepth
-			q_snprintf (fmodelist[num_fmodes].modedesc, MAX_DESC, "%d x %d", (fmodelist[num_fmodes].width), (fmodelist[num_fmodes].height));
-			//Con_SafePrintf ("fmodelist[%d].modedesc = %s maxdepth %d\n", num_fmodes, fmodelist[num_fmodes].modedesc, diminfo.MaxDepth);
+		if (!GetDisplayInfoData(handle, (UBYTE *)&diminfo, sizeof(diminfo), DTAG_DIMS, 0))
+			continue;
 
-			num_fmodes++;
-		}
+#ifdef __AROS__
+		if (diminfo.MaxDepth != 24 || diminfo.Nominal.MaxX + 1 < MIN_WIDTH)	continue;
+#else
+		if (diminfo.MaxDepth != 8 || diminfo.Nominal.MaxX + 1 < MIN_WIDTH)	continue;
+#endif
+
+		fmodelist[num_fmodes].width = diminfo.Nominal.MaxX + 1;
+		fmodelist[num_fmodes].height = diminfo.Nominal.MaxY + 1;
+		fmodelist[num_fmodes].fullscreen = 1;
+		fmodelist[num_fmodes].bpp = 8; // diminfo.MaxDepth
+		fmodelist[num_fmodes].modeid = id;
+		q_snprintf (fmodelist[num_fmodes].modedesc, MAX_DESC, "%d x %d", (fmodelist[num_fmodes].width), (fmodelist[num_fmodes].height));
+#ifdef PLATFORM_AMIGAOS3
+		if (dispinfo.PropertyFlags & DIPF_IS_LACE)
+			q_strlcat(fmodelist[num_fmodes].modedesc, "i", MAX_DESC);
+		if (monitorid == PAL_MONITOR_ID)
+			q_strlcat(fmodelist[num_fmodes].modedesc, " PAL", MAX_DESC);
+		else if (monitorid == NTSC_MONITOR_ID)
+			q_strlcat(fmodelist[num_fmodes].modedesc, " NTSC", MAX_DESC);
+		else if (monitorid == DBLPAL_MONITOR_ID)
+			q_strlcat(fmodelist[num_fmodes].modedesc, " DblPAL", MAX_DESC);
+		else if (monitorid == DBLNTSC_MONITOR_ID)
+			q_strlcat(fmodelist[num_fmodes].modedesc, " DblNTSC", MAX_DESC);
+		else if (monitorid == EURO36_MONITOR_ID)
+			q_strlcat(fmodelist[num_fmodes].modedesc, " Euro36", MAX_DESC);
+		else if (monitorid == EURO72_MONITOR_ID)
+			q_strlcat(fmodelist[num_fmodes].modedesc, " Euro72", MAX_DESC);
+		else if (monitorid == SUPER72_MONITOR_ID)
+			q_strlcat(fmodelist[num_fmodes].modedesc, " Super72", MAX_DESC);
+		else if (monitorid == VGA_MONITOR_ID)
+			q_strlcat(fmodelist[num_fmodes].modedesc, " VGA", MAX_DESC);
+		else
+			q_strlcat(fmodelist[num_fmodes].modedesc, " RTG", MAX_DESC);
+
+		fmodelist[num_fmodes].noadapt = (!CyberGfxBase || !IsCyberModeID(id));
+#endif
+		//Con_SafePrintf ("fmodelist[%d].modedesc = %s maxdepth %d id %08x\n", num_fmodes, fmodelist[num_fmodes].modedesc, diminfo.MaxDepth, id);
+
+		if (++num_fmodes == MAX_MODE_LIST)
+			break;
 	}
 
 	if (num_fmodes == 0)
@@ -298,11 +390,20 @@ static void VID_PrepareModes (void)
 		return;
 	}
 
+	if (num_fmodes > 1)
+		qsort(fmodelist, num_fmodes, sizeof(vmode_t), sort_modes);
 	nummodes = &num_fmodes;
 	modelist = fmodelist;
 
-	vid_maxwidth = fmodelist[num_fmodes-1].width;
-	vid_maxheight = fmodelist[num_fmodes-1].height;
+	/*vid_maxwidth = fmodelist[num_fmodes-1].width;
+	vid_maxheight = fmodelist[num_fmodes-1].height;*/
+	if ((screen = LockPubScreen(NULL)))
+	{
+		vid_maxwidth = screen->Width;
+		vid_maxheight = screen->Height;
+		UnlockPubScreen(NULL, screen);
+		screen = NULL;
+	}
 
 	// see if we have 320x240 among the available modes
 	for (i = 0; i < num_fmodes; i++)
@@ -360,34 +461,80 @@ static void VID_NumModes_f (void)
 	Con_Printf ("%d video modes in current list\n", *nummodes);
 }
 
-static void VID_ShowInfo_f (void)
-{
-	// TODO: implement me
-}
-
 //====================================
 
 
-static int VID_SetMode (int modenum, unsigned char *palette)
+static void VID_DestroyWindow (void)
+{
+	if (window)
+	{
+		CloseWindow(window);
+		window = NULL;
+	}
+
+	if (buffer)
+	{
+		FreeVec(buffer);
+		buffer = NULL;
+	}
+
+	/*if (pointermem)
+	{
+		FreeVec(pointermem);
+		pointermem = NULL;
+	}*/
+
+	if (screen)
+	{
+		CloseScreen(screen);
+		screen = NULL;
+	}
+}
+
+static qboolean VID_SetMode (int modenum, const unsigned char *palette)
 {
 	ULONG flags;
+	qboolean eightbit = false;
 
 	in_mode_set = true;
 
-	VID_Shutdown();
+	VID_DestroyWindow ();
+
+	if (!vid_config_fscr.integer && (screen = LockPubScreen(NULL)))
+	{
+		eightbit = (GetBitMapAttr(screen->RastPort.BitMap, BMA_DEPTH) <= 8);
+		UnlockPubScreen(NULL, screen);
+		screen = NULL;
+
+		if (eightbit)
+		{
+			struct EasyStruct es;
+
+			es.es_StructSize = sizeof(es);
+			es.es_Flags = 0;
+			es.es_Title = (STRPTR) ENGINE_NAME;
+			es.es_TextFormat = (STRPTR) "Windowed mode requires 15 bit or higher color depth screen\nFalling back to fullscreen.";
+			es.es_GadgetFormat = (STRPTR) "OK";
+
+			EasyRequest(0, &es, 0, 0);
+
+			Cvar_SetQuick (&vid_config_fscr, "1");
+		}
+	}
 
 	flags = WFLG_ACTIVATE | WFLG_RMBTRAP;
 
 	if (vid_config_fscr.integer)
 	{
-	    ULONG ModeID;
+		ULONG ModeID;
 
-	    ModeID = BestCModeIDTags(
+		/*ModeID = BestCModeIDTags(
 			CYBRBIDTG_Depth, 8,
 			CYBRBIDTG_NominalWidth, modelist[modenum].width,
 			CYBRBIDTG_NominalHeight, modelist[modenum].height,
-			TAG_DONE);
-	
+			TAG_DONE);*/
+		ModeID = modelist[modenum].modeid;
+
 		screen = OpenScreenTags(0,
 			ModeID != INVALID_ID ? SA_DisplayID : TAG_IGNORE, ModeID,
 			SA_Width, modelist[modenum].width,
@@ -403,7 +550,9 @@ static int VID_SetMode (int modenum, unsigned char *palette)
 	}
 	else
 	{
-		Cvar_SetValueQuick (&vid_config_fscr, 0);
+		if (eightbit) /* shouldn't happen, but.. */
+			Sys_Error("failed OpenScreen ()");
+		Cvar_SetQuick (&vid_config_fscr, "0");
 		flags |=  WFLG_DRAGBAR | WFLG_DEPTHGADGET | WFLG_CLOSEGADGET;
 	}
 
@@ -419,12 +568,10 @@ static int VID_SetMode (int modenum, unsigned char *palette)
 	if (window)
 	{
 		/*pointermem = AllocVec(256, MEMF_ANY | MEMF_CLEAR);
-		
-		if (pointermem)
-		{*/
+		if (pointermem) {*/
 			vid.height = vid.conheight = modelist[modenum].height;
 			vid.rowbytes = vid.conrowbytes = vid.width = vid.conwidth = modelist[modenum].width;
-			buffer = AllocVec(vid.width * vid.height, MEMF_ANY);
+			buffer = (pixel_t *) AllocVec(vid.width * vid.height, MEMF_ANY);
 
 			if (buffer)
 			{
@@ -441,6 +588,8 @@ static int VID_SetMode (int modenum, unsigned char *palette)
 					modestate = (screen) ? MS_FULLDIB : MS_WINDOWED;
 					Cvar_SetValueQuick (&vid_config_swx, modelist[vid_modenum].width);
 					Cvar_SetValueQuick (&vid_config_swy, modelist[vid_modenum].height);
+					if (screen)
+						Cvar_SetValueQuick (&vid_config_mon, (float)(modelist[vid_modenum].modeid & MONITOR_ID_MASK));
 
 					//IN_HideMouse ();
 
@@ -450,20 +599,29 @@ static int VID_SetMode (int modenum, unsigned char *palette)
 
 					Con_SafePrintf ("Video Mode: %ux%ux%d\n", vid.width, vid.height, modelist[modenum].bpp);
 
+					#ifdef PLATFORM_AMIGAOS3
+					vid.noadapt = modelist[modenum].noadapt;
+					#endif
 					in_mode_set = false;
 					vid.recalc_refdef = 1;
 
 					return true;
 				}
 				FreeVec(buffer);
+				buffer = NULL;
 			}
-			/*FreeVec(pointermem);
+		/*	FreeVec(pointermem);
+			pointermem = NULL;
 		}*/
 		CloseWindow(window);
+		window = NULL;
 	}
 
 	if (screen)
+	{
 		CloseScreen(screen);
+		screen = NULL;
+	}
 
 	in_mode_set = false;
 	return false;
@@ -472,7 +630,7 @@ static int VID_SetMode (int modenum, unsigned char *palette)
 
 static void VID_ChangeVideoMode (int newmode)
 {
-	int		status, temp;
+	int		temp;
 
 	temp = scr_disabled_for_loading;
 	scr_disabled_for_loading = true;
@@ -480,8 +638,7 @@ static void VID_ChangeVideoMode (int newmode)
 	BGM_Pause ();
 	S_ClearBuffer ();
 
-	status = VID_SetMode (newmode, vid_curpal);
-	if (!status)
+	if (!VID_SetMode (newmode, vid_curpal))
 	{
 		if (vid_modenum == newmode)
 			Sys_Error ("Couldn't set video mode");
@@ -489,8 +646,7 @@ static void VID_ChangeVideoMode (int newmode)
 		// failed setting mode, probably due to insufficient
 		// memory. go back to previous mode.
 		Cvar_SetValueQuick (&vid_mode, vid_modenum);
-		status = VID_SetMode (vid_modenum, vid_curpal);
-		if (!status)
+		if (!VID_SetMode (vid_modenum, vid_curpal))
 			Sys_Error ("Couldn't set video mode");
 	}
 
@@ -521,69 +677,82 @@ void VID_UnlockBuffer(void)
 }
 
 
-void VID_SetPalette(unsigned char *palette)
+void VID_SetPalette(const unsigned char *palette)
 {
+	const unsigned char *p;
+	unsigned char *pp;
 	int i;
 
-	ULONG spal[1 + (256 * 3) + 1];
-
 	palette_changed = true;
 
-	memcpy (vid_curpal, palette, sizeof(vid_curpal));
+	if (palette != vid_curpal)
+		memcpy(vid_curpal, palette, sizeof(vid_curpal));
 
 	if (screen)
 	{
-		spal[0] = 256 << 16;
+		ULONG spal[1 + (256 * 3) + 1];
+		ULONG *sp = spal;
 
-		for (i = 0; i < 256; i++)
+		*sp++ = 256 << 16;
+
+		for (i = 0, p = palette; i < 256; i++)
 		{
-			spal[1 + (i * 3)] = ((unsigned int)palette[i * 3]) << 24;
-			spal[2 + (i * 3)] = ((unsigned int)palette[i * 3 + 1]) << 24;
-			spal[3 + (i * 3)] = ((unsigned int)palette[i * 3 + 2]) << 24;
+			*sp++ = ((ULONG) *p++) << 24;
+			*sp++ = ((ULONG) *p++) << 24;
+			*sp++ = ((ULONG) *p++) << 24;
 		}
 
-		spal[1 + (3 * 256)] = 0;
+		*sp = 0;
 
 		LoadRGB32(&screen->ViewPort, spal);
 	}
 
-	for (i = 0; i < 256; i++)
+	for (i = 0, p = palette, pp = ppal; i < 256; i++)
 	{
-		if (host_byteorder == BIG_ENDIAN)
+		if (host_bigendian)
 		{
-			pal[i * 4] = 0;
-			pal[i * 4 + 1] = palette[i * 3 + 0];
-			pal[i * 4 + 2] = palette[i * 3 + 1];
-			pal[i * 4 + 3] = palette[i * 3 + 2];
+			*pp++ = 0;
+			*pp++ = *p++;
+			*pp++ = *p++;
+			*pp++ = *p++;
 		}
 		else
 		{
-			pal[i * 4] = palette[i * 3 + 2];
-			pal[i * 4 + 1] = palette[i * 3 + 1];
-			pal[i * 4 + 2] = palette[i * 3 + 0];
-			pal[i * 4 + 3] = 0;
+			*pp++ = p[2];
+			*pp++ = p[1];
+			*pp++ = p[0];
+			*pp++ = 0;
+			p += 3;
 		}
 	}
 }
 
-void VID_ShiftPalette(unsigned char *palette)
+void VID_ShiftPalette(const unsigned char *palette)
 {
 	VID_SetPalette(palette);
 }
 
-void VID_Init (unsigned char *palette)
+void VID_Init (const unsigned char *palette)
 {
-	int		width, height, i, temp;
+	int		width, height, i, temp, monitor;
 	const char	*read_vars[] = {
 				"vid_config_fscr",
+				"vid_config_mon",
 				"vid_config_swx",
 				"vid_config_swy" };
 #define num_readvars	( sizeof(read_vars)/sizeof(read_vars[0]) )
 
+#ifdef PLATFORM_AMIGAOS3
+	CyberGfxBase = OpenLibrary("cybergraphics.library", 0);
+	/*if (!CyberGfxBase)
+		Sys_Error ("Cannot open cybergraphics.library!");*/
+#endif
+
 	temp = scr_disabled_for_loading;
 	scr_disabled_for_loading = true;
 
 	Cvar_RegisterVariable (&vid_config_fscr);
+	Cvar_RegisterVariable (&vid_config_mon);
 	Cvar_RegisterVariable (&vid_config_swy);
 	Cvar_RegisterVariable (&vid_config_swx);
 	Cvar_RegisterVariable (&vid_config_gly);
@@ -592,7 +761,6 @@ void VID_Init (unsigned char *palette)
 	Cvar_RegisterVariable (&_enable_mouse);
 	Cvar_RegisterVariable (&vid_showload);
 
-	Cmd_AddCommand ("vid_showinfo", VID_ShowInfo_f);
 	Cmd_AddCommand ("vid_listmodes", VID_ListModes_f);
 	Cmd_AddCommand ("vid_nummodes", VID_NumModes_f);
 	Cmd_AddCommand ("vid_restart", VID_Restart_f);
@@ -624,6 +792,7 @@ void VID_Init (unsigned char *palette)
 
 	width = vid_config_swx.integer;
 	height = vid_config_swy.integer;
+	monitor = vid_config_mon.integer;
 
 	// user is always right ...
 	i = COM_CheckParm("-width");
@@ -639,17 +808,42 @@ void VID_Init (unsigned char *palette)
 			height = 3 * width / 4;
 	}
 
+	// pick the appropriate list
+	if (vid_config_fscr.integer)
+	{
+		modelist = fmodelist;
+		nummodes = &num_fmodes;
+	}
+	else
+	{
+		modelist = wmodelist;
+		nummodes = &num_wmodes;
+	}
+
 	// user requested a mode either from the config or from the
 	// command line
 	// scan existing modes to see if this is already available
 	// if not, add this as the last "valid" video mode and set
 	// vid_mode to it only if it doesn't go beyond vid_maxwidth
 	i = 0;
-	while (i < *nummodes)
+	if (vid_config_fscr.integer)
 	{
-		if (modelist[i].width == width && modelist[i].height == height)
-			break;
-		i++;
+		while (i < *nummodes)
+		{
+			if (modelist[i].width == width && modelist[i].height == height && (modelist[i].modeid & MONITOR_ID_MASK) == monitor)
+				break;
+			i++;
+		}
+	}
+	if (i == 0 || i == *nummodes)
+	{
+		i = 0;
+		while (i < *nummodes)
+		{
+			if (modelist[i].width == width && modelist[i].height == height)
+				break;
+			i++;
+		}
 	}
 	if (i < *nummodes)
 	{
@@ -663,6 +857,10 @@ void VID_Init (unsigned char *palette)
 		modelist[*nummodes].height = height;
 		modelist[*nummodes].fullscreen = 1;
 		modelist[*nummodes].bpp = 8;
+		modelist[*nummodes].modeid = INVALID_ID;
+#ifdef PLATFORM_AMIGAOS3
+		modelist[*nummodes].noadapt = false;
+#endif
 		q_snprintf (modelist[*nummodes].modedesc, MAX_DESC, "%d x %d (user mode)", width, height);
 		Cvar_SetValueQuick (&vid_mode, *nummodes);
 		(*nummodes)++;
@@ -677,8 +875,7 @@ void VID_Init (unsigned char *palette)
 	vid.colormap = host_colormap;
 	vid.fullbright = 256 - LittleLong (*((int *)vid.colormap + 2048));
 
-	i = VID_SetMode(vid_mode.integer, palette);
-	if ( !i )
+	if (!VID_SetMode (vid_mode.integer, palette))
 	{
 		if (vid_mode.integer == vid_default)
 			Sys_Error ("Couldn't set video mode");
@@ -688,8 +885,7 @@ void VID_Init (unsigned char *palette)
 				"Trying the default mode\n", vid_mode.integer);
 		//Cvar_SetQuick (&vid_config_fscr, "0");
 		Cvar_SetValueQuick (&vid_mode, vid_default);
-		i = VID_SetMode(vid_default, palette);
-		if ( !i )
+		if (!VID_SetMode (vid_default, palette))
 			Sys_Error ("Couldn't set video mode");
 	}
 
@@ -706,29 +902,15 @@ void VID_Init (unsigned char *palette)
 
 void VID_Shutdown (void)
 {
-	if (window)
-	{
-		CloseWindow(window);
-		window = NULL;
-	}
-	
-	if (buffer)
-	{
-		FreeVec(buffer);
-		buffer = NULL;
-	}
+	VID_DestroyWindow ();
 
-	/*if (pointermem)
+#ifdef PLATFORM_AMIGAOS3
+	if (CyberGfxBase)
 	{
-		FreeVec(pointermem);
-		pointermem = NULL;
-	}*/
-
-	if (screen)
-	{
-		CloseScreen(screen);
-		screen = NULL;
+		CloseLibrary(CyberGfxBase);
+		CyberGfxBase = NULL;
 	}
+#endif
 }
 
 
@@ -736,6 +918,19 @@ static void FlipScreen (vrect_t *rects)
 {
 	while (rects)
 	{
+#ifdef PLATFORM_AMIGAOS3
+		if (!CyberGfxBase)
+		{
+			WriteChunkyPixels(window->RPort,
+							rects->x,
+							rects->y,
+							rects->width,
+							rects->height,
+							vid.buffer,
+							vid.rowbytes);
+		}
+		else
+#endif
 		if (screen)
 		{
 			WritePixelArray(vid.buffer,
@@ -755,7 +950,7 @@ static void FlipScreen (vrect_t *rects)
 							rects->x,
 							rects->y,
 							vid.rowbytes,
-							window->RPort, pal,
+							window->RPort, ppal,
 							window->BorderLeft + rects->x,
 							window->BorderTop + rects->y,
 							rects->width,
@@ -826,7 +1021,7 @@ void D_EndDirectRect (int x, int y, int width, int height)
 	}
 	else
 	{
-		WriteLUTPixelArray(directbitmap, 0, 0, width, window->RPort, pal, window->BorderLeft + x, window->BorderTop + y, width, height, CTABFMT_XRGB8);
+		WriteLUTPixelArray(directbitmap, 0, 0, width, window->RPort, ppal, window->BorderLeft + x, window->BorderTop + y, width, height, CTABFMT_XRGB8);
 	}
 
 	directbitmap = NULL;
@@ -837,58 +1032,8 @@ void D_EndDirectRect (int x, int y, int width, int height)
 void D_ShowLoadingSize (void)
 {
 #if defined(DRAW_PROGRESSBARS)
-	static int prev_perc;
-	int		cur_perc;
-	vrect_t		rect;
-	viddef_t	save_vid;	// global video state
-
-	if (!vid_showload.integer)
-		return;
-
-	if (!window)
-		return;
-
-	cur_perc = loading_stage * 100;
-	if (total_loading_size)
-		cur_perc += current_loading_size * 100 / total_loading_size;
-	if (cur_perc == prev_perc)
-		return;
-	prev_perc = cur_perc;
-
-	save_vid = vid;
-	if (vid.numpages == 1)
-	{
-		VID_LockBuffer ();
-
-		if (!vid.direct)
-			Sys_Error ("NULL vid.direct pointer");
-
-		vid.buffer = vid.direct;
-
-		SCR_DrawLoading();
-
-		VID_UnlockBuffer ();
-
-		rect.x = 0;
-		rect.y = 0;
-		rect.width = vid.width;
-		rect.height = 112;
-		rect.pnext = NULL;
-
-		FlipScreen (&rect);
-	}
-	else
-	{
-
-		vid.buffer = (byte *)screen->pixels;
-		vid.rowbytes = screen->pitch;
-
-		SCR_DrawLoading();
-
-	}
-
-	vid = save_vid;
-#endif	/* DRAW_PROGRESSBARS */
+#error NOT IMPLEMENTED
+#endif
 }
 #endif
 
@@ -926,7 +1071,7 @@ Handles switching between fullscreen/windowed modes
 and brings the mouse to a proper state afterwards
 ================
 */
-extern qboolean menu_disabled_mouse;
+//extern qboolean menu_disabled_mouse;
 void VID_ToggleFullscreen(void)
 {
 	// implement this...
@@ -939,6 +1084,7 @@ void VID_ToggleFullscreen(void)
 
 static int	vid_menunum;
 static int	vid_cursor;
+static vmode_t	*vid_menulist;	// this changes when vid_menu_fs changes
 static qboolean	want_fstoggle, need_apply;
 static qboolean	vid_menu_firsttime = true;
 
@@ -982,6 +1128,7 @@ static void VID_MenuDraw (void)
 	{	// settings for entering the menu first time
 		vid_menunum = vid_modenum;
 		vid_menu_fs = (modestate != MS_WINDOWED);
+		vid_menulist = (modestate == MS_WINDOWED) ? wmodelist : fmodelist;
 		vid_cursor = (num_fmodes) ? 0 : VID_RESOLUTION;
 		vid_menu_firsttime = false;
 	}
@@ -995,9 +1142,9 @@ static void VID_MenuDraw (void)
 
 	M_Print (76, 92 + 8*VID_RESOLUTION, "Resolution: ");
 	if (vid_menunum == vid_modenum)
-		M_PrintWhite (76+12*8, 92 + 8*VID_RESOLUTION, modelist[vid_menunum].modedesc);
+		M_PrintWhite (76+12*8, 92 + 8*VID_RESOLUTION, vid_menulist[vid_menunum].modedesc);
 	else
-		M_Print (76+12*8, 92 + 8*VID_RESOLUTION, modelist[vid_menunum].modedesc);
+		M_Print (76+12*8, 92 + 8*VID_RESOLUTION, vid_menulist[vid_menunum].modedesc);
 
 	if (need_apply)
 	{
@@ -1008,6 +1155,26 @@ static void VID_MenuDraw (void)
 	M_DrawCharacter (64, 92 + vid_cursor*8, 12+((int)(realtime*4)&1));
 }
 
+static int match_windowed_fullscr_modes (void)
+{
+	int	l;
+	vmode_t	*tmplist;
+	int	*tmpcount;
+
+	// choose the new mode
+	tmplist = (vid_menu_fs) ? fmodelist : wmodelist;
+	tmpcount = (vid_menu_fs) ? &num_fmodes : &num_wmodes;
+	for (l = 0; l < *tmpcount; l++)
+	{
+		if (tmplist[l].width == vid_menulist[vid_menunum].width &&
+		    tmplist[l].height == vid_menulist[vid_menunum].height)
+		{
+			return l;
+		}
+	}
+	return 0;
+}
+
 /*
 ================
 VID_MenuKey
@@ -1015,6 +1182,8 @@ VID_MenuKey
 */
 static void VID_MenuKey (int key)
 {
+	int	*tmpnum;
+
 	switch (key)
 	{
 	case K_ESCAPE:
@@ -1028,6 +1197,7 @@ static void VID_MenuKey (int key)
 		case VID_RESET:
 			vid_menu_fs = (modestate != MS_WINDOWED);
 			vid_menunum = vid_modenum;
+			vid_menulist = (modestate == MS_WINDOWED) ? wmodelist : fmodelist;
 			vid_cursor = (num_fmodes) ? 0 : VID_RESOLUTION;
 			break;
 		case VID_APPLY:
@@ -1035,6 +1205,8 @@ static void VID_MenuKey (int key)
 			{
 				Cvar_SetValueQuick(&vid_mode, vid_menunum);
 				Cvar_SetValueQuick(&vid_config_fscr, vid_menu_fs);
+				modelist = (vid_menu_fs) ? fmodelist : wmodelist;
+				nummodes = (vid_menu_fs) ? &num_fmodes : &num_wmodes;
 				VID_Restart_f();
 			}
 			vid_cursor = (num_fmodes) ? 0 : VID_RESOLUTION;
@@ -1047,8 +1219,10 @@ static void VID_MenuKey (int key)
 		{
 		case VID_FULLSCREEN:
 			vid_menu_fs = !vid_menu_fs;
-			if (fs_toggle_works)
-				VID_ToggleFullscreen();
+			vid_menunum = match_windowed_fullscr_modes();
+			vid_menulist = (vid_menu_fs) ? fmodelist : wmodelist;
+			/*if (fs_toggle_works)
+				VID_ToggleFullscreen();*/
 			break;
 		case VID_RESOLUTION:
 			S_LocalSound ("raven/menu1.wav");
@@ -1064,14 +1238,19 @@ static void VID_MenuKey (int key)
 		{
 		case VID_FULLSCREEN:
 			vid_menu_fs = !vid_menu_fs;
-			if (fs_toggle_works)
-				VID_ToggleFullscreen();
+			vid_menunum = match_windowed_fullscr_modes();
+			vid_menulist = (vid_menu_fs) ? fmodelist : wmodelist;
+			/*if (fs_toggle_works)
+				VID_ToggleFullscreen();*/
 			break;
 		case VID_RESOLUTION:
 			S_LocalSound ("raven/menu1.wav");
+			tmpnum = (vid_menu_fs) ? &num_fmodes : &num_wmodes;
 			vid_menunum++;
-			if (vid_menunum >= *nummodes)
-				vid_menunum = *nummodes - 1;
+			/*if (vid_menunum >= *nummodes)
+				vid_menunum = *nummodes - 1;*/
+			if (vid_menunum >= *tmpnum)
+				vid_menunum--;
 			break;
 		}
 		return;
