@@ -1,7 +1,7 @@
 /*
  * vid_mgl4.c -- Win32 video driver using SciTech MGL-4.05
  * This is X86-WIN32 only: No WIN64 because no 64 bit MGL.
- * $Id: vid_mgl4.c 5004 2012-11-05 16:38:26Z sezero $
+ * $Id: vid_mgl4.c 5676 2016-07-20 12:56:03Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
@@ -148,7 +148,7 @@ static byte		*vid_surfcache;
 static int	vid_surfcachesize;
 static int	VID_highhunkmark;
 
-unsigned char	vid_curpal[256*3];
+static unsigned char	vid_curpal[256*3];	/* save for mode changes */
 
 unsigned short	d_8to16table[256];
 unsigned int	d_8to24table[256];
@@ -204,7 +204,7 @@ static byte	backingbuf[48*48];
 static void VID_MenuDraw (void);
 static void VID_MenuKey (int key);
 
-static int VID_SetMode (int modenum, unsigned char *palette);
+static qboolean VID_SetMode (int modenum, const unsigned char *palette);
 static void AppActivate(BOOL fActive, BOOL minimize);
 static LRESULT WINAPI MainWndProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
 
@@ -330,7 +330,6 @@ VID_Windowed_f
 */
 static void VID_Windowed_f (void)
 {
-
 	VID_SetMode (vid_windowed_mode.integer, vid_curpal);
 }
 
@@ -342,7 +341,6 @@ VID_Fullscreen_f
 */
 static void VID_Fullscreen_f (void)
 {
-
 	VID_SetMode (vid_fullscreen_mode.integer, vid_curpal);
 }
 
@@ -1603,7 +1601,7 @@ static void VID_RestoreOldMode (int original_mode)
 	inerror = false;
 }
 
-static int VID_SetMode (int modenum, unsigned char *palette)
+static qboolean VID_SetMode (int modenum, const unsigned char *palette)
 {
 	int		original_mode, temp;
 	qboolean	status;
@@ -1628,7 +1626,7 @@ static int VID_SetMode (int modenum, unsigned char *palette)
 		else
 		{
 			Cvar_SetValueQuick (&vid_mode, (float)vid_modenum);
-			return 0;
+			return false;
 		}
 	}
 
@@ -1830,7 +1828,7 @@ void VID_UnlockBuffer (void)
 }
 
 
-void	VID_SetPalette (unsigned char *palette)
+void	VID_SetPalette (const unsigned char *palette)
 {
 	int		i;
 	palette_t	pal[256];
@@ -1890,7 +1888,8 @@ void	VID_SetPalette (unsigned char *palette)
 		}
 	}
 
-	memcpy (vid_curpal, palette, sizeof(vid_curpal));
+	if (palette != vid_curpal)
+		memcpy(vid_curpal, palette, sizeof(vid_curpal));
 
 	if (syscolchg)
 	{
@@ -1900,7 +1899,7 @@ void	VID_SetPalette (unsigned char *palette)
 }
 
 
-void	VID_ShiftPalette (unsigned char *palette)
+void	VID_ShiftPalette (const unsigned char *palette)
 {
 	VID_SetPalette (palette);
 }
@@ -2048,7 +2047,7 @@ static void VID_ForceMode_f (void)
 }
 
 
-void	VID_Init (unsigned char *palette)
+void	VID_Init (const unsigned char *palette)
 {
 	int		i, bestmatch, bestmatchmetric, t, dr, dg, db;
 	int		basenummodes;
