@@ -1,6 +1,6 @@
 /*
  * vid_win.c -- Win32 video driver
- * $Id: vid_win.c 5004 2012-11-05 16:38:26Z sezero $
+ * $Id: vid_win.c 5701 2016-07-28 20:00:43Z sezero $
  *
  * Adapted from Quake2 and from an initial work by MH with many
  * modifications to make it work in Hexen II: Hammer of Thyrion.
@@ -8,7 +8,7 @@
  *
  * - TODO: Add back more low resolutions.
  * - TODO: Add DDRAW (see Quake2)
- * - TODO: Better video mode management? Early config reading?
+ * - TODO: Better video mode management?
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  *
@@ -102,7 +102,7 @@ static byte		*vid_surfcache;
 static int	vid_surfcachesize;
 static int	VID_highhunkmark;
 
-unsigned char	vid_curpal[256*3];
+static unsigned char	vid_curpal[256*3];	/* save for mode changes */
 
 unsigned short	d_8to16table[256];
 unsigned int	d_8to24table[256];
@@ -142,7 +142,7 @@ static HDC maindc = NULL;
 static void VID_MenuDraw (void);
 static void VID_MenuKey (int key);
 
-static int VID_SetMode (int modenum, unsigned char *palette);
+static qboolean VID_SetMode (int modenum, const unsigned char *palette);
 static void AppActivate(BOOL fActive, BOOL minimize);
 static LRESULT WINAPI MainWndProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
 
@@ -184,7 +184,6 @@ VID_CheckAdequateMem
 */
 static qboolean VID_CheckAdequateMem (int width, int height)
 {
-#if 0	/* MH says there will always be enough */
 	int		tbuffersize;
 
 	tbuffersize = width * height * sizeof (*d_pzbuffer);
@@ -203,7 +202,6 @@ static qboolean VID_CheckAdequateMem (int width, int height)
 	{
 		return false;		// not enough memory for mode
 	}
-#endif
 
 	return true;
 }
@@ -224,7 +222,6 @@ static qboolean VID_AllocBuffers (int width, int height)
 
 	tbuffersize += tsize;
 
-#if 0	/* as in VID_CheckAdequateMem() above. */
 // see if there's enough memory, allowing for the normal mode 0x13 pixel,
 // z, and surface buffers
 	//if ((host_parms->memsize - tbuffersize + SURFCACHE_SIZE_AT_320X200 +
@@ -237,7 +234,6 @@ static qboolean VID_AllocBuffers (int width, int height)
 		Con_SafePrintf ("Not enough memory for video mode\n");
 		return false;		// not enough memory for mode
 	}
-#endif
 
 	vid_surfcachesize = tsize;
 
@@ -306,7 +302,7 @@ static void VID_ShutdownDIB (void)
 }
 
 
-static void VID_CreateDIB (int width, int height, unsigned char *palette)
+static void VID_CreateDIB (int width, int height, const unsigned char *palette)
 {
 	dibinfo_t   dibheader;
 	BITMAPINFO *pbmiDIB = &dibheader.bi;
@@ -881,7 +877,7 @@ static void VID_RestoreOldMode (int original_mode)
 	inerror = false;
 }
 
-static int VID_SetMode (int modenum, unsigned char *palette)
+static qboolean VID_SetMode (int modenum, const unsigned char *palette)
 {
 	int		original_mode, temp;
 	qboolean	status;
@@ -906,7 +902,7 @@ static int VID_SetMode (int modenum, unsigned char *palette)
 		else
 		{
 			Cvar_SetValueQuick (&vid_mode, (float)vid_modenum);
-			return 0;
+			return false;
 		}
 	}
 
@@ -1038,11 +1034,13 @@ void VID_UnlockBuffer (void)
 }
 
 
-void	VID_SetPalette (unsigned char *palette)
+void	VID_SetPalette (const unsigned char *palette)
 {
 	int		i;
 	RGBQUAD		colors[256];
-	unsigned char *pal = palette;
+	const unsigned char *pal;
+
+	pal = palette;
 
 	if (!Minimized)
 	{
@@ -1071,11 +1069,12 @@ void	VID_SetPalette (unsigned char *palette)
 		}
 	}
 
-	memcpy (vid_curpal, palette, sizeof (vid_curpal));
+	if (palette != vid_curpal)
+		memcpy(vid_curpal, palette, sizeof (vid_curpal));
 }
 
 
-void	VID_ShiftPalette (unsigned char *palette)
+void	VID_ShiftPalette (const unsigned char *palette)
 {
 	VID_SetPalette (palette);
 }
@@ -1220,7 +1219,7 @@ static void VID_ForceMode_f (void)
 }
 
 
-void	VID_Init (unsigned char *palette)
+void	VID_Init (const unsigned char *palette)
 {
 	int		i, bestmatch, bestmatchmetric, t, dr, dg, db;
 	byte		*ptmp;
