@@ -1,6 +1,6 @@
 /*
  * gl_main.c
- * $Id: gl_rmain.c 5131 2013-02-20 12:56:24Z sezero $
+ * $Id: gl_rmain.c 5578 2016-05-05 07:23:35Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
@@ -21,7 +21,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-
 #include "quakedef.h"
 
 entity_t	r_worldentity;
@@ -95,7 +94,6 @@ cvar_t	gl_clear = {"gl_clear", "0", CVAR_NONE};
 cvar_t	gl_cull = {"gl_cull", "1", CVAR_NONE};
 cvar_t	gl_ztrick = {"gl_ztrick", "0", CVAR_ARCHIVE};
 cvar_t	gl_zfix = {"gl_zfix", "1", CVAR_ARCHIVE};
-cvar_t	gl_multitexture = {"gl_multitexture", "0", CVAR_ARCHIVE};
 cvar_t	gl_smoothmodels = {"gl_smoothmodels", "1", CVAR_NONE};
 cvar_t	gl_affinemodels = {"gl_affinemodels", "0", CVAR_NONE};
 cvar_t	gl_polyblend = {"gl_polyblend", "1", CVAR_NONE};
@@ -980,8 +978,13 @@ static void R_DrawAliasModel (entity_t *e)
 		glShadeModel_fp (GL_SMOOTH);
 	glTexEnvf_fp(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 
+#if defined(__AMIGA__) && defined(REFGL_MINIGL)
+	if (gl_affinemodels.integer)
+		glDisable_fp (MGL_PERSPECTIVE_MAPPING);
+#else
 	if (gl_affinemodels.integer)
 		glHint_fp (GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
+#endif
 
 	R_SetupAliasFrame (e, paliashdr);
 
@@ -1003,8 +1006,13 @@ static void R_DrawAliasModel (entity_t *e)
 	glTexEnvf_fp (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 
 	glShadeModel_fp (GL_FLAT);
+#if defined(__AMIGA__) && defined(REFGL_MINIGL)
+	if (gl_affinemodels.integer)
+		glEnable_fp (MGL_PERSPECTIVE_MAPPING);
+#else
 	if (gl_affinemodels.integer)
 		glHint_fp (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
+#endif
 
 	glPopMatrix_fp ();
 
@@ -1523,15 +1531,15 @@ static void GL_DrawBlendPoly (void)
 GL_DoGamma
 
 Uses GL_DrawBlendPoly() for gamma correction.
+Idea originally from LordHavoc.
 This trick is useful if normal ways of gamma
 adjustment fail: In case of 3dfx Voodoo1/2/Rush,
 we can't use 3dfx specific extensions in unix,
 so this can be our friend at a cost of 4-5 fps.
 To be called from R_PolyBlend().
-Idea originally nicked from LordHavoc, re-worked
-and extended by muff - 5 Feb 2001.
 =================
 */
+#if 0
 static void GL_DoGamma (void)
 {
 	if (v_gamma.value >= 1)
@@ -1544,6 +1552,7 @@ static void GL_DoGamma (void)
 
 	glBlendFunc_fp (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 }
+#endif
 
 /*
 ============
@@ -1570,8 +1579,7 @@ static void R_PolyBlend (void)
 		GL_DrawBlendPoly ();
 	}
 
-	if (gl_dogamma)
-		GL_DoGamma ();
+	/*GL_DoGamma ();*/
 
 	glDisable_fp (GL_BLEND);
 	glEnable_fp (GL_TEXTURE_2D);
