@@ -1,6 +1,6 @@
 /*
  * server/host_cmd.c -- console commands
- * $Id: host_cmd.c 5157 2013-03-02 10:55:20Z sezero $
+ * $Id: host_cmd.c 5610 2016-06-25 05:55:42Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
@@ -24,7 +24,7 @@
  */
 
 #include "quakedef.h"
-#include <ctype.h>
+#include "q_ctype.h"
 
 static	double	old_svtime;
 	/* sv.time of prev. level when changing levels, saved by changelevel2().
@@ -1025,17 +1025,7 @@ static int LoadGamestate (const char *level, const char *startspot, int ClientsM
 				 * the map.)  -- O.S. */
 					Con_DPrintf("%s: entnum %d >= sv.num_edicts (%d)\n",
 							__thisfunc__, entnum, sv.num_edicts);
-					if (entnum < MAX_EDICTS)
-					{
-						sv.num_edicts = entnum + 1;
-					}
-					else
-					{
-					/* if we reach here, it can only mean trouble: the
-					 * saved game is either incompatible or corrupt. */
-						Con_DPrintf("%s: entity %d NOT restored!\n",
-									__thisfunc__, entnum);
-					}
+					sv.num_edicts = entnum + 1;
 				}
 
 				SV_LinkEdict (ent, false);
