@@ -2,7 +2,7 @@
  * sv_model.c -- model loading and caching
  * models are the only shared resource between a client and server
  * running on the same machine.
- * $Id: sv_model.c 5171 2013-03-10 12:00:17Z sezero $
+ * $Id: sv_model.c 5290 2014-04-28 20:50:25Z sezero $
  *
  * This version of model.[ch] are based on the quake dedicated server
  * application lhnqserver by Forest 'LordHavoc' Hale, with simplified
@@ -490,10 +490,14 @@ static void CalcSurfaceExtents (msurface_t *s, int firstedge, int numedges)
 
 		for (j = 0; j < 2; j++)
 		{
-			val = v->position[0] * tex->vecs[j][0] + 
-				v->position[1] * tex->vecs[j][1] +
-				v->position[2] * tex->vecs[j][2] +
-				tex->vecs[j][3];
+			/* added double casts so that 64 bit/sse2 builds' precision
+			 * matches that of x87 floating point. took from QuakeSpasm,
+			 * patch by Eric Wasylishen.  */
+			val =	((double)v->position[0] * (double)tex->vecs[j][0]) +
+				((double)v->position[1] * (double)tex->vecs[j][1]) +
+				((double)v->position[2] * (double)tex->vecs[j][2]) +
+				(double)tex->vecs[j][3];
+
 			if (val < mins[j])
 				mins[j] = val;
 			if (val > maxs[j])
