@@ -1,7 +1,7 @@
 /*
  * sys_dos.c -- DOS system interface code.
  * from quake1 source with adaptations for uhexen2.
- * $Id: sys_dos.c 4767 2012-06-16 20:48:51Z sezero $
+ * $Id: sys_dos.c 5256 2013-10-31 08:50:34Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  *
@@ -372,8 +372,8 @@ int Sys_CopyFile (const char *frompath, const char *topath)
 {
 	char	buf[COPY_READ_BUFSIZE];
 	int	in, out;
-	int		err = 0;
 	long	remaining, count;
+	struct ftime	ft;
 
 	in = open (frompath, O_RDONLY | O_BINARY);
 	if (in < 0)
@@ -396,33 +396,22 @@ int Sys_CopyFile (const char *frompath, const char *topath)
 		return 1;
 	}
 
-	memset (buf, 0, sizeof(buf));
 	while (remaining)
 	{
 		if (remaining < sizeof(buf))
 			count = remaining;
-		else
-			count = sizeof(buf);
+		else	count = sizeof(buf);
 
 		if (read(in, buf, count) < 0)
-		{
-			err = -1;
 			break;
-		}
-
 		if (write(out, buf, count) < 0)
-		{
-			err = -1;
 			break;
-		}
 
 		remaining -= count;
 	}
 
-	if (!err)
-	{
-	// restore the file's timestamp
-		struct ftime	ft;
+	if (remaining == 0) {
+	/* restore the file's timestamp */
 		if (getftime(in, &ft) == 0)
 			setftime(out, &ft);
 	}
@@ -430,7 +419,7 @@ int Sys_CopyFile (const char *frompath, const char *topath)
 	close (in);
 	close (out);
 
-	return err;
+	return remaining;
 }
 
 /*
