@@ -1,7 +1,7 @@
 /*
  * sys_dos.c -- DOS system interface code.
  * from quake1 source with adaptations for uhexen2.
- * $Id: sys_dos.c 4767 2012-06-16 20:48:51Z sezero $
+ * $Id: sys_dos.c 5455 2015-10-03 15:47:10Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  *
@@ -40,6 +40,7 @@
 #include "quakedef.h"
 #include "dosisms.h"
 #include "debuglog.h"
+#include "sys_dxe.h"
 
 #define MIN_MEM_ALLOC	0x1000000	/* minimum 16 mb */
 #define STD_MEM_ALLOC	0x2000000	/* standart 32 mb */
@@ -48,6 +49,9 @@
  * Norberto Alfredo Bensa
  */
 int	_crt0_startup_flags = _CRT0_FLAG_UNIX_SBRK;
+#ifdef GLQUAKE /* need at least 1MB stack for 3dfx. */
+unsigned int _stklen = 1048576;  /* FS: FIXME TUNE. */
+#endif
 
 int		end_of_memory;
 static qboolean	lockmem, lockunlockmem, unlockmem;
@@ -80,8 +84,6 @@ cvar_t			sys_throttle = {"sys_throttle", "0.02", CVAR_ARCHIVE};
 
 qboolean		isDedicated;
 
-static int		minmem;
-
 float			fptest_temp;
 
 extern char	start_of_memory __asm__("start");
@@ -210,6 +212,8 @@ static void Sys_DetectWin95 (void)
 	}
 }
 
+#ifndef GLQUAKE
+static int		minmem;
 
 static void *dos_getmaxlockedmem (int *size)
 {
@@ -379,6 +383,7 @@ UpdateSbrk:
 	*size = working_size;
 	return working_memory;
 }
+#endif
 
 
 int Sys_mkdir (const char *path, qboolean crash)
@@ -438,8 +443,8 @@ int Sys_CopyFile (const char *frompath, const char *topath)
 {
 	char	buf[COPY_READ_BUFSIZE];
 	int	in, out;
-	int		err = 0;
 	long	remaining, count;
+	struct ftime	ft;
 
 	in = open (frompath, O_RDONLY | O_BINARY);
 	if (in < 0)
@@ -462,33 +467,22 @@ int Sys_CopyFile (const char *frompath, const char *topath)
 		return 1;
 	}
 
-	memset (buf, 0, sizeof(buf));
 	while (remaining)
 	{
 		if (remaining < sizeof(buf))
 			count = remaining;
-		else
-			count = sizeof(buf);
+		else	count = sizeof(buf);
 
 		if (read(in, buf, count) < 0)
-		{
-			err = -1;
 			break;
-		}
-
 		if (write(out, buf, count) < 0)
-		{
-			err = -1;
 			break;
-		}
 
 		remaining -= count;
 	}
 
-	if (!err)
-	{
-	// restore the file's timestamp
-		struct ftime	ft;
+	if (remaining == 0) {
+	/* restore the file's timestamp */
 		if (getftime(in, &ft) == 0)
 			setftime(out, &ft);
 	}
@@ -496,7 +490,7 @@ int Sys_CopyFile (const char *frompath, const char *topath)
 	close (in);
 	close (out);
 
-	return err;
+	return remaining;
 }
 
 /*
@@ -604,6 +598,8 @@ static void Sys_Init (void)
 
 	_go32_interrupt_stack_size = 4 * 1024;
 	_go32_rmcb_stack_size = 4 * 1024;
+
+	Sys_InitDXE3();
 }
 
 void Sys_Shutdown (void)
@@ -946,10 +942,19 @@ static void Sys_GetMemory (void)
 		quakeparms.memsize = (int) (atof(com_argv[j + 1]) * 1024 * 1024);
 		quakeparms.membase = malloc (quakeparms.memsize);
 	}
+#ifdef GLQUAKE
+	else
+	{
+		/* 16 mb is usually enough. Leave rest of mem for gl driver */
+		quakeparms.memsize = (int) 0x1000000;
+		quakeparms.membase = malloc (quakeparms.memsize);
+	}
+#else
 	else
 	{
 		quakeparms.membase = dos_getmaxlockedmem (&quakeparms.memsize);
 	}
+#endif
 
 	printf("malloc'd: %d\n", quakeparms.memsize);
 
