@@ -1,6 +1,6 @@
 /*
  * sys_unix.c -- Unix system interface code
- * $Id: sys_unix.c 5033 2013-01-07 10:00:49Z sezero $
+ * $Id: sys_unix.c 5341 2014-10-18 14:04:11Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 2001 contributors of the Anvil of Thyrion project
@@ -34,9 +34,9 @@
 
 #include <errno.h>
 #include <unistd.h>
-#if USE_PASSWORD_FILE && DO_USERDIRS
+#if DO_USERDIRS
 #include <pwd.h>
-#endif	/* USE_PASSWORD_FILE */
+#endif
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <fcntl.h>
@@ -78,7 +78,11 @@ int Sys_mkdir (const char *path, qboolean crash)
 {
 	int rc = mkdir (path, 0777);
 	if (rc != 0 && errno == EEXIST)
-		rc = 0;
+	{
+		struct stat st;
+		if (stat(path, &st) == 0 && S_ISDIR(st.st_mode))
+			rc = 0;
+	}
 	if (rc != 0 && crash)
 	{
 		rc = errno;
@@ -138,9 +142,9 @@ int Sys_CopyFile (const char *frompath, const char *topath)
 	char	buf[COPY_READ_BUFSIZE];
 	FILE	*in, *out;
 	struct stat	st;
-	int		err = 0;
-//	off_t		remaining, count;
-	size_t		remaining, count;
+	struct utimbuf	tm;
+/*	off_t	remaining, count;*/
+	size_t	remaining, count;
 
 	if (stat(frompath, &st) != 0)
 	{
@@ -162,22 +166,15 @@ int Sys_CopyFile (const char *frompath, const char *topath)
 	}
 
 	remaining = st.st_size;
-	memset (buf, 0, sizeof(buf));
 	while (remaining)
 	{
 		if (remaining < sizeof(buf))
 			count = remaining;
-		else
-			count = sizeof(buf);
+		else	count = sizeof(buf);
 
-		fread (buf, 1, count, in);
-		err = ferror (in);
-		if (err)
+		if (fread(buf, 1, count, in) != count)
 			break;
-
-		fwrite (buf, 1, count, out);
-		err = ferror (out);
-		if (err)
+		if (fwrite(buf, 1, count, out) != count)
 			break;
 
 		remaining -= count;
@@ -186,16 +183,15 @@ int Sys_CopyFile (const char *frompath, const char *topath)
 	fclose (in);
 	fclose (out);
 
-	if (!err)
-	{
-	// restore the file's timestamp
-		struct utimbuf		tm;
+	if (remaining == 0) {
+	/* restore the file's timestamp */
 		tm.actime = time (NULL);
 		tm.modtime = st.st_mtime;
 		utime (topath, &tm);
+		return 0;
 	}
 
-	return err;
+	return 1;
 }
 
 /*
@@ -307,6 +303,17 @@ static void Sys_Init (void)
 {
 /* do we really need these with opengl ?? */
 	Sys_SetFPCW();
+#if defined(SDLQUAKE)
+	if (SDL_Init(0) < 0)
+		Sys_Error("SDL failed to initialize.");
+#endif
+}
+
+static void Sys_AtExit (void)
+{
+#if defined(SDLQUAKE)
+	SDL_Quit();
+#endif
 }
 
 
@@ -536,15 +543,12 @@ static int Sys_GetUserdir (char *dst, size_t dstsize)
 {
 	size_t		n;
 	const char	*home_dir = NULL;
-#if USE_PASSWORD_FILE
 	struct passwd	*pwent;
 
-	pwent = getpwuid( getuid() );
+	pwent = getpwuid(getuid());
 	if (pwent == NULL)
 		perror("getpwuid");
-	else
-		home_dir = pwent->pw_dir;
-#endif
+	else	home_dir = pwent->pw_dir;
 	if (home_dir == NULL)
 		home_dir = getenv("HOME");
 	if (home_dir == NULL)
@@ -761,6 +765,7 @@ int main (int argc, char **argv)
 		Sys_Error ("Insufficient memory.\n");
 
 	Sys_Init ();
+	atexit (Sys_AtExit);
 
 	Host_Init();
 
