@@ -1,6 +1,6 @@
 /*
  * sys_win.c -- Windows system interface code
- * $Id: sys_win.c 4832 2012-07-05 16:10:14Z sezero $
+ * $Id: sys_win.c 5431 2015-09-08 21:40:35Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 2005-2012  O.Sezer <sezero@users.sourceforge.net>
@@ -237,6 +237,41 @@ void Sys_MakeCodeWriteable (unsigned long startaddr, unsigned long length)
 #endif	/* id386, !GLQUAKE */
 
 
+/*
+================
+Sys_SetDPIAware
+================
+*/
+typedef enum { dpi_unaware = 0, dpi_system_aware = 1, dpi_monitor_aware = 2 } dpi_awareness;
+typedef BOOL (WINAPI *SetProcessDPIAwareFunc)();
+typedef HRESULT (WINAPI *SetProcessDPIAwarenessFunc)(dpi_awareness value);
+
+static void Sys_SetDPIAware (void)
+{
+	HMODULE hUser32, hShcore;
+	SetProcessDPIAwarenessFunc setDPIAwareness;
+	SetProcessDPIAwareFunc setDPIAware;
+
+	/* We do not handle the OS scaling our window. Call
+	 * SetProcessDpiAwareness() or SetProcessDPIAware()
+	 * to opt out of scaling.
+	 */
+	hShcore = LoadLibraryA ("Shcore.dll");
+	hUser32 = LoadLibraryA ("user32.dll");
+	setDPIAwareness = (SetProcessDPIAwarenessFunc) (hShcore ? GetProcAddress (hShcore, "SetProcessDpiAwareness") : NULL);
+	setDPIAware = (SetProcessDPIAwareFunc) (hUser32 ? GetProcAddress (hUser32, "SetProcessDPIAware") : NULL);
+
+	if (setDPIAwareness) /* Windows 8.1+ */
+		setDPIAwareness (dpi_monitor_aware);
+	else if (setDPIAware) /* Vista, Win7 or 8.0 */
+		setDPIAware ();
+
+	if (hShcore)
+		FreeLibrary (hShcore);
+	if (hUser32)
+		FreeLibrary (hUser32);
+}
+
 /*
 ================
 Sys_Init
@@ -275,6 +310,9 @@ static void Sys_Init (void)
 		}
 	}
 
+	if (!isDedicated)
+		Sys_SetDPIAware ();
+
 	timeBeginPeriod (1);	/* 1 ms timer precision */
 	starttime = timeGetTime ();
 
