@@ -2,7 +2,7 @@
  * model.c -- model loading and caching
  * models are the only shared resource between a client and server
  * running on the same machine.
- * $Id: model.c 5171 2013-03-10 12:00:17Z sezero $
+ * $Id: model.c 5680 2016-07-21 05:37:46Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
@@ -371,10 +371,12 @@ static void Mod_LoadTextures (lump_t *l)
 	texture_t	*anims[10];
 	texture_t	*altanims[10];
 	dmiptexlump_t *m;
+#ifdef WAL_TEXTURES
 	// external WAL texture loading
 	char		texname[MAX_QPATH];
 	int		mark;
 	miptex_wal_t	*mt_wal;
+#endif
 
 	if (!l->filelen)
 	{
@@ -399,6 +401,7 @@ static void Mod_LoadTextures (lump_t *l)
 		for (j = 0; j < MIPLEVELS; j++)
 			mt->offsets[j] = LittleLong (mt->offsets[j]);
 
+#ifdef WAL_TEXTURES
 		if (!r_texture_external.integer)
 			goto bsp_tex_internal;
 		// try an external wal texture file first
@@ -450,6 +453,7 @@ static void Mod_LoadTextures (lump_t *l)
 		else
 		{	// load internal bsp pixel data
 bsp_tex_internal:
+#endif /* WAL_TEXTURES */
 			if ( (mt->width & 15) || (mt->height & 15) )
 				Sys_Error ("Texture %s is not 16 aligned", mt->name);
 			pixels = mt->width*mt->height/64*85;
@@ -463,7 +467,9 @@ bsp_tex_internal:
 				tx->offsets[j] = mt->offsets[j] + sizeof(texture_t) - sizeof(miptex_t);
 			// the pixels immediately follow the structures
 			memcpy ( tx+1, mt+1, pixels);
+#ifdef WAL_TEXTURES
 		}
+#endif
 
 		if (!strncmp(mt->name,"sky",3))
 			R_InitSky (tx);
@@ -836,10 +842,14 @@ static void CalcSurfaceExtents (msurface_t *s)
 
 		for (j = 0; j < 2; j++)
 		{
-			val = v->position[0] * tex->vecs[j][0] + 
-				v->position[1] * tex->vecs[j][1] +
-				v->position[2] * tex->vecs[j][2] +
-				tex->vecs[j][3];
+			/* added double casts so that 64 bit/sse2 builds' precision
+			 * matches that of x87 floating point. took from QuakeSpasm,
+			 * patch by Eric Wasylishen.  */
+			val =	((double)v->position[0] * (double)tex->vecs[j][0]) +
+				((double)v->position[1] * (double)tex->vecs[j][1]) +
+				((double)v->position[2] * (double)tex->vecs[j][2]) +
+				(double)tex->vecs[j][3];
+
 			if (val < mins[j])
 				mins[j] = val;
 			if (val > maxs[j])
@@ -1297,14 +1307,17 @@ static void Mod_LoadPlanes (lump_t *l)
 RadiusFromBounds
 =================
 */
-static float RadiusFromBounds (vec3_t mins, vec3_t maxs)
+static float RadiusFromBounds (const vec3_t mins, const vec3_t maxs)
 {
 	int		i;
 	vec3_t	corner;
+	vec_t	a, b;
 
 	for (i = 0; i < 3; i++)
 	{
-		corner[i] = fabs(mins[i]) > fabs(maxs[i]) ? fabs(mins[i]) : fabs(maxs[i]);
+		b = fabs(maxs[i]);
+		a = fabs(mins[i]);
+		corner[i] = (a > b)? a : b;
 	}
 
 	return VectorLength (corner);
