@@ -1,6 +1,6 @@
 /*
  * r_main.c
- * $Id: r_main.c 5141 2013-02-25 20:01:05Z sezero $
+ * $Id: r_main.c 5704 2016-08-01 11:15:30Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
@@ -484,6 +484,8 @@ Guaranteed to be called before the first refresh
 */
 #if defined(PLATFORM_DOS) || defined(SVGAQUAKE)
 #define NOT_VGA_MODE (vid.aspect <= 1.10f)	/* no Hor+ for weirdass VGA modes */
+#elif defined(PLATFORM_AMIGAOS3)
+#define NOT_VGA_MODE (!vid.noadapt)	/* no Hor+ for Amiga native chipset modes */
 #else
 #define NOT_VGA_MODE true
 #endif
