@@ -1,6 +1,6 @@
 /*
  * sys_amiga.c -- Amiga system interface code
- * $Id: sys_amiga.c 4982 2012-10-04 20:10:32Z sezero $
+ * $Id: sys_amiga.c 5714 2016-08-11 08:47:12Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 2012  Szilard Biro <col.lawrence@gmail.com>
@@ -38,8 +38,18 @@
 #include "sdl_inc.h"
 #endif	/* SDLQUAKE */
 
-#ifdef __MORPHOS__
-int __stack = 0x200000; /* 2 MB stack */
+int __stack = 0x100000; /* 1 MB stack */
+#ifdef __AROS__
+#include "incstack.h"
+/* The problem here is that our real main never returns: the exit
+ * point of program is either Sys_Quit() or Sys_Error(). One way
+ * of making the real main return to the incstack.h main wrapper
+ * is setjmp()'ing in real main and longjmp()'ing from the actual
+ * exit points, avoiding exit(). */
+#include <setjmp.h>
+static jmp_buf exit_buf;
+static int my_rc = 0;
+#define HAVE_AROS_MAIN_WRAPPER
 #endif
 
 // heapsize: minimum 16mb, standart 32 mb, max is 96 mb.
@@ -52,15 +62,21 @@ int __stack = 0x200000; /* 2 MB stack */
 cvar_t		sys_nostdout = {"sys_nostdout", "0", CVAR_NONE};
 cvar_t		sys_throttle = {"sys_throttle", "0.02", CVAR_ARCHIVE};
 
+#ifdef PLATFORM_AMIGAOS3
+#define USE_ECLOCK_TIMER
+#define MY_TIMERUNIT	UNIT_ECLOCK
+#else
+#define MY_TIMERUNIT	UNIT_MICROHZ
 static double		starttime;
+#endif
 static qboolean		first = true;
 
 struct timerequest	*timerio;
 struct MsgPort		*timerport;
-#ifdef __AROS__
-struct Device		*TimerBase;
-#else
+#if defined(__MORPHOS__) || defined(__VBCC__)
 struct Library		*TimerBase;
+#else
+struct Device		*TimerBase;
 #endif
 
 
@@ -115,7 +131,7 @@ long Sys_filesize (const char *path)
 {
 	long size = -1;
 	BPTR fh = Open((const STRPTR) path, MODE_OLDFILE);
-	if (fh != NULL)
+	if (fh)
 	{
 		struct FileInfoBlock *fib = (struct FileInfoBlock*)
 					AllocDosObject(DOS_FIB, NULL);
@@ -134,7 +150,7 @@ int Sys_FileType (const char *path)
 {
 	int type = FS_ENT_NONE;
 	BPTR fh = Open((const STRPTR) path, MODE_OLDFILE);
-	if (fh != NULL)
+	if (fh)
 	{
 		struct FileInfoBlock *fib = (struct FileInfoBlock*)
 					AllocDosObject(DOS_FIB, NULL);
@@ -181,7 +197,7 @@ int Sys_CopyFile (const char *frompath, const char *topath)
 		FreeDosObject(DOS_FIB, fib);
 		if (remaining < 0)
 		{
-			Con_Printf("%s: can't determine size for %s\n", __thisfunc__, frompath);
+			Con_Printf ("%s: can't determine size for %s\n", __thisfunc__, frompath);
 			Close(in);
 			return 1;
 		}
@@ -200,17 +216,14 @@ int Sys_CopyFile (const char *frompath, const char *topath)
 		return 1;
 	}
 
-	memset (buf, 0, sizeof(buf));
 	while (remaining)
 	{
 		if (remaining < sizeof(buf))
 			count = remaining;
-		else
-			count = sizeof(buf);
+		else	count = sizeof(buf);
 
 		if (Read(in, buf, count) == -1)
 			break;
-
 		if (Write(out, buf, count) == -1)
 			break;
 
@@ -247,13 +260,10 @@ static STRPTR pattern_helper (const char *pat)
 	const char	*p;
 	int	n;
 
-	for (n = 0, p = pat; p != NULL; )
+	for (n = 0, p = pat; *p != '\0'; ++p, ++n)
 	{
-		p = strchr (p, '*');
-		if (p != NULL)
-		{
-			++n;	++p;
-		}
+		if ((p = strchr (p, '*')) == NULL)
+			break;
 	}
 
 	if (n == 0)
@@ -287,7 +297,7 @@ const char *Sys_FindFirstFile (const char *path, const char *pattern)
 	if (apath)
 		Sys_Error ("Sys_FindFirst without FindClose");
 
-	apath = AllocMem (sizeof(struct AnchorPath) + PATH_SIZE, MEMF_CLEAR);
+	apath = AllocVec (sizeof(struct AnchorPath) + PATH_SIZE, MEMF_CLEAR);
 	if (!apath)
 		return NULL;
 
@@ -300,7 +310,8 @@ const char *Sys_FindFirstFile (const char *path, const char *pattern)
 		oldcurrentdir = CurrentDir(newdir);
 	else
 	{
-		FreeMem(apath, sizeof(struct AnchorPath) + PATH_SIZE);
+		FreeVec(apath);
+		apath = NULL;
 		return NULL;
 	}
 
@@ -325,17 +336,35 @@ const char *Sys_FindNextFile (void)
 
 void Sys_FindClose (void)
 {
-	if (apath == NULL)
+	if (!apath)
 		return;
 	MatchEnd(apath);
-	FreeMem(apath, sizeof(struct AnchorPath) + PATH_SIZE);
+	FreeVec(apath);
 	UnLock(CurrentDir(oldcurrentdir));
-	oldcurrentdir = NULL;
+	oldcurrentdir = 0;
 	apath = NULL;
 	Z_Free(pattern_str);
 	pattern_str = NULL;
 }
 
+/* File existence check with the "Please insert volume XXX"
+ * system requester disabled.  */
+qboolean Sys_PathExistsQuiet(const char *name)
+{
+	struct Process *self = (struct Process *) FindTask(NULL);
+	APTR oldwinptr; BPTR lock;
+
+	oldwinptr = self->pr_WindowPtr;
+	self->pr_WindowPtr = (APTR) -1;
+	lock = Lock((const STRPTR) name, ACCESS_READ);
+	self->pr_WindowPtr = oldwinptr;
+	if (lock) {
+		UnLock(lock);
+		return true;
+	}
+	return false;
+}
+
 /*
 ===============================================================================
 
@@ -364,17 +393,20 @@ Sys_Init
 */
 static void Sys_Init (void)
 {
+	/*MaskExceptions ();*/
+	Sys_SetFPCW ();
+
 	if ((timerport = CreateMsgPort()))
 	{
 		if ((timerio = (struct timerequest *)CreateIORequest(timerport, sizeof(struct timerequest))))
 		{
-			if (OpenDevice((STRPTR) TIMERNAME, UNIT_MICROHZ,
+			if (OpenDevice((STRPTR) TIMERNAME, MY_TIMERUNIT,
 					(struct IORequest *) timerio, 0) == 0)
 			{
-#ifdef __AROS__
-				TimerBase = timerio->tr_node.io_Device;
-#else
+#if defined(__MORPHOS__) || defined(__VBCC__)
 				TimerBase = (struct Library *)timerio->tr_node.io_Device;
+#else
+				TimerBase = timerio->tr_node.io_Device;
 #endif
 			}
 			else
@@ -398,6 +430,11 @@ static void Sys_Init (void)
 	timerio->tr_time.tv_micro = 1;
 	SendIO((struct IORequest *) timerio);
 	WaitIO((struct IORequest *) timerio);
+
+#if defined(SDLQUAKE)
+	if (SDL_Init(0) < 0)
+		Sys_Error("SDL failed to initialize.");
+#endif
 }
 
 static void Sys_AtExit (void)
@@ -417,12 +454,17 @@ static void Sys_AtExit (void)
 		DeleteMsgPort(timerport);
 		TimerBase = NULL;
 	}
+#if defined(SDLQUAKE)
+	SDL_Quit();
+#endif
 }
 
 void Sys_ErrorMessage(const char *string)
 {
 	struct EasyStruct es;
 
+	if (!IntuitionBase) return;
+
 	es.es_StructSize = sizeof(es);
 	es.es_Flags = 0;
 	es.es_Title = (STRPTR) ENGINE_NAME " error";
@@ -461,7 +503,13 @@ void Sys_Error (const char *error, ...)
 	putc ('\n', stderr);
 	Sys_ErrorMessage (text);
 
+#ifdef HAVE_AROS_MAIN_WRAPPER
+	Sys_AtExit();
+	my_rc = 1;
+	longjmp(exit_buf, 1);
+#else
 	exit (1);
+#endif
 }
 
 void Sys_PrintTerm (const char *msgtxt)
@@ -479,7 +527,12 @@ void Sys_Quit (void)
 {
 	Host_Shutdown();
 
+#ifdef HAVE_AROS_MAIN_WRAPPER
+	Sys_AtExit();
+	longjmp(exit_buf, 1);
+#else
 	exit (0);
+#endif
 }
 
 
@@ -490,6 +543,21 @@ Sys_DoubleTime
 */
 double Sys_DoubleTime (void)
 {
+#if defined(USE_ECLOCK_TIMER)
+	static ULONG old_lo;
+	ULONG E_Freq;
+	struct EClockVal eclock;
+
+	E_Freq = ReadEClock(&eclock);
+
+	if (first)
+	{
+		first = false;
+		old_lo = eclock.ev_lo;
+		return 0.0;
+	}
+	return (double)(eclock.ev_lo - old_lo) / (double)E_Freq;
+#else
 	struct timeval	tp;
 	double		now;
 
@@ -505,6 +573,7 @@ double Sys_DoubleTime (void)
 	}
 
 	return now - starttime;
+#endif
 }
 
 char *Sys_DateTimeString (char *buf)
@@ -569,8 +638,12 @@ void Sys_SendKeyEvents (void)
 }
 
 #if !(defined(__AROS__) || defined(__MORPHOS__))
+#if !defined(IPTR) /* SDI headers may define it */
 typedef ULONG IPTR;
-#endif /* AROS IPTR */
+#define IPTR IPTR
+#endif
+#endif
+
 #define MAX_CLIPBOARDTXT	MAXCMDLINE	/* 256 */
 char *Sys_GetClipboardData (void)
 {
@@ -617,13 +690,7 @@ static int Sys_GetBasedir (char *argv0, char *dst, size_t dstsize)
 		return 0;
 	return -1;
 #else
-	struct Task *self;
-	BPTR lock;
-
-	self = FindTask(NULL);
-	lock = ((struct Process *) self)->pr_CurrentDir;
-
-	if (NameFromLock(lock, (STRPTR) dst, dstsize) != 0)
+	if (NameFromLock(((struct Process *) FindTask(NULL))->pr_CurrentDir, (STRPTR) dst, dstsize) != 0)
 		return 0;
 	return -1;
 #endif
@@ -721,6 +788,11 @@ int main (int argc, char **argv)
 	int			i;
 	double		time, oldtime, newtime;
 
+#ifdef HAVE_AROS_MAIN_WRAPPER
+	if (setjmp(exit_buf))
+		return my_rc;
+#endif
+
 	PrintVersion();
 
 	if (argc > 1)
@@ -730,13 +802,13 @@ int main (int argc, char **argv)
 			if ( !(strcmp(argv[i], "-v")) || !(strcmp(argv[i], "-version" )) ||
 				!(strcmp(argv[i], "--version")) )
 			{
-				exit(0);
+				return 0;
 			}
 			else if ( !(strcmp(argv[i], "-h")) || !(strcmp(argv[i], "-help" )) ||
 				  !(strcmp(argv[i], "-?")) || !(strcmp(argv[i], "--help")) )
 			{
 				PrintHelp(argv[0]);
-				exit (0);
+				return 0;
 			}
 		}
 	}
@@ -788,7 +860,9 @@ int main (int argc, char **argv)
 	if (!parms.membase)
 		Sys_Error ("Insufficient memory.\n");
 
+#ifndef HAVE_AROS_MAIN_WRAPPER
 	atexit (Sys_AtExit);
+#endif
 	Sys_Init ();
 
 	Host_Init();
