@@ -1,6 +1,6 @@
 /*
  * sv_edict.c -- entity dictionary
- * $Id: pr_edict.c 5175 2013-03-11 08:01:13Z sezero $
+ * $Id: pr_edict.c 5202 2013-06-28 17:28:22Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
@@ -1447,8 +1447,8 @@ void ED_LoadFromFile (const char *data)
 ===============
 PR_GetProgFilename
 
-find the correct progs.dat filename
-based on map name, using maplist.txt
+return the correct progs filename based on map name
+by parsing maplist.txt
 ===============
 */
 static const char def_progname[] = "hwprogs.dat";
@@ -1458,84 +1458,75 @@ static const char *PR_GetProgFilename (void)
 #if !USE_MULTIPLE_PROGS
 	return def_progname;
 #else
+/* original format:
+ * line #1 : <number of lines excluding this one>
+ * line #2+: <map name><one space><prog filename>
+ */
 	static char	finalprogname[MAX_QPATH];
 	unsigned int	id0, id1;
-	FILE	*FH;
+	fshandle_t	FH;
 
-	FS_OpenFile (maplist_name, &FH, &id1);
-	if (FH == NULL)
+	FH.length = (long) FS_OpenFile (maplist_name, & FH.file, &id1);
+	FH.pak = file_from_pak;
+	if (FH.file == NULL)
 		return def_progname;
 	else if (FS_FileExists(def_progname, &id0) && id1 < id0)
 	{
 		Con_DPrintf("ignored %s from a gamedir with lower priority\n", maplist_name);
-		return def_progname;
+		goto _fail;
 	}
 	else
 	{
-		char	build[2048], *test;
-		char	mapname[MAX_QPATH], progname[MAX_QPATH];
-		int	i, j, k;
+		char	build[256], *test;
+		int	entries;
 
-		strcpy(finalprogname, def_progname);
+		FH.pos = 0;
+		FH.start = ftell(FH.file);
 
-		// Format of maplist.txt :
-		// Line #1 : <number of lines excluding this one>
-		// Line #2+: <map name><one space><prog filename>
-		fgets(build, sizeof(build), FH);
-		j = atoi(build);
-		for (i = 0; i < j; i++)
+		if (!FS_fgets(build, sizeof(build), &FH))
+			goto _fail;
+		entries = atoi(build);
+		if (entries <= 0)
+			goto _fail;
+
+		while (--entries >= 0)
 		{
-			k = 0;
-			memset (build, 0, sizeof(build));
-			test = fgets (build, sizeof(build), FH);
-			if (test)
+			if (!(test = FS_fgets (build, sizeof(build), &FH)))
+				goto _fail; /* unexpected EOF */
+			while (*test)
 			{
-				// remove end-of-line characters
-				while (build[k])
+				if (*test == '\r' || *test == '\n')
 				{
-					if (build[k] == '\r' || build[k] == '\n')
-						build[k] = '\0';
-					// while we're here, replace tabs with spaces
-					if (build[k] == '\t')
-						build[k] = ' ';
-					k++;
+					*test = '\0';
+					break;
 				}
-				// go to the last character
-				while (build[k] == 0 && k > 0)
-					k--;
-				// remove trailing spaces
-				while (k > 0)
-				{
-					if (build[k] == ' ')
-					{
-						build[k] = '\0';
-						k--;
-					}
-					else
-						break;
-				}
-				test = strchr(build, ' ');
-				if (test)
-				{
-					*test = 0;
-					strcpy(mapname, build);
-					// in case someone uses more than one space
+				if (*test == '\t')
 					*test = ' ';
-					while (*test == ' ')
-						test++;
-					strcpy(progname, test);
-					if (q_strcasecmp(mapname, sv.name) == 0)
-					{
-						strcpy(finalprogname, progname);
-						break;
-					}
-				}
+				++test;
+			}
+			while (--test > &build[0])
+			{
+				if (*test == ' ')
+					*test = '\0';
+				else
+					break;
+			}
+			if (!(test = strchr(build, ' ')))
+				continue;
+			*test = 0;
+			if (q_strcasecmp(build, sv.name) == 0)
+			{
+				FS_fclose (&FH);
+				while (*(++test) == ' ')
+					;
+				q_strlcpy(finalprogname, test, sizeof(finalprogname));
+				return finalprogname;
 			}
 		}
-		fclose (FH);
 	}
-
-	return finalprogname;
+_fail:
+	FS_fclose (&FH);
+	return def_progname;
 #endif	/* end of USE_MULTIPLE_PROGS */
 }
 
