@@ -1,6 +1,6 @@
 /*
  * pr_exec.c -- PROGS execution
- * $Id: pr_exec.c 5041 2013-01-11 10:05:22Z sezero $
+ * $Id: pr_exec.c 5244 2013-09-25 17:01:22Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
@@ -25,7 +25,7 @@
 // HEADER FILES ------------------------------------------------------------
 
 #include "quakedef.h"
-#include <ctype.h>
+#include "q_ctype.h"
 
 // MACROS ------------------------------------------------------------------
 
@@ -1039,7 +1039,7 @@ void PR_Profile_f (void)
 		}
 		else if (*s == 's' || *s == 'S')
 		{ // Save to file
-			if (i + 1 < Cmd_Argc() && !isdigit(*Cmd_Argv(i + 1)))
+			if (i + 1 < Cmd_Argc() && !q_isdigit(*Cmd_Argv(i + 1)))
 			{
 				i++;
 				saveName = FS_MakePath(FS_USERDIR, NULL, Cmd_Argv(i));
@@ -1049,7 +1049,7 @@ void PR_Profile_f (void)
 				saveName = FS_MakePath(FS_USERDIR, NULL, "profile.txt");
 			}
 		}
-		else if (isdigit(*s))
+		else if (q_isdigit(*s))
 		{ // Specify function count
 			funcCount = atoi(Cmd_Argv(i));
 			if (funcCount < 1)
