@@ -1,6 +1,6 @@
 /*
  * sv_ccmds.c -- console commands
- * $Id: sv_ccmds.c 4828 2012-07-01 14:00:25Z sezero $
+ * $Id: sv_ccmds.c 5404 2015-04-01 09:47:28Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
@@ -62,13 +62,13 @@ static void SV_SetMaster_f (void)
 		if (master_adr[i-1].port == 0)
 			master_adr[i-1].port = BigShort (PORT_MASTER);
 
-		Con_Printf ("Master server at %s\n", NET_AdrToString (master_adr[i-1]));
+		Con_Printf ("Master server at %s\n", NET_AdrToString (&master_adr[i-1]));
 
 		Con_Printf ("Sending a ping.\n");
 
 		data[0] = A2A_PING;
 		data[1] = 0;
-		NET_SendPacket (2, data, master_adr[i-1]);
+		NET_SendPacket (2, data, &master_adr[i-1]);
 	}
 
 	svs.last_heartbeat = -99999;
@@ -307,7 +307,6 @@ static void SV_Map_f (void)
 {
 	char	level[MAX_QPATH];
 	char	expanded[MAX_QPATH];
-	FILE	*f;
 	char	_startspot[MAX_QPATH];
 	char	*startspot;
 
@@ -321,6 +320,7 @@ static void SV_Map_f (void)
 		}
 		return;
 	}
+
 	q_strlcpy (level, Cmd_Argv(1), sizeof(level));
 	if (Cmd_Argc() == 2)
 	{
@@ -334,13 +334,11 @@ static void SV_Map_f (void)
 
 	// check to make sure the level exists
 	q_snprintf (expanded, sizeof(expanded), "maps/%s.bsp", level);
-	FS_OpenFile (expanded, &f, NULL);
-	if (!f)
+	if (!FS_FileExists(expanded, NULL))
 	{
 		Con_Printf ("Can't find %s\n", expanded);
 		return;
 	}
-	fclose (f);
 
 	SV_BroadcastCommand ("changing\n");
 	SV_SendMessagesToAll ();
@@ -456,7 +454,7 @@ static void SV_Status_f (void)
 	avg = 1000 * svs.stats.latched_active / STATFRAMES;
 	pak = (float)svs.stats.latched_packets/ STATFRAMES;
 
-	Con_Printf ("net address      : %s\n",NET_AdrToString (net_local_adr));
+	Con_Printf ("net address      : %s\n",NET_AdrToString (&net_local_adr));
 	Con_Printf ("cpu utilization  : %3i%%\n",(int)cpu);
 	Con_Printf ("avg response time: %i ms\n",(int)avg);
 	Con_Printf ("packets/frame    : %5.2f\n", pak);
@@ -501,7 +499,7 @@ static void SV_Status_f (void)
 			else
 				Con_Printf("\n");
 
-			s = NET_BaseAdrToString ( cl->netchan.remote_address);
+			s = NET_BaseAdrToString (&cl->netchan.remote_address);
 			Con_Printf ("  %-16.16s", s);
 			if (cl->state == cs_connected)
 			{
@@ -529,7 +527,7 @@ static void SV_Status_f (void)
 				continue;
 			Con_Printf ("%5i %6i ", (int)cl->edict->v.frags,  cl->userid);
 
-			s = NET_BaseAdrToString ( cl->netchan.remote_address);
+			s = NET_BaseAdrToString (&cl->netchan.remote_address);
 			Con_Printf ("%s", s);
 			l = 16 - strlen(s);
 			for (j = 0; j < l; j++)
