@@ -1,6 +1,6 @@
 /*
  * sv_send.c -- server communication module
- * $Id: sv_send.c 4767 2012-06-16 20:48:51Z sezero $
+ * $Id: sv_send.c 5404 2015-04-01 09:47:28Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 1997-1998  Raven Software Corp.
@@ -69,7 +69,7 @@ static void SV_FlushRedirect (void)
 		senddata[4] = A2C_PRINT;
 		memcpy (&senddata[5], outputbuf, strlen(outputbuf)+1);
 
-		NET_SendPacket (strlen(senddata)+1, senddata, net_from);
+		NET_SendPacket (strlen(senddata)+1, senddata, &net_from);
 	}
 	else if (sv_redirected == RD_CLIENT)
 	{
