@@ -1,7 +1,7 @@
 /*
  * sys.h: non-portable functions
  * relies on: arch_def.h
- * $Id: sys.h 4767 2012-06-16 20:48:51Z sezero $
+ * $Id: sys.h 5310 2014-07-01 17:05:25Z sezero $
  *
  * Copyright (C) 1996-1997  Id Software, Inc.
  * Copyright (C) 2005-2012  O.Sezer <sezero@users.sourceforge.net>
@@ -31,14 +31,6 @@ double Sys_DoubleTime (void);
 
 char *Sys_ConsoleInput (void);
 
-/* disable use of password file on platforms where they
-   are not necessary or not possible. */
-#if defined(PLATFORM_DOS) || defined(PLATFORM_AMIGA) || \
-    defined(PLATFORM_WINDOWS)
-#undef	USE_PASSWORD_FILE
-#define	USE_PASSWORD_FILE	0
-#endif	/* _PASSWORD_FILE */
-
 /* disable user directories on platforms where they
  * aren't necessary or not possible. */
 #if defined(PLATFORM_DOS) || defined(PLATFORM_AMIGA) || \
