@@ -18,15 +18,15 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <stdlib.h>
-#include <string.h>
-#include <math.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#ifndef WIN32_LEAN_AND_MEAN
+#define WIN32_LEAN_AND_MEAN
+#endif
 
-#include "timidity.h"
 #include "timidity_internal.h"
-#include "instrum.h"
-#include "tables.h"
-#include "common.h"
 
 #if !defined(TIMIDITY_USE_DLS) /* stubs */
 MidDLSPatches *mid_dlspatches_load (MidIStream *stream)
@@ -39,9 +39,28 @@ void mid_dlspatches_free (MidDLSPatches *data)
 }
 
 #else /* DLS support: */
+
+#ifdef _WIN32
+#include <windows.h>
+#ifndef DEFINE_GUID
+#include <basetyps.h> /* guiddef.h not in all SDKs -- e.g. mingw.org */
+#endif
+#include <mmsystem.h>
+#else /* not-windows */
+#include "dls_compat.h"
+#endif
+#include "dls1.h"
+#include "dls2.h"
+#include <string.h>
+#include <math.h>
+
 #include "instrum_dls.h"
+#include "instrum.h"
+#include "tables.h"
+#include "common.h"
 
 /* ------- load_riff.h ------- */
+
 typedef struct _RIFF_Chunk {
     uint32 magic;
     uint32 length;
@@ -51,16 +70,17 @@ typedef struct _RIFF_Chunk {
     struct _RIFF_Chunk *next;
 } RIFF_Chunk;
 
-static void LoadRIFF(MidIStream *stream, RIFF_Chunk **chunk);
-static void FreeRIFF(RIFF_Chunk *chunk);
-
-/* ------- load_riff.c ------- */
 #define RIFF	0x46464952	/* "RIFF" */
 #define LIST	0x5453494c	/* "LIST" */
 
+/* ------- load_riff.c ------- */
+
+static void LoadRIFF(MidIStream *stream, RIFF_Chunk **chunk);
+static void FreeRIFF(RIFF_Chunk *chunk);
+
 static void AllocRIFFChunk(RIFF_Chunk **chunk)
 {
-    *chunk = (RIFF_Chunk *)safe_malloc(sizeof(RIFF_Chunk));
+    *chunk = (RIFF_Chunk *)timi_calloc(sizeof(RIFF_Chunk));
 }
 
 static void FreeRIFFChunk(RIFF_Chunk *chunk)
@@ -69,7 +89,7 @@ static void FreeRIFFChunk(RIFF_Chunk *chunk)
 	FreeRIFFChunk(chunk->child);
     if (chunk->next)
 	FreeRIFFChunk(chunk->next);
-    free(chunk);
+    timi_free(chunk);
 }
 
 static int ChunkHasSubType(uint32 magic)
@@ -98,7 +118,7 @@ static int ChunkHasSubChunks(uint32 magic)
     return 0;
 }
 
-static void LoadSubChunks(RIFF_Chunk *chunk, uint8 *data, uint32 left)
+static int LoadSubChunks(RIFF_Chunk *chunk, uint8 *data, uint32 left)
 {
     uint8 *subchunkData;
     uint32 subchunkDataLen;
@@ -107,6 +127,7 @@ static void LoadSubChunks(RIFF_Chunk *chunk, uint8 *data, uint32 left)
 	RIFF_Chunk *child;
 	RIFF_Chunk *next, *prev = NULL;
 	AllocRIFFChunk(&child);
+	if (!child) return -1;
 	for (next = chunk->child; next; next = next->next)
 	    prev = next;
 	if (prev)  prev->next = child;
@@ -140,11 +161,13 @@ static void LoadSubChunks(RIFF_Chunk *chunk, uint8 *data, uint32 left)
 	    subchunkDataLen -= 4;
         }
 	if (ChunkHasSubChunks(child->magic))
-	    LoadSubChunks(child, subchunkData, subchunkDataLen);
+	    if (LoadSubChunks(child, subchunkData, subchunkDataLen) < 0)
+		return -1;
 
 	data += child->length;
 	left -= child->length;
     }
+    return 0;
 }
 
 static void LoadRIFF(MidIStream *stream, RIFF_Chunk **out)
@@ -156,7 +179,7 @@ static void LoadRIFF(MidIStream *stream, RIFF_Chunk **out)
 
     /* Allocate the chunk structure */
     AllocRIFFChunk(out);
-    chunk = *out;
+    if (!(chunk = *out)) return;
 
     /* Make sure the file is in RIFF format */
     mid_istream_read(stream, &tmpUint32, sizeof(uint32), 1);
@@ -169,7 +192,13 @@ static void LoadRIFF(MidIStream *stream, RIFF_Chunk **out)
 	*out = NULL;
 	return;
     }
-    chunk->data = (uint8 *)safe_malloc(chunk->length);
+    chunk->data = (uint8 *)timi_calloc(chunk->length);
+    if (!chunk->data) {
+	DEBUG_MSG("Out of memory\n");
+	FreeRIFF(chunk);
+	*out = NULL;
+	return;
+    }
     if (mid_istream_read(stream, chunk->data, chunk->length, 1) != 1) {
 	DEBUG_MSG("IO error\n");
 	FreeRIFF(chunk);
@@ -187,12 +216,15 @@ static void LoadRIFF(MidIStream *stream, RIFF_Chunk **out)
 	subchunkDataLen -= 4;
     }
     if (ChunkHasSubChunks(chunk->magic))
-	LoadSubChunks(chunk, subchunkData, subchunkDataLen);
+	if (LoadSubChunks(chunk, subchunkData, subchunkDataLen) < 0) {
+	    FreeRIFF(chunk);
+	    *out = NULL;
+	}
 }
 
 static void FreeRIFF(RIFF_Chunk *chunk)
 {
-    free(chunk->data);
+    timi_free(chunk->data);
     FreeRIFFChunk(chunk);
 }
 
@@ -201,44 +233,8 @@ static void FreeRIFF(RIFF_Chunk *chunk)
 /* This code is based on the DLS spec version 1.1, available at:
  * http://www.midi.org/about-midi/dls/dlsspec.shtml */
 
-#if defined(_WIN32)	/* use windows native headers */
-#ifndef WIN32_LEAN_AND_MEAN
-#define WIN32_LEAN_AND_MEAN
-#endif
-#include <windows.h>
-#ifndef DEFINE_GUID
-#include <basetyps.h>	/* guiddef.h not in all SDKs, e.g. mingw.org */
-#endif
-#include <mmsystem.h>
-#else
-/* Some typedefs so the public dls headers don't need to be modified */
-#define FAR
-typedef uint8	BYTE;
-typedef sint16	SHORT;
-typedef uint16	USHORT;
-typedef uint16	WORD;
-typedef sint32	LONG;
-typedef uint32	ULONG;
-typedef uint32	DWORD;
-#define mmioFOURCC(A, B, C, D)					\
-		(((A) <<  0) | ((B) <<  8) | ((C) << 16) | ((D) << 24))
-#define DEFINE_GUID(A, B, C, E, F, G, H, I, J, K, L, M)
-#endif
-
-#include "dls1.h"
-#include "dls2.h"
-
-typedef struct _WaveFMT {
-    WORD wFormatTag;
-    WORD wChannels;
-    DWORD dwSamplesPerSec;
-    DWORD dwAvgBytesPerSec;
-    WORD wBlockAlign;
-    WORD wBitsPerSample;
-} WaveFMT;
-
 typedef struct _DLS_Wave {
-    WaveFMT *format;
+    PCMWAVEFORMAT *format;
     uint8 *data;
     uint32 length;
     WSMPL *wsmp;
@@ -278,8 +274,6 @@ struct _MidDLSPatches {
     const char *comments;
 };
 
-/* ------- load_dls.c ------- */
-
 #ifndef FOURCC_LIST /* in mmsystem.h for windows. */
 #define FOURCC_LIST	0x5453494c	/* "LIST" */
 #endif
@@ -304,11 +298,12 @@ struct _MidDLSPatches {
 #define FOURCC_ISRF	mmioFOURCC('I','S','R','F')
 #define FOURCC_ITCH	mmioFOURCC('I','T','C','H')
 
+/* ------- load_dls.c ------- */
 
 static void FreeRegions(DLS_Instrument *instrument)
 {
     if (instrument->regions) {
-	free(instrument->regions);
+	timi_free(instrument->regions);
 	instrument->regions = NULL;
     }
 }
@@ -317,7 +312,7 @@ static void AllocRegions(DLS_Instrument *instrument)
 {
     size_t datalen = (instrument->header->cRegions * sizeof(DLS_Region));
     FreeRegions(instrument);
-    instrument->regions = (DLS_Region *)safe_malloc(datalen);
+    instrument->regions = (DLS_Region *)timi_calloc(datalen);
 }
 
 static void FreeInstruments(MidDLSPatches *data)
@@ -326,7 +321,7 @@ static void FreeInstruments(MidDLSPatches *data)
 	uint32 i;
 	for (i = 0; i < data->cInstruments; ++i)
 	    FreeRegions(&data->instruments[i]);
-	free(data->instruments);
+	timi_free(data->instruments);
 	data->instruments = NULL;
     }
 }
@@ -335,13 +330,13 @@ static void AllocInstruments(MidDLSPatches *data)
 {
     int datalen = (data->cInstruments * sizeof(DLS_Instrument));
     FreeInstruments(data);
-    data->instruments = (DLS_Instrument *)safe_malloc(datalen);
+    data->instruments = (DLS_Instrument *)timi_calloc(datalen);
 }
 
 static void FreeWaveList(MidDLSPatches *data)
 {
     if (data->waveList) {
-	free(data->waveList);
+	timi_free(data->waveList);
 	data->waveList = NULL;
     }
 }
@@ -350,16 +345,18 @@ static void AllocWaveList(MidDLSPatches *data)
 {
     int datalen = (data->ptbl->cCues * sizeof(DLS_Wave));
     FreeWaveList(data);
-    data->waveList = (DLS_Wave *)safe_malloc(datalen);
+    data->waveList = (DLS_Wave *)timi_calloc(datalen);
 }
 
-static void Parse_colh(MidDLSPatches *data, RIFF_Chunk *chunk)
+static int Parse_colh(MidDLSPatches *data, RIFF_Chunk *chunk)
 {
     data->cInstruments = SWAPLE32(*(uint32 *)chunk->data);
     AllocInstruments(data);
+    if (data->instruments) return 0;
+    return -1;
 }
 
-static void Parse_insh(MidDLSPatches *data, RIFF_Chunk *chunk, DLS_Instrument *instrument)
+static int Parse_insh(MidDLSPatches *data, RIFF_Chunk *chunk, DLS_Instrument *instrument)
 {
     INSTHEADER *header = (INSTHEADER *)chunk->data;
     header->cRegions = SWAPLE32(header->cRegions);
@@ -367,6 +364,8 @@ static void Parse_insh(MidDLSPatches *data, RIFF_Chunk *chunk, DLS_Instrument *i
     header->Locale.ulInstrument = SWAPLE32(header->Locale.ulInstrument);
     instrument->header = header;
     AllocRegions(instrument);
+    if (instrument->regions) return 0;
+    return -1;
 }
 
 static void Parse_rgnh(MidDLSPatches *data, RIFF_Chunk *chunk, DLS_Region *region)
@@ -497,13 +496,13 @@ static void Parse_INFO_INS(MidDLSPatches *data, RIFF_Chunk *chunk, DLS_Instrumen
     }
 }
 
-static void Parse_ins(MidDLSPatches *data, RIFF_Chunk *chunk, DLS_Instrument *instrument)
+static int Parse_ins(MidDLSPatches *data, RIFF_Chunk *chunk, DLS_Instrument *instrument)
 {
     for (chunk = chunk->child; chunk; chunk = chunk->next) {
 	uint32 magic = (chunk->magic == FOURCC_LIST) ? chunk->subtype : chunk->magic;
 	switch(magic) {
 	case FOURCC_INSH:
-	    Parse_insh(data, chunk, instrument);
+	    if (Parse_insh(data, chunk, instrument) < 0) return -1;
 	    break;
 	case FOURCC_LRGN:
 	    Parse_lrgn(data, chunk, instrument);
@@ -517,9 +516,10 @@ static void Parse_ins(MidDLSPatches *data, RIFF_Chunk *chunk, DLS_Instrument *in
 	    break;
 	}
     }
+    return 0;
 }
 
-static void Parse_lins(MidDLSPatches *data, RIFF_Chunk *chunk)
+static int Parse_lins(MidDLSPatches *data, RIFF_Chunk *chunk)
 {
     uint32 instrument = 0;
     for (chunk = chunk->child; chunk; chunk = chunk->next) {
@@ -527,13 +527,15 @@ static void Parse_lins(MidDLSPatches *data, RIFF_Chunk *chunk)
 	switch(magic) {
 	case FOURCC_INS:
 	    if (instrument < data->cInstruments)
-		Parse_ins(data, chunk, &data->instruments[instrument++]);
+		if (Parse_ins(data, chunk, &data->instruments[instrument++]) < 0)
+		    return -1;
 	    break;
 	}
     }
+    return 0;
 }
 
-static void Parse_ptbl(MidDLSPatches *data, RIFF_Chunk *chunk)
+static int Parse_ptbl(MidDLSPatches *data, RIFF_Chunk *chunk)
 {
     uint32 i;
     POOLTABLE *ptbl = (POOLTABLE *)chunk->data;
@@ -544,16 +546,18 @@ static void Parse_ptbl(MidDLSPatches *data, RIFF_Chunk *chunk)
     for (i = 0; i < ptbl->cCues; ++i)
 	data->ptblList[i].ulOffset = SWAPLE32(data->ptblList[i].ulOffset);
     AllocWaveList(data);
+    if (data->waveList) return 0;
+    return -1;
 }
 
 static void Parse_fmt(MidDLSPatches *data, RIFF_Chunk *chunk, DLS_Wave *wave)
 {
-    WaveFMT *fmt = (WaveFMT *)chunk->data;
-    fmt->wFormatTag = SWAPLE16(fmt->wFormatTag);
-    fmt->wChannels = SWAPLE16(fmt->wChannels);
-    fmt->dwSamplesPerSec = SWAPLE32(fmt->dwSamplesPerSec);
-    fmt->dwAvgBytesPerSec = SWAPLE32(fmt->dwAvgBytesPerSec);
-    fmt->wBlockAlign = SWAPLE16(fmt->wBlockAlign);
+    PCMWAVEFORMAT *fmt = (PCMWAVEFORMAT *)chunk->data;
+    fmt->wf.wFormatTag = SWAPLE16(fmt->wf.wFormatTag);
+    fmt->wf.nChannels = SWAPLE16(fmt->wf.nChannels);
+    fmt->wf.nSamplesPerSec = SWAPLE32(fmt->wf.nSamplesPerSec);
+    fmt->wf.nAvgBytesPerSec = SWAPLE32(fmt->wf.nAvgBytesPerSec);
+    fmt->wf.nBlockAlign = SWAPLE16(fmt->wf.nBlockAlign);
     fmt->wBitsPerSample = SWAPLE16(fmt->wBitsPerSample);
     wave->format = fmt;
 }
@@ -643,22 +647,17 @@ static void Parse_INFO_DLS(MidDLSPatches *data, RIFF_Chunk *chunk)
     }
 }
 
+
+/* ------- instrum_dls.c ------- */
+
 static void do_dlspatches_load(MidIStream *stream, MidDLSPatches **out)
 {
     RIFF_Chunk *chunk;
     MidDLSPatches *data;
 
-    *out = NULL;
-    if (setjmp(malloc_env)) {
-	DEBUG_MSG("Out of memory\n");
-	fail:
-	mid_dlspatches_free(*out);
-	*out = NULL;
-	return;
-    }
-
-    *out = (MidDLSPatches *)safe_malloc(sizeof(MidDLSPatches));
-    data = *out;
+    *out = (MidDLSPatches *)timi_calloc(sizeof(MidDLSPatches));
+    if (!(data = *out))
+	goto fail;
     LoadRIFF(stream, &data->chunk);
     if (!data->chunk)
 	goto fail;
@@ -667,13 +666,13 @@ static void do_dlspatches_load(MidIStream *stream, MidDLSPatches **out)
 	uint32 magic = (chunk->magic == FOURCC_LIST) ? chunk->subtype : chunk->magic;
 	switch(magic) {
 	case FOURCC_COLH:
-	    Parse_colh(data, chunk);
+	    if (Parse_colh(data, chunk) < 0) goto fail;
 	    break;
 	case FOURCC_LINS:
-	    Parse_lins(data, chunk);
+	    if (Parse_lins(data, chunk) < 0) goto fail;
 	    break;
 	case FOURCC_PTBL:
-	    Parse_ptbl(data, chunk);
+	    if (Parse_ptbl(data, chunk) < 0) goto fail;
 	    break;
 	case FOURCC_WVPL:
 	    Parse_wvpl(data, chunk);
@@ -683,6 +682,10 @@ static void do_dlspatches_load(MidIStream *stream, MidDLSPatches **out)
 	    break;
 	}
     }
+    return;
+fail:
+    mid_dlspatches_free(*out);
+    *out = NULL;
 }
 
 MidDLSPatches *mid_dlspatches_load(MidIStream *stream)
@@ -694,18 +697,14 @@ MidDLSPatches *mid_dlspatches_load(MidIStream *stream)
 
 void mid_dlspatches_free(MidDLSPatches *data)
 {
-    if (!data )
-	return;
+    if (!data) return;
     if (data->chunk)
 	FreeRIFF(data->chunk);
     FreeInstruments(data);
     FreeWaveList(data);
-    free(data);
+    timi_free(data);
 }
 
-
-/* ------- instrum_dls.c ------- */
-
 /* convert timecents to sec */
 static double to_msec(int timecent)
 {
@@ -789,13 +788,18 @@ static void load_region_dls(MidSong *song, MidSample *sample, DLS_Instrument *in
   sample->high_vel = rgn->header->RangeVelocity.usHigh;
 
   sample->modes = MODES_16BIT;
-  sample->sample_rate = wave->format->dwSamplesPerSec;
+  sample->sample_rate = wave->format->wf.nSamplesPerSec;
   sample->data_length = wave->length / 2;
-  sample->data = (sample_t *)safe_malloc(wave->length + 2);
+  sample->data = (sample_t *)timi_calloc(wave->length + 4);
+  if (!sample->data) {
+    song->oom = 1;
+    return;
+  }
   memcpy(sample->data, wave->data, wave->length);
-  /* initialize the added extra sample space (see the +2 bytes in
-     allocation) using the last actual sample:  */
-  sample->data[sample->data_length] = sample->data[sample->data_length-1];
+  /* initialize the 2 extra samples at the end (those +4 bytes) */
+#if 0 /* no need - alloc'ed using timi_calloc() */
+  sample->data[sample->data_length] = sample->data[sample->data_length+1] = 0;
+#endif
   if (rgn->wsmp->cSampleLoops) {
     sample->modes |= (MODES_LOOPING|MODES_SUSTAIN);
     sample->loop_start = rgn->wsmp_loop->ulStart / 2;
@@ -865,6 +869,21 @@ static void load_region_dls(MidSong *song, MidSample *sample, DLS_Instrument *in
   sample->loop_end <<= FRACTION_BITS;
 }
 
+static void free_mid_instrument(MidInstrument *ip)
+{
+  MidSample *sp;
+  int i;
+  if (!ip) return;
+  if (ip->sample) {
+    for (i=0; i<ip->samples; i++) {
+      sp=&(ip->sample[i]);
+      timi_free(sp->data);
+    }
+    timi_free(ip->sample);
+  }
+  timi_free(ip);
+}
+
 void load_instrument_dls(MidSong *song, MidInstrument **out,
 			 int drum, int bank, int instrument)
 {
@@ -873,8 +892,7 @@ void load_instrument_dls(MidSong *song, MidInstrument **out,
   DLS_Instrument *dls_ins = NULL;
 
   *out = NULL;
-  if (!song->dlspatches)
-    return;
+  if (!song->dlspatches) return;
 
 #if 0
   drum = drum ? 0x80000000 : 0;
@@ -898,16 +916,21 @@ void load_instrument_dls(MidSong *song, MidInstrument **out,
     return;
   }
 
-  *out = (MidInstrument *)safe_malloc(sizeof(MidInstrument));
-  inst = *out;
+  inst = (MidInstrument *)timi_calloc(sizeof(MidInstrument));
+  if (!inst) goto oom1;
   inst->samples = dls_ins->header->cRegions;
-  inst->sample = (MidSample *)safe_malloc(sizeof(MidSample) * inst->samples);
+  inst->sample = (MidSample *)timi_calloc(sizeof(MidSample) * inst->samples);
+  if (! inst->sample) goto oom1;
   /*
   printf("Found %s instrument %d in bank %d named %s with %d regions\n",
 	 drum ? "drum" : "melodic", instrument, bank, dls_ins->name, inst->samples);
   */
-  for (i = 0; i < dls_ins->header->cRegions; ++i)
+  for (i = 0; i < dls_ins->header->cRegions; ++i) {
     load_region_dls(song, &inst->sample[i], dls_ins, i);
+    if (song->oom) goto oom1;
+  }
+  *out = inst;
+  return;
 
 #else /* fixed drum loading code from Vavoom svn repository rev. 4175 */
   if (drum) goto _dodrum;
@@ -932,12 +955,16 @@ void load_instrument_dls(MidSong *song, MidInstrument **out,
     return;
   }
 
-  *out = (MidInstrument *)safe_malloc(sizeof(MidInstrument));
-  inst = *out;
+  inst = (MidInstrument *)timi_calloc(sizeof(MidInstrument));
+  if (!inst) goto oom1;
   inst->samples = dls_ins->header->cRegions;
-  inst->sample = (MidSample *)safe_malloc(sizeof(MidSample) * inst->samples);
-  for (i = 0; i < dls_ins->header->cRegions; ++i)
+  inst->sample = (MidSample *)timi_calloc(sizeof(MidSample) * inst->samples);
+  if (! inst->sample) goto oom1;
+  for (i = 0; i < dls_ins->header->cRegions; ++i) {
     load_region_dls(song, &inst->sample[i], dls_ins, i);
+    if (song->oom) goto oom1;
+  }
+  *out = inst;
   return;
 
 _dodrum:
@@ -971,12 +998,19 @@ _dodrum:
     return;
   }
 
-  *out = (MidInstrument *)safe_malloc(sizeof(MidInstrument));
-  inst = *out;
+  inst = (MidInstrument *)timi_calloc(sizeof(MidInstrument));
+  if (!inst) goto oom1;
   inst->samples = 1;
-  inst->sample = (MidSample *)safe_malloc(sizeof(MidSample));
-  load_region_dls(song, &inst->sample[0], dls_ins, drum);
+  inst->sample = (MidSample *)timi_calloc(sizeof(MidSample));
+  if (! inst->sample) goto oom1;
+  load_region_dls(song, inst->sample, dls_ins, drum);
+  if (song->oom) goto oom1;
+  *out = inst;
+  return;
 #endif /* fix from Vavoom */
+oom1:
+  song->oom = 1;
+  free_mid_instrument(inst);
 }
 
 #endif /* TIMIDITY_USE_DLS */
