@@ -23,11 +23,9 @@
 #ifndef TIMIDITY_INSTRUM_DLS_H
 #define TIMIDITY_INSTRUM_DLS_H
 
+#define load_instrument_dls TIMI_NAMESPACE(load_instrument_dls)
+
 extern void load_instrument_dls(MidSong *song, MidInstrument **out,
 				int drum, int bank, int instrument);
 
-#if !defined(TIMIDITY_USE_DLS)
-#define load_instrument_dls(S, OUTINST, D, B, I)  *(OUTINST)=NULL
-#endif
-
 #endif /* TIMIDITY_INSTRUM_DLS_H */
