@@ -20,15 +20,14 @@
  * playmidi.c -- random stuff in need of rearrangement
  */
 
-#if HAVE_CONFIG_H
-#  include <config.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
 #endif
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "timidity.h"
 #include "timidity_internal.h"
 #include "instrum.h"
 #include "playmidi.h"
@@ -89,26 +88,28 @@ static void select_sample(MidSong *song, int v, MidInstrument *ip, int vel)
     }
 
   f=song->voice[v].orig_frequency;
-  for (i=0; i<s; i++)
+  for (i=0; i<s; i++, sp++)
     {
-      if (sp->low_vel <= vel && sp->high_vel >= vel &&
-          sp->low_freq <= f && sp->high_freq >= f)
+#ifdef TIMIDITY_USE_DLS
+      if (sp->low_vel > vel || sp->high_vel < vel)
+	continue;
+#endif
+      if (sp->low_freq <= f && sp->high_freq >= f)
 	{
 	  song->voice[v].sample=sp;
 	  return;
 	}
-      sp++;
     }
 
   /*
      No suitable sample found! We'll select the sample whose root
      frequency is closest to the one we want. (Actually we should
-     probably convert the low, high, and root frequencies to MIDI note
-     values and compare those.) */
-
+     probably convert the low, high, and root frequencies to MIDI
+     note values and compare those.)
+   */
   cdiff=0x7FFFFFFF;
   closest=sp=ip->sample;
-  for(i=0; i<s; i++)
+  for(i=0; i<s; i++, sp++)
     {
       diff=sp->root_freq - f;
       if (diff<0) diff=-diff;
@@ -117,10 +118,8 @@ static void select_sample(MidSong *song, int v, MidInstrument *ip, int vel)
 	  cdiff=diff;
 	  closest=sp;
 	}
-      sp++;
     }
   song->voice[v].sample=closest;
-  return;
 }
 
 static void recompute_freq(MidSong *song, int v)
