@@ -206,18 +206,20 @@ static const char *DigitDefs[] =
 //
 //==========================================================================
 
+__attribute__((__noreturn__)) static void usage (void) {
+	printf("usage: genmodel [-opt] [-archive path] model.hc\n");
+	exit(1);
+}
+
 int main(int argc, char **argv)
 {
 	int		i, j;
 	char	path[1024], bakname[1024];
 
-	printf("GENMODEL Version "VERSION_TEXT" ("__DATE__")\n\n");
+	printf("GENMODEL Version " VERSION_TEXT " (" __DATE__ ")\n\n");
 
-	if (argc == 1) {
-	_usage:
-		printf("usage: genmodel [-opt] [-archive path] model.hc\n");
-		exit (1);
-	}
+	if (argc == 1)
+		usage ();
 
 	ValidateByteorder ();
 
@@ -229,7 +231,7 @@ int main(int argc, char **argv)
 	i = CheckParm("-archive");
 	if (i != 0)
 	{
-		if (i >= argc - 1) goto _usage;
+		if (i >= argc - 1) usage ();
 		archive = true;
 		q_strlcpy(archivedir, argv[++i], sizeof(archivedir));
 		printf("Archiving source to: %s\n", archivedir);
@@ -239,7 +241,7 @@ int main(int argc, char **argv)
 		DoOpts = true;
 	if (i < j) i = j;
 	if (i + 1 > argc - 1)
-		goto _usage;
+		usage ();
 
 	q_strlcpy(path, argv[i + 1], sizeof(path));
 	SetQdirFromPath("");		/* (path); */
@@ -664,6 +666,7 @@ static void OptimizeVertices(void)
 	printf("Optimizing vertices...");
 
 	memset(vert_used, 0, sizeof(vert_used));
+	memset(vert_replacement, 0, sizeof(vert_replacement)); // make static analyzers happy
 	num_unique = 0;
 
 	// search for common points among all the frames
@@ -699,6 +702,7 @@ static void OptimizeVertices(void)
 		}
 	}
 
+#if 0 /* this is a dead loop because of the commented out code. */
 	// recompute the light normals
 	for (i = 0 ; i < model.numframes ; i++)
 	{
@@ -747,6 +751,7 @@ static void OptimizeVertices(void)
 			in->v[j].lightnormalindex = maxdotindex;
 		}*/
 	}
+#endif
 
 	// create substitution list
 	num_unique = 0;
